﻿using System;
using System.Windows;
using tee3.avd;
using System.Runtime.InteropServices;

namespace AVDTest
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        public static string AvdServerUrl   = "121.41.102.126:8080";
        public static string AvdAppKey      = "demo_access";
        public static string AvdAppSecret   = "demo_secret";

        private IntPtr render1 = IntPtr.Zero;
        private IntPtr render2 = IntPtr.Zero;
        private string roomId = "r1000";

        private string message = "hello 大家好!!";
        private bool isRenderInit;

        public MainWindow()
        {
            InitializeComponent();
            Loaded += MainWindow_Loaded;
            Closed += MainWindow_Closed;
        }
        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            this.IsEnabled = false;

            InitRenders();

            AVDEngine engine = AVDEngine.instance();
            engine.onInitResult += Engine_onInitResult;
            engine.onScheduleRoomResult += Engine_onScheduleRoomResult;
            engine.onGetRoomResult += Engine_onGetRoomResult;
            engine.onCancelRoomResult += Engine_onCancelRoomResult;
            if (0 != engine.initAsync(AvdServerUrl, AvdAppKey, AvdAppSecret))
            {
                MessageBox.Show("初始化AVD引擎失败!");
                Close();
            }
        }
        ///
        private void ButtonScheduleRoom_Click(object sender, RoutedEventArgs e)
        {
            AVDEngine.instance().scheduleRoomAsync("testroom", "", "", "fanghe", "", "1234", "fanghe", 4);
        }
        private void ButtonGetRoom_Click(object sender, RoutedEventArgs e)
        {
            Console.WriteLine("Get Room , Id：{0}", this.roomId);
            AVDEngine.instance().getRoomByRoomIdAsync(this.roomId);
        }
        private void ButtonCancelRoom_Click(object sender, RoutedEventArgs e)
        {
            Console.WriteLine("Cancel Room Id：{0}", this.roomId);
            AVDEngine.instance().cancelRoomAsync(this.roomId);
        }

        private void Engine_onInitResult(int result)
        {
            if (0 == result)
            {
                this.IsEnabled = true;
            }
            Console.WriteLine("Engine_onInitResult：{0}", result);
        }
        private void Engine_onScheduleRoomResult(uint callId, int result, string roomId)
        {
            Console.WriteLine("Engine_onScheduleRoomResult：{0}, roomId：{1}", result, roomId);
            this.roomId = roomId;
        }
        private void Engine_onCancelRoomResult(int result, string roomId)
        {
            Console.WriteLine("Engine_onCancelRoomResult：{0}, Room Id：{1}", result, roomId);

        }
        private void Engine_onGetRoomResult(uint callId, int result, ref RoomInfo room)
        {
            Console.WriteLine("Engine_onGetRoomResult：{0}, Room Name：{1}", result, Utils.GetUtf8String(room.roomName));
        }

        private void MainWindow_Closed(object sender, EventArgs e)
        {
            AVDEngine.instance().uninit();
        }

        // 初始化播放窗体
        private void InitRenders()
        {
            if (!isRenderInit)
            {
                render1 = AVDEngine.createRender(win1.Handle, false, false);
                render2 = AVDEngine.createRender(win2.Handle, false, false);
                isRenderInit = true;
            }
        }
        // 释放播放窗体
        private void DestoryRenders()
        {
            if (isRenderInit)
            {
                AVDEngine.destoryRender(render1);
                AVDEngine.destoryRender(render2);
                isRenderInit = false;
            }
        }

        ///
        private Room room;
        private MUserManager musers;
        private MVideo  mvideo;
        private MAudio  maudio;
        private MChat   mchat;
        private MScreen mscreen;
        private void ButtonJoinRoom_Click(object sender, RoutedEventArgs e)
        {
            if (!AVDEngine.instance().isWorking())
            {
                MessageBox.Show("引擎没有初始化！");
                return;
            }
            room = Room.obtain(this.roomId);
            if (null == room)
            {
                MessageBox.Show("初始化会议失败！");
                return;
            }
            room.onJoinResult += Room_onJoinResult;
            room.joinAsync(generateNewUserId(), generateNewUserName(), "");
        }
        private string generateNewUserId()
        {
           return Guid.NewGuid().ToString();
        }
        private string generateNewUserName()
        {
            Random rd = new Random();
            return "user_" + rd.Next(100).ToString();
        }
        private void ButtonLeaveRoom_Click(object sender, RoutedEventArgs e)
        {
            if (null != room)
            {
                room.leave();
                Room.destoryRoom(room);
                room = null;
                musers = null;
            }
        }
        private void Room_onJoinResult(int result)
        {
            Console.WriteLine("Room_onJoinResult：{0}", result);
            if (0 != result) {
                return ;
            }

            musers = MUserManager.getUserManager(room);
            mvideo = MVideo.getVideo(room);
            maudio = MAudio.getAudio(room);
            mchat  = MChat.getChat(room);
            mscreen = MScreen.getScreen(room);

            mvideo.onPublishCameraNotify += Video_onPublishCameraNotify;
            mvideo.onUnpublishCameraNotify += Video_onUnpublishCameraNotify;
            mvideo.onSubscribeResult += Video_onSubscribeResult;
        }
        // video delegate
        private void Video_onPublishCameraNotify(ref Camera from)
        {
            if (mvideo.isSelfDevice(from.videodevice.device.id))
            {
                Console.WriteLine("是自己的视频，显示在1中");
                mvideo.attachRender(from.videodevice.device.id, render1);
                return;
            }
            mvideo.subscribeAsync(from.videodevice.device.id);
        }
        private void Video_onUnpublishCameraNotify(ref Camera from)
        {
            if (mvideo.isSelfDevice(from.videodevice.device.id))
            {
                mvideo.detachRender(render1);
                return;
            }
            else
            {
                mvideo.detachRender(render2);
                mvideo.unsubscribeAsync(from.videodevice.device.id);
            }
        }
        private void Video_onSubscribeResult(int result, string deviceId)
        {
            if (null == mvideo)
                return;
            mvideo.attachRender(deviceId, render2);
        }

        //
        private void ButtonPreviewLocalCamera_Click(object sender, RoutedEventArgs e)
        {
            if (null == mvideo)
                return;
            mvideo.previewLocalCamera(render1);
        }
        private void ButtonUnpreviewLocalCamera_Click(object sender, RoutedEventArgs e)
        {
            if (null == mvideo)
                return;
            mvideo.unpreviewLocalCamera();
        }

        private void ButtonPublishLocalCamera_Click(object sender, RoutedEventArgs e)
        {
            if (null == mvideo)
                return;
            mvideo.publishLocalCameraAsync();
        }

        private void ButtonUnpublishLocalCamera_Click(object sender, RoutedEventArgs e)
        {
            if (null == mvideo)
                return;
            mvideo.unpublishLocalCameraAsync();
        }

        private void ButtonOpenMicrophone_Click(object sender, RoutedEventArgs e)
        {
            if (null == maudio)
                return;
            maudio.openMicrophoneAsync();
        }

        private void ButtonCloseMicrophone_Click(object sender, RoutedEventArgs e)
        {
            if (null == maudio)
                return;
            maudio.closeMicrophoneAsync();
        }

        private void ButtonSendPublicMessage_Click(object sender, RoutedEventArgs e)
        {
            if (null == mchat)
                return;
            mchat.sendPublicMessage(message);
        }


    }
}
