﻿

#ifndef __BUTTON1_H__
#define __BUTTON1_H__
#include "QtPlatform.h"
#include <QtGui/qimage.h>


#include "WidgetSizer.h"
#include <vector>

using namespace std;

class CButton1 : public QWidget, public CWidgetSizer<CButton1>
{
public:
	Q_OBJECT
public:

	vector<QImage*>		_images;
	int					_image_index;
	
	CButton1(QWidget *_parent, int x, int y, const char* image_file)
		:QWidget(_parent)
	{
		QImage* image = new QImage(image_file);

		_images.push_back(image);

		_image_index = 0;

		int width = image->width();
		int height = image->height();

		setGeometry(QRect(x,y,width,height));

		setMouseTracking(true);
	}

	CButton1(QWidget *_parent, QRect rc)
		:QWidget(_parent)
	{
		_image_index = 0;

		setGeometry(rc);

		setMouseTracking(true);
	}
	
	~CButton1()
	{
		for (int i = 0; i < _images.size(); ++i)
		{
			delete _images[i];
		}
	}

	void clear()
	{
		for (int i = 0; i < _images.size(); ++i)
		{
			delete _images[i];
		}
		_images.clear();
	}

	void add_image(const char* image_file)
	{
		QImage* image = new QImage(image_file);

		_images.push_back(image);
	}

	void set_image_index(int index)
	{
		if (index >= _images.size()) index = _images.size() - 1;
		if (index < 0) index = 0;
		_image_index = index;
	}

	void set_image(QImage* image)
	{
		for (int i = 0; i < _images.size(); ++i)
		{
			delete _images[i];
		}
		_images.clear();
		_images.push_back(image);
		_image_index = 0;
		repaint();
	}

signals:

	void click();

protected:

	void paintEvent(QPaintEvent *_event)
	{
		QPainter painter(this);

		if (_image_index < _images.size())
		{
			QImage* image = _images[_image_index];
			int w = image->width();
			int h = image->height();
			painter.drawImage(0,0,*image,0,0,w,h);
		}
	}

	void leaveEvent(QEvent *event)
	{
	}

	void mousePressEvent(QMouseEvent* event)
	{
		event->accept();
		activateWindow();
        //setFocus(Qt::MouseFocusReason);
	}

	void mouseMoveEvent(QMouseEvent *_event)
	{
		setCursor(Qt::ArrowCursor);
	}
	void mouseReleaseEvent(QMouseEvent* event)
	{
		if (event->button() == Qt::LeftButton)
		{
			repaint();
			if (rect().contains(event->pos()))
			{
				emit click();
			}
		}
		event->accept();
	}

	void mouseDoubleClickEvent(QMouseEvent* event)
	{
		event->accept();
	}

private:

};


#endif


