﻿

#ifndef __BUTTON2_H__
#define __BUTTON2_H__

#include <QtGui/QtGui>
#include <QtGui/qimage.h>
#include "QtPlatform.h"

#include "WidgetSizer.h"

class CButton2 : public QWidget, public CWidgetSizer<CButton2>
{
public:
	Q_OBJECT
public:

	QImage*		_image;
	int			_state;

	CButton2(QWidget *_parent, int x, int y, const char* image_file)
		:QWidget(_parent)
	{
		_image = new QImage(image_file);

		int width = _image->width();
		int height = _image->height();

		setGeometry(QRect(x,y,width / 2,height));

		setMouseTracking(true);
		_state = 0;
	}

	CButton2(QWidget *_parent, int x, int y, const char* image_normal, const char* image_hover)
		:QWidget(_parent)
	{
		// 合并图像
		QImage* image1 = new QImage(image_normal);
		QImage* image2 = new QImage(image_hover);

		_image = new QImage(image1->width()*2, image1->height(), QImage::Format_ARGB32);

		int width = image1->width();
		int height = image1->height();

		{
			QPainter dc(_image);
			dc.drawImage(0,0, *image1);
			dc.drawImage(width,0, *image2);
		}

		delete image1;
		delete image2;

		setGeometry(QRect(x,y,width,height));

		setMouseTracking(true);
		_state = 0;
	}

	virtual ~CButton2()
	{
		if (_image) delete _image;
	}

	void setImage(const char* image_file)
	{
		if (_image) {delete _image; _image=NULL;}
		_image = new QImage(image_file);

		int width = _image->width();
		int height = _image->height();

		setGeometry(QRect(x(),y(),width / 2,height));
		repaint();
	}

signals:
	void mouseIn(bool in);
	void click();

protected:

	virtual void paintEvent(QPaintEvent *_event)
	{
		QPainter painter(this);
		int w = _image->width() / 2;
		int h = _image->height();
		painter.drawImage(0,0,*_image,w*_state,0,w,h);
	}

	virtual void leaveEvent(QEvent *event)
	{
		_state = 0;
		repaint();
		emit mouseIn(false);
	}

	virtual void mousePressEvent(QMouseEvent* event)
	{
		if (event->button() == Qt::LeftButton)
		{
			_state = 1;
			repaint();
		}
		event->accept();
		activateWindow();
        //setFocus(Qt::MouseFocusReason);
	}

	virtual void mouseMoveEvent(QMouseEvent* event)
	{
		if (_state != 1)
		{
			_state = 1;
			repaint();
		}
		event->accept();
		setCursor(Qt::ArrowCursor);
	}

	virtual void mouseReleaseEvent(QMouseEvent *event)
	{
		if (event->button() == Qt::LeftButton)
		{
			_state = 0;
			repaint();
			if (rect().contains(event->pos()))
			{
				emit click();
			}
			event->accept();
		}
	}

	virtual void mouseDoubleClickEvent(QMouseEvent * event)
	{
		event->accept();
    }
	void focusInEvent(QFocusEvent * event)
	{
		_state = 1;
		repaint();
    }
	void focusOutEvent(QFocusEvent *event)
	{
		_state = 0;
		repaint();
	}
	
	void enterEvent(QEvent *event) {
		emit mouseIn(true);
	}
private:

};


#endif


