﻿

#ifndef __BUTTON3_FLASH_H__
#define __BUTTON3_FLASH_H__



#include <QtGui/qimage.h>
#include "QtPlatform.h"

#include "WidgetSizer.h"

class CButton3Flash : public QWidget, public CWidgetSizer<CButton3Flash>
{
public:
	Q_OBJECT
public:

	QImage*		_image;
	int			_state;
	QImage*		_flash_image;
	int			_flash_time_ticket;
	int			_timer_id;
	
	CButton3Flash(QWidget *_parent, int x, int y, const char* image_file)
		:QWidget(_parent)
	{
		_flash_image = NULL;
		_flash_time_ticket = 0;
		_timer_id = 0;

		_image = new QImage(image_file);

		int width = _image->width();
		int height = _image->height();

		setGeometry(QRect(x,y,width / 3,height));

		setMouseTracking(true);
		_state = 0;
	}

	CButton3Flash(QWidget *_parent, int x, int y, const char* image_normal, const char* image_hover, const char* image_click)
		:QWidget(_parent)
	{
		_flash_image = NULL;
		_flash_time_ticket = 0;
		_timer_id = 0;

		// 合并图像
		QImage* image1 = new QImage(image_normal);
		QImage* image2 = new QImage(image_hover);
		QImage* image3 = new QImage(image_click);

		_image = new QImage(image1->width()*3, image1->height(), QImage::Format_ARGB32);

		int width = image1->width();
		int height = image1->height();

		{
			QPainter dc(_image);
			dc.drawImage(0,0, *image1);
			dc.drawImage(width,0, *image2);
			dc.drawImage(width*2,0,*image3);
		}

		delete image1;
		delete image2;
		delete image3;

		setGeometry(QRect(x,y,width,height));

		setMouseTracking(true);
		_state = 0;
	}

	~CButton3Flash()
	{
		if (_image) delete _image;
	}

	void set_flash(const char* image_file)
	{
		if (_flash_image)
		{
			delete _flash_image;
			_flash_image = NULL;
		}
		if (image_file)
		{
			_flash_image = new QImage(image_file);
			_timer_id = startTimer(1000);
		}
		else
		{
			killTimer(_timer_id);
		}
		repaint();
	}

	bool is_flash()
	{
		return _flash_image != NULL;
	}

signals:

	void click();

protected:

	void paintEvent(QPaintEvent *_event)
	{
		QPainter dc(this);

		if (_flash_image)
		{
			int w = _image->width() / 2;
			int h = _image->height();
			int icon_index = _flash_time_ticket % 2;
			dc.drawImage(0,0,*_flash_image,w*icon_index,0,w,h);
		}
		else
		{
			int w = _image->width() / 3;
			int h = _image->height();
			dc.drawImage(0,0,*_image,w*_state,0,w,h);
		}
	}

	void leaveEvent(QEvent *event)
	{
		_state = 0;
		repaint();
	}

	void timerEvent(QTimerEvent * event)
	{
		if (_flash_image)
		{
			_flash_time_ticket++;
			repaint();
		}
	}

	void mousePressEvent(QMouseEvent* event)
	{
		if (event->button() == Qt::LeftButton)
		{
			_state = 2;
			repaint();
		}
		event->accept();
		activateWindow();
        //setFocus(Qt::MouseFocusReason);
	}

	void mouseMoveEvent(QMouseEvent *_event)
	{
		if (_state != 1)
		{
			_state = 1;
			repaint();
		}
		setCursor(Qt::ArrowCursor);
	}

	void mouseReleaseEvent(QMouseEvent* event)
	{
		if (event->button() == Qt::LeftButton)
		{
			_state = 0;
			repaint();
			if (rect().contains(event->pos()))
			{
				emit click();
			}
		}
		event->accept();
	}

	void mouseDoubleClickEvent(QMouseEvent * event)
	{
		event->accept();
	}
private:

};


#endif


