﻿

#ifndef __BUTTON4_H__
#define __BUTTON4_H__


#include <QtGui/qimage.h>
#include "QtPlatform.h"

#include "WidgetSizer.h"

class CButton4 : public QWidget, public CWidgetSizer<CButton4>
{
public:
	Q_OBJECT
public:

	QImage*		_image;
	int			_state;
	
	CButton4(QWidget *_parent, int x, int y, const char* image_file)
		:QWidget(_parent)
	{
		_image = new QImage(image_file);

		int width = _image->width();
		int height = _image->height();

		setGeometry(QRect(x,y,width / 4,height));

		setMouseTracking(true);
		_state = 0;
	}

	CButton4(QWidget *_parent, int x, int y, const char* image_disable, const char* image_normal, const char* image_hover, const char* image_click)
		:QWidget(_parent)
	{
		// 合并图像
		QImage* image1 = new QImage(image_disable);
		QImage* image2 = new QImage(image_normal);
		QImage* image3 = new QImage(image_hover);
		QImage* image4 = new QImage(image_click);

		_image = new QImage(image1->width()*4, image1->height(), QImage::Format_ARGB32);

		int width = image1->width();
		int height = image1->height();

		{
			QPainter dc(_image);
			dc.drawImage(0,0, *image1);
			dc.drawImage(width,0, *image2);
			dc.drawImage(width*2,0,*image3);
			dc.drawImage(width*3,0,*image4);
		}

		delete image1;
		delete image2;
		delete image3;
		delete image4;

		setGeometry(QRect(x,y,width,height));

		setMouseTracking(true);
		_state = 0;
	}
	
	~CButton4()
	{
		if (_image) delete _image;
	}

signals:

	void click();

protected:

	void paintEvent(QPaintEvent *_event)
	{
		QPainter painter(this);
		int w = _image->width() / 4;
		int h = _image->height();

		if (!this->isEnabled())
		{
			painter.drawImage(0,0,*_image,0,0,w,h);
		}
		else
		{
			painter.drawImage(0,0,*_image,w*(_state+1),0,w,h);
		}
	}

	void leaveEvent(QEvent *event)
	{
		_state = 0;
		repaint();
	}

	void mousePressEvent(QMouseEvent* event)
	{
		if (event->button() == Qt::LeftButton)
		{
			_state = 2;
			repaint();
		}
		event->accept();
		activateWindow();
        //setFocus(Qt::MouseFocusReason);
	}

	void mouseMoveEvent(QMouseEvent *_event)
	{
		if (_state != 1)
		{
			_state = 1;
			repaint();
		}
		setCursor(Qt::ArrowCursor);
	}

	void mouseReleaseEvent(QMouseEvent* event)
	{
		if (event->button() == Qt::LeftButton)
		{
			_state = 0;
			repaint();
			if (rect().contains(event->pos()))
			{
				emit click();
			}
		}
		event->accept();
	}

	void mouseDoubleClickEvent(QMouseEvent * event)
	{
		event->accept();
	}
	void focusInEvent(QFocusEvent * event)
	{
		_state = 1;
		repaint();
	}
	void focusOutEvent(QFocusEvent *event)
	{
		_state = 0;
		repaint();
	}
private:

};


#endif


