﻿#include "LineEdit.h"
#include <QtGui/QPainter>
#include <QtGui/qevent.h>
CLineEdit::CLineEdit( int id, QWidget* parent, QString text, QColor color)
	: CRoundRadiusWnd(parent), _text(text), _id(id), _backGroundColor(color), _isMethodInPut(false)
{
	_textEditor = new QLineEditExt(_text, this);
	_textEditor->setContextMenuPolicy(Qt::NoContextMenu);
	connect(_textEditor, SIGNAL(textChanged(const QString)), this, SLOT(ChangeText(const QString)));
	connect(_textEditor, SIGNAL(MethodStringIn(QString, QString)), this, SLOT(HandleMethodStringIn(QString, QString)));
	QPalette palette = _textEditor->palette();
	palette.setColor(QPalette::Base , QColor());
	palette.setColor(QPalette::Text , QColor(0x33,0x33,0x33));
	_textEditor->setPalette(palette);
	_textEditor->setStyleSheet(QString("QLineEdit{border-width:0;border-style:outset;background-color: rgb(%1,%2,%3);color:rgb(0x33,0x33,0x33);}").arg(color.red()).arg(color.green()).arg(color.blue()));
	QFont fontEdit;
#ifdef _MSC_VER
    fontEdit.setFamily("Microsoft YaHei");
#endif
	fontEdit.setPixelSize(12);
	fontEdit.setBold(false);
	_textEditor->setFont(fontEdit);

	_is_show_frame = true;

	//_textEditor->setEchoMode(QLineEdit::Password);
	ResetTextRect(_rect);
}

CLineEdit::~CLineEdit()
{

}

void CLineEdit::EnablePassword(bool is_enable)
{
	if (is_enable)
	{
		_textEditor->setEchoMode(QLineEdit::Password);
	}
	else
	{
		_textEditor->setEchoMode(QLineEdit::Normal);

	}
}

void CLineEdit::SetText(QString text)
{
	_text = text;
	_textEditor->setText(_text);

	_textEditor->setCursorPosition(0);
}

QString CLineEdit::GetText()
{
	_text = _textEditor->text();
	return _text;
}

void CLineEdit::setValidator(const QValidator * v)
{
	_textEditor->setValidator(v);
}

void CLineEdit::paintEvent( QPaintEvent *event )
{
	QPainter dc(this);
	dc.fillRect(_rect, QBrush(_backGroundColor));

	if (_is_show_frame)
	{
		dc.setPen(QColor(92,169, 184));
		DrawBorder(dc);
	}
	if (!_backGroundImgStr.isEmpty())
	{
		QImage img(_backGroundImgStr);
		dc.drawImage(rect(), img, QRect(0, 0, img.width(), img.height()));
	}
	int x = _textEditor->cursorPosition();
	if (_text.isEmpty()&& !_isMethodInPut)
	{
		dc.setPen(QColor(153, 153, 153));
		QRect emptyStrRc(_textRect.left() + 5, _textRect.top() + 4, _textRect.width() - 5, _textRect.height());
		dc.drawText(emptyStrRc, _empty_str);
	}
}

void CLineEdit::resizeEvent( QResizeEvent *event )
{
	CRoundRadiusWnd::resizeEvent(event);
	QRect rect = this->frameGeometry();
	_rect = QRect(0,0,rect.width(),rect.height());
	ResetTextRect(_rect);
	_textEditor->setGeometry(_textRect);
}

void CLineEdit::ResetTextRect(QRect& rc)
{
	int h = 20;
	
	int top = (_rect.height() - h)/2;
	if (top < 0 || top >= _rect.height())
	{
		top = 0;
	}
	_textRect = QRect(TEXT_LEFT_SPACE_VAL, top, _rect.width()-TEXT_LEFT_SPACE_VAL*2, h);
}

void CLineEdit::ChangeText(const QString text)
{
	_text = _textEditor->text();
	emit textChanged();
}

void CLineEdit::SetReadOnly( bool val)
{
	_textEditor->setReadOnly(val);
}

void CLineEdit::SetAlignFlag(int flag)
{
	_textAlignFlag =  flag;
	_textEditor->setAlignment((Qt::Alignment)flag);
}

void CLineEdit::SetMaxLength( unsigned int length )
{
	_textEditor->setMaxLength(length);
}

void CLineEdit::HideFrame()
{
	_is_show_frame = false;
}

QLineEdit* CLineEdit::GetEditObj()
{
	return _textEditor;
}

void CLineEdit::SetEmptyText(const QString emptyStr)
{
	_empty_str = emptyStr;
}

void CLineEdit::HandleMethodStringIn(QString commitString, QString preeditString)
{
	if (!preeditString.isEmpty())
	{
		_isMethodInPut = true;
	}
	else
	{
		_isMethodInPut = false;
	}
	/*if (preeditString.isEmpty()&&commitString.isEmpty()  || preeditString.isEmpty()&&commitString.isEmpty())
	{
	_
	}*/
}

void CLineEdit::SetBackgroundImg(const QString img)
{
	_backGroundImgStr = img;
}

void QLineEditExt::inputMethodEvent(QInputMethodEvent *event)
{
	QLineEdit::inputMethodEvent(event);
	emit MethodStringIn(event->commitString(),event->preeditString());
}


QLineEditExt::QLineEditExt(const QString text, QWidget* parent)
:QLineEdit(text, parent)
{

}

QLineEditExt::~QLineEditExt()
{

}
