﻿#include "ListBase.h"
#include "VScroll.h"
#include "ListItemBase.h"



CListBase::CListBase(QWidget* parent,QColor backGroundColor, int setFlag ) : QWidget(parent)
{
	_scrollAt = CVScroll::InitHoldOn;
	_itemUniquenessId = 0;
	_scrollPos = 0;
	_backGroundColor = backGroundColor;
	_itemAllHeight = 0;
	_itemShowMax = 0;
	_itemWidth = 0;
	_itemHeight = 0;
	_isScrollPosChanged = 0;
	_scrollWidth = SCROLL_WIDTH;
	_emptyStr = QSTRING("暂无数据");
	setMouseTracking(true);
	_vscroll = new CVScroll(this, 10);
	_vscroll->SetBgColor(_backGroundColor, _backGroundColor);
	_vscroll->SetColor(QColor(165,165,165));
	_vscroll->SetSizer(-1, 0,0,0,_scrollWidth,-1);
}
CListBase::~CListBase()
{
	QList<CListItemBase*>::iterator it = _list.begin();
	for (;it!= _list.end(); it++)
	{
		_list.erase(it);
		CListItemBase* item = *it;
		if (item != NULL)
		{
			delete item;
		}
	}
}

void CListBase::PushFront( CListItemBase* item )
{
	QList<CListItemBase*>::iterator it = _list.begin();
	bool isIn = false;
	for (;it!= _list.end(); it++)
	{
		CListItemBase* tempItem = *it;
		if (item == tempItem)
		{
			isIn = true;
		}
	}
	if (!isIn)
	{
		_list.push_front(item);
		_itemAllHeight = GetAllItemHeight();
		_vscroll->SetScroll(_itemAllHeight, _rect.height(),_scrollAt);
		_vscroll->SetScrollPos(_scrollPos+item->GetItemHeight());
		connect(item, SIGNAL(needResize(CListItemBase*, QRect)), this, SLOT(ChangeItemSize(CListItemBase*, QRect)));
		connect(item, SIGNAL(click(CListItemBase*)), this, SLOT(ReceiveClickEvent(CListItemBase*)));
		connect(item, SIGNAL(clickBtn(CListItemBase*, unsigned int)), this, SLOT(ReceiveClickBtnEvent(CListItemBase*, unsigned int)));
		connect(item, SIGNAL(doubleClick(CListItemBase*)), this, SLOT(ReceiveDoubleClickEvent(CListItemBase*)));
	}
}


void CListBase::Insert( CListItemBase* beforeItem, CListItemBase* item )
{
	QList<CListItemBase*>::iterator it = _list.begin();
	QList<CListItemBase*>::iterator beforeIt = _list.end();
	if (beforeItem != NULL)
	{
		for (;it!= _list.end(); it++)
		{
			CListItemBase* tempItem = *it;
			if (tempItem == beforeItem)
			{
				beforeIt = it;
				break;
			}
		}
	}
	_list.insert(beforeIt,item);
	_itemAllHeight = GetAllItemHeight();
	_vscroll->SetScroll(_itemAllHeight, _rect.height(),_scrollAt);
	connect(item, SIGNAL(needResize(CListItemBase*, QRect)), this, SLOT(ChangeItemSize(CListItemBase*, QRect)));
	connect(item, SIGNAL(click(CListItemBase*)), this, SLOT(ReceiveClickEvent(CListItemBase*)));
	connect(item, SIGNAL(clickBtn(CListItemBase*, unsigned int)), this, SLOT(ReceiveClickBtnEvent(CListItemBase*, unsigned int)));
	connect(item, SIGNAL(doubleClick(CListItemBase*)), this, SLOT(ReceiveDoubleClickEvent(CListItemBase*)));
	
}


//
void CListBase::PushBack(CListItemBase* item)
{
	QList<CListItemBase*>::iterator it = _list.begin();
	bool isIn = false;
	for (;it!= _list.end(); it++)
	{
		CListItemBase* tempItem = *it;
		if (item == tempItem)
		{
			isIn = TRUE;
		}
	}
	if (!isIn)
	{
		_list.push_back(item);
		_itemAllHeight = GetAllItemHeight();
		_vscroll->SetScroll(_itemAllHeight, _rect.height(),_scrollAt);
		connect(item, SIGNAL(needResize(CListItemBase*, QRect)), this, SLOT(ChangeItemSize(CListItemBase*, QRect)));
		connect(item, SIGNAL(click(CListItemBase*)), this, SLOT(ReceiveClickEvent(CListItemBase*)));
		connect(item, SIGNAL(clickBtn(CListItemBase*, unsigned int)), this, SLOT(ReceiveClickBtnEvent(CListItemBase*, unsigned int)));
		connect(item, SIGNAL(doubleClick(CListItemBase*)), this, SLOT(ReceiveDoubleClickEvent(CListItemBase*)));
	}
	
}
void CListBase::Erase(CListItemBase* item)
{
	QList<CListItemBase*>::iterator it = _list.begin();
	for (;it!= _list.end(); it++)
	{
		if (*it == item )
		{
			_list.erase(it);
			item->hide();
			delete item;
			break;
		}
	}
}
void CListBase::Erase(int itemId)
{
	QList<CListItemBase*>::iterator it = _list.begin();
	for (;it!= _list.end(); it++)
	{
		CListItemBase* item = *it;
		if (item->GetId() == itemId )
		{
			_list.erase(it);
			item->hide();
			delete item;
			break;
		}
	}
}
int CListBase::ItemHittest(int pos )
{
	if (_list.size() > 0)
	{
		int itemHeightAdd = 0;
		for (int index = 0; index< _list.size() ; index++)
		{
			int nextItemTop = itemHeightAdd + _list[index]->GetItemHeight();
			if (pos >= itemHeightAdd && pos < nextItemTop)
			{
				return index;
			}
			itemHeightAdd += _list[index]->GetItemHeight();
			
		}
	}
	return -1;
}
int CListBase::GetAllItemHeight( )
{
	int itemHeightSum = 0;
	if (_list.size() > 0)
	{
		for (int index = 0; index< _list.size() ; index++)
		{
			itemHeightSum += _list[index]->GetItemHeight();
		}
	}
	return itemHeightSum;
}
void CListBase::LayOut()
{
	
	_itemAllHeight = GetAllItemHeight();
	int spaceToScroll = 0;
	if (_itemAllHeight >= _rect.height())
	{
		if (_itemAllHeight > _rect.height())
		{
			_vscroll->show();
		}
		else
		{
			_vscroll->hide();
		}
		/*_vscroll->OnSizer();*/
		_vscroll->SetScroll(_itemAllHeight,_rect.height(),_scrollAt);
		_scrollPos = _vscroll->GetScrollPos();
		if (_itemWidth == 0)
		{
			spaceToScroll = _scrollWidth;
		}
		else
		{
			spaceToScroll = _rect.width() - _itemWidth;
		}
	}
	else
	{
		if (_scrollPos != 0)
		{
			_vscroll->SetScrollPos(0);
			_scrollPos = 0;
		}
		
		if (_itemWidth == 0)
		{
		}
		else
		{
			spaceToScroll = _rect.width() - _itemWidth;
		}
		_vscroll->hide();
	}
	int scrollItemIndex = 0;
	if (_scrollPos>0)
	{
		scrollItemIndex = ItemHittest(_scrollPos);
	}
	int itemHeightHideFront = 0;
	if (_list.size()>0 && _scrollPos >0)
	{
		for (int index = 0; index < scrollItemIndex; index++)
		{
			if (index >= _list.size())
			{
				break;
			}
			_list[index]->hide();
			itemHeightHideFront += _list[index]->GetItemHeight();
		}
	}
	int itemHeightShowMiddle = 0;
	int frameShowHeight = 0;
	if (_list.size() > 0)
	{
		int i = 0;
		for (int index = scrollItemIndex; frameShowHeight < _rect.height() ; index++,i++)
		{
			if (index >= _list.size())
			{
				break;
			}

			int tempHeight ;
			if (_itemHeight != 0)
			{
				//FrameList 中每个Item高度被限定
				tempHeight = _itemHeight;
			}
			{
				//FrameList 中每个Item 高度由Item决定
				tempHeight = _list[index]->GetItemHeight();
			}
			if (_scrollPos-itemHeightHideFront>0)
			{
				if (i == 0)
				{
					int partHeightOfShowItem1 = tempHeight -(_scrollPos-itemHeightHideFront);
					_list[index]->SetSizer(0, -1, spaceToScroll, _rect.height() - partHeightOfShowItem1, -1, tempHeight);
					frameShowHeight -= (_scrollPos-itemHeightHideFront);
				}
				else
				{
					_list[index]->SetSizer(0, itemHeightShowMiddle-(_scrollPos-itemHeightHideFront), spaceToScroll, _rect.height()+_scrollPos-itemHeightHideFront, -1, tempHeight);
				}
			}
			else
			{
				_list[index]->SetSizer(0, itemHeightShowMiddle, spaceToScroll, -1, -1, tempHeight);
			}
			
			_list[index]->OnSizer();
			_list[index]->show();
			itemHeightShowMiddle += tempHeight;
			frameShowHeight += tempHeight;
		}
		_itemShowMax = i;
	}
	
	if (_list.size() > scrollItemIndex + _itemShowMax )
	{
		for (int index = scrollItemIndex + _itemShowMax; index < _list.size(); index++)
		{
			if (index >= _list.size())
			{
				break;
			}
			_list[index]->hide();
		}
	}
}
void CListBase::paintEvent(QPaintEvent *event)
{
	QPainter dc(this);
	dc.fillRect(_rect,QBrush(_backGroundColor));
	if (_list.isEmpty())
	{	
		QFont font = dc.font();
		font.setPixelSize(12);
		font.setBold(true);
#ifdef _MSC_VER
        font.setFamily("Microsoft YaHei");
#endif
		dc.setFont(font);
		dc.setPen(QColor(0x99,0x99,0x99));
		
		dc.drawText(_rect, Qt::AlignCenter, _emptyStr);
	}
	//判断scroll是否改变
	if (_scrollPos != _vscroll->GetScrollPos())
	{
		_isScrollPosChanged = 1;
	}
	else
	{
		_isScrollPosChanged = 0;
	}
	//item布局
	LayOut();
	//底部信号
	//当scroll到底部的时候
	if (_isScrollPosChanged && (_itemAllHeight - _rect.height() <= _scrollPos) && _scrollPos > 0)
	{
		emit onBottom();
	}
	event->accept();
}
void CListBase::mousePressEvent(QMouseEvent *event)
{

}
void CListBase::leaveEvent(QEvent *event)
{

}
void CListBase::mouseReleaseEvent(QMouseEvent *event)
{

}
void CListBase::mouseMoveEvent(QMouseEvent *event)
{
	setCursor(Qt::ArrowCursor);
}
void CListBase::resizeEvent(QResizeEvent * event)
{
	QRect rect = this->frameGeometry();
	_rect = QRect(0,0,rect.width(),rect.height());
	
	QList<CListItemBase*>::iterator it = _list.begin();
	for (;it!= _list.end(); it++)
	{
		CListItemBase* item = *it;
		if (item != NULL)
		{
			item->OnSizer();
		}
	}
	_vscroll->OnSizer();
}
void CListBase::wheelEvent(QWheelEvent * event)
{
	_vscroll->wheelEvent(event);
}
void CListBase::ChangeItemSize(CListItemBase* item, QRect rect)
{
	if (item != NULL);
	{
		item->SetSizer(0, 0, 0, -1, -1, rect.height());
	}
}
void CListBase::ReceiveClickEvent(CListItemBase* item)
{
	//QObject* w = QObject::sender();

	//for (int i = 0; i < this->_list.size(); ++i)
	//{
	//	CProjectListItemBase* item = _list[i];

	//	if (item == w)
	//	{
	//		itemId = i;
	//		break;
	//	}
	//}

	emit click(item);
}
void CListBase::ReceiveDoubleClickEvent(CListItemBase* item)
{
	//QObject* w = QObject::sender();

	//for (int i = 0; i < this->_list.size(); ++i)
	//{
	//	CProjectListItemBase* item = _list[i];

	//	if (item == w)
	//	{
	//		itemId = i;
	//		break;
	//	}
	//}

	emit doubleClick(item);
}
void CListBase::ReceiveClickBtnEvent(CListItemBase* item, unsigned int option)
{
	//QObject* w = QObject::sender();

	//for (int i = 0; i < this->_list.size(); ++i)
	//{
	//	CProjectListItemBase* item = _list[i];

	//	if (item == w)
	//	{
	//		itemId = i;
	//		break;
	//	}
	//}

	emit clickBtn(item, option);
}

bool CListBase::IsEmpty()
{
	return _list.isEmpty();
}

void CListBase::ClearAll()
{
	QList<CListItemBase*>::iterator it = _list.begin();
	CListItemBase* item = NULL;
	for (;it!= _list.end(); it++)
	{
		item = *it;
		if (item != NULL)
		{
			_list.erase(it);
			delete item;
			item = NULL;
		}
	}
}

void CListBase::SetItemWidth( int width )
{
	_itemWidth = width;
}
void CListBase::SetItemHeight(int height)
{
	_itemAllHeight = height;
}
void CListBase::SetScrollWidth( int width )
{
	_scrollWidth = width;
	_vscroll->SetSizer(-1,0,0,0,width,-1);
}
//只有在Item高度一定的情况下才有效
int CListBase::GetShowMax()
{
	if (_itemHeight != 0)
	{
		return _rect.height()/_itemHeight+2;
	}
	else
	{
		return -1;
	}
	
}
CListItemBase* CListBase::GetItemByItemId( unsigned int id )
{
	QList<CListItemBase*>::iterator it = _list.begin();
	for (; it != _list.end(); it++)
	{
		if ((*it)->GetId() == id)
		{
			return *it;
		}
	}
	return NULL;
}
CListItemBase* CListBase::GetItemByIndex( unsigned int index )
{
	if ((0<=index) && ((int)index <= (int)(_list.size()-1)))
	{
		return _list[index];
	}
	return NULL;
}

int CListBase::GetSize()
{
	return _list.size();
}

int CListBase::CreateUniquenessId()
{
	return _itemUniquenessId++;
}

void CListBase::sort(frame_sort func)
{
	std::vector<CListItemBase*> v;
	for (int i = 0; i < _list.size(); ++i)
	{
		v.push_back(_list.at(i));
	}

	std::sort(v.begin(),v.end(),func);

	_list.clear();
	for (int i = 0; i < v.size(); ++i)
	{
		_list.push_back(v[i]);
	}
}

int CListBase::GetMaxID()
{
	int sizeOfList = _list.size();
	int id = 0;
	//查看notice是否已存在
	for (int index = 0; index < sizeOfList; index ++)
	{
		CListItemBase* tempItem = _list[index];
		if (tempItem && id < tempItem->GetId())
		{
			id = tempItem->GetId();
		}
	}
	return id;
}

void CListBase::SetScrollPos( int pos )
{
	_scrollPos = pos;
	_vscroll->SetScrollPos(pos);
}

void CListBase::SetEmptyString(QString str)
{
	_emptyStr = str;
}

CVScroll* CListBase::GetSCroll()
{
	return _vscroll;
}

//
//CProjectDocListFrame::CProjectDocListFrame(QWidget* parent, QColor backGroundColor, bool bIsForDocHistory)
//	:CListBase(parent, backGroundColor), m_bIsForDocHistory(bIsForDocHistory)
//{
//	_dropMenu = new CDropMenu(this);
//	connect(_dropMenu,SIGNAL(click(int)), this, SLOT(ClickDropMenu(int)));
//	connect(_vscroll, SIGNAL(scroll_change()), this, SLOT(ChangeDropMenu()));
//}
//CProjectDocListFrame::~CProjectDocListFrame()
//{
//
//}
//void CProjectDocListFrame::PushBack(CListItemBase* item)
//{
//	CListBase::PushBack(item);
//	connect(item, SIGNAL(clickOptBtn(unsigned int, unsigned int)), this,SLOT(OperateBtn(unsigned int, unsigned int)) );
//}
//
//void CProjectDocListFrame::PushFront( CListItemBase* item )
//{
//	CListBase::PushFront(item);
//	connect(item, SIGNAL(clickOptBtn(unsigned int, unsigned int)), this,SLOT(OperateBtn(unsigned int, unsigned int)) );
//}
//
//void CProjectDocListFrame::Insert( CListItemBase* fitem, CListItemBase* item )
//{
//	CListBase::Insert(fitem,item);
//	connect(item, SIGNAL(clickOptBtn(unsigned int, unsigned int)), this,SLOT(OperateBtn(unsigned int, unsigned int)) );
//}
//
//
//void CProjectDocListFrame::OperateBtn(unsigned int id, unsigned int type)
//{
//	CProjectDocListItem* tempItem = (CProjectDocListItem*)GetItemByItemId(id);
//	if (tempItem == NULL)
//	{
//		return;
//	}
//	if (OperatorDrop == type)
//	{
//		/*_menuItemBak = tempItem;*/
//		ShowDropMenu(tempItem);
//	}
//	else
//	{
//		emit clickBtn(id, type);
//	}
//}
//
//void CProjectDocListFrame::ClickDropMenu(int type)
//{
//	CProjectDocListItemBase* tempItem = _dropMenu->GetRelateItem();
//	if (NULL == tempItem)
//	{
//		return;
//	}
//	emit clickBtn(tempItem->GetId(), type);
//	_dropMenu->hide();
//}
//
//void CProjectDocListFrame::ShowDropMenu( CListItemBase* item )
//{
//	CProjectDocListItem* tempItem = (CProjectDocListItem*)item;
//	if (tempItem == NULL)
//	{
//		return;
//	}
//	CDocOperateBtn* objBtn = tempItem->GetOperateBtn();
//	QPoint pt = objBtn->mapToGlobal(QPoint(0,0));
//	QRect itemRc = objBtn->frameGeometry();
//	pt.setY(pt.ry()+itemRc.height()-2);
//	pt.setX(pt.rx()+1);
//
//	Document doc = GetMOffice()->GetProjectManager()->GetDocument(tempItem->GetDocument().GetID());
//
//	if (GetProjectUIManager()->IsCanEditDocumnet(tempItem->GetDocument().GetID()))
//	{
//		_dropMenu->AddDropItem(OperatorEdit, QSTRING("编辑"), true);
//		if (doc.IsShared())
//		{
//			_dropMenu->EraseDropItem(OperatorShare);
//			_dropMenu->AddDropItem(OperatorCancelShare, QSTRING("取消共享"), true);
//		}
//		else
//		{
//			_dropMenu->EraseDropItem(OperatorCancelShare);
//			_dropMenu->AddDropItem(OperatorShare, QSTRING("共享"), true);
//		}
//		_dropMenu->AddDropItem(OperatorDelete, QSTRING("删除"), true);
//	}
//	else
//	{
//		_dropMenu->EraseDropItem(OperatorShare);
//		_dropMenu->EraseDropItem(OperatorCancelShare);
//		_dropMenu->EraseDropItem(OperatorEdit);
//		_dropMenu->EraseDropItem(OperatorDelete);
//	}
//
//	_dropMenu->Show(tempItem, pt, true);
//
//	//Project project = GetMOffice()->GetProjectManager()->GetProject(tempItem->GetDocument().GetProjectID());
///*
//	if (tempItem->GetDocument().GetCreatorID() == GetProjectUIManager()->GetSelf().GetID())
//	{
//		// 		_dropMenu->SetItemEnable(CDropMenu::DocOperateBtn_Edit, true);
//		// 		_dropMenu->SetItemEnable(CDropMenu::DocOperateBtn_Delete, true);
//		// 		_dropMenu->SetItemEnable(CDropMenu::DocOperateBtn_Share, true);
//		if (!m_bIsForDocHistory)
//		{
//			_dropMenu->AddDropItem(OperatorEdit, QSTRING("编辑"), true);
//			if ( !tempItem->GetDocument().IsShared() )
//			{
//				_dropMenu->EraseDropItem(OperatorCancelShare);
//				_dropMenu->AddDropItem(OperatorShare, QSTRING("共享"), true);
//			}
//			else
//			{
//				_dropMenu->EraseDropItem(OperatorShare);
//				_dropMenu->AddDropItem(OperatorCancelShare, QSTRING("取消共享"), true);
//			}
//		}
//        _dropMenu->AddDropItem(OperatorDelete, QSTRING("删除"), true);
//	}
//	else if (project.GetCreatorID() == GetProjectUIManager()->GetSelf().GetID())
//	{
//
//	}
//	else
//	{
//		if (!m_bIsForDocHistory)
//		{
//			_dropMenu->EraseDropItem(OperatorEdit);
//			_dropMenu->EraseDropItem(OperatorShare);
//			_dropMenu->EraseDropItem(OperatorCancelShare);
//		}
//		_dropMenu->EraseDropItem(OperatorDelete);
//		// 		_dropMenu->SetItemEnable(CDropMenu::DocOperateBtn_Edit, false);
//		// 		_dropMenu->SetItemEnable(CDropMenu::DocOperateBtn_Delete, false);
//		// 		_dropMenu->SetItemEnable(CDropMenu::DocOperateBtn_Share, false);
//	}
//
//	_dropMenu->Show(tempItem, pt, true);
//*/
//}
//
//void CProjectDocListFrame::ChangeDropMenu()
//{
//	_dropMenu->hide();
//}
