﻿
#include "MessageBox.h"
#include "TextLabel.h"
#include "Button2.h"
#include "Button3.h"

#include "ModalWindowManager.h"


CMessageDialog::~CMessageDialog()
{
	GetModalWindowManager()->Remove(this);

	if (_closeBtn != NULL)
	{
		delete _closeBtn;
	}
	if (_okBtn != NULL)
	{
		delete _okBtn;
	}
	if (_cancelBtn != NULL)
	{
		delete _cancelBtn;
	}
	if (_textLabel != NULL)
	{
		delete _textLabel;
	}
}

CMessageDialog::CMessageDialog( QWidget* parent, QString title, QString content, StandardButton buttons /*= Normal*/ )
	: FrameWindow(parent,Qt::FramelessWindowHint | Qt::Tool | Qt::WindowStaysOnTopHint),_content(content), _title(title)
{
	GetModalWindowManager()->AddWidget(this);

#ifdef MACOS
	setAttribute(Qt::WA_TranslucentBackground);
#endif

	//QIcon icon(":/mOffice/Resources/mOffice.ico");
	//this->setWindowIcon(icon);

/*	QWidget::setWindowModality(Qt::WindowModal);*/
	/////
	setMouseTracking(true);

	_round_radius = 2;
	/////
	int height = 160;
	int width = 280;
	QRect rc;
	if (parent == NULL)
	{
		/*setWindowModality(Qt::ApplicationModal);*/
		rc = QApplication::desktop()->availableGeometry(); 
	}
	else
	{
		setWindowModality(Qt::WindowModal);
		rc = parent->frameGeometry();
	}
	int x = (rc.width() - width) / 2 + rc.left();
	int y = (rc.height() - height) / 2 + rc.top();

	setGeometry(QRect(x,y,width,height));
	
	_backGroundColor = QColor(51, 51, 51);
	_id = 0;
	_buttonsFlag = buttons;
	InitMessageDialog();
}


void CMessageDialog::paintEvent( QPaintEvent *event )
{
	QPainter dc(this);
	dc.fillRect(rect(), QBrush(_backGroundColor));
	//dc.fillRect(QRect(10,30,_rect.width() - 20,_rect.height()-30-40),QBrush(QColor(221,221,221)));
	QFont font = dc.font();
	font.setPixelSize(12);
#ifdef _MSC_VER
    font.setFamily("Microsoft YaHei");
#endif
	dc.setFont(font);
	dc.setPen(QColor(255,255,255));
	dc.drawText(QRect(10,0,rect().width()-10,30),Qt::AlignVCenter, _title);
}

void CMessageDialog::resizeEvent( QResizeEvent *event )
{

	FrameWindow::resizeEvent(event);

	_closeBtn->OnSizer();
	_okBtn->OnSizer();
	_cancelBtn->OnSizer();
	_textLabel->OnSizer();

}

void CMessageDialog::InitMessageDialog()
{
	_closeBtn = new CButton2(this, 0,0, ":/nice2meet/Resources/image/frame_button_closed.png");
	_closeBtn->SetSizer(-1,5,5,-1,-1,-1);
	_closeBtn->hide();
	_okBtn = new CButton3(this, 0,0, ":/nice2meet/Resources/image/frame_button_bottom_Define.png");
	_okBtn->SetSizer(65,-1,-1,10,-1,-1);
	_okBtn->hide();
	_cancelBtn = new CButton3(this , 0,0, ":/nice2meet/Resources/image/frame_button_bottom_Cancel.png");
	_cancelBtn->SetSizer(-1,-1,65,10,-1,-1);
	_cancelBtn->hide();
	connect(_closeBtn, SIGNAL(click()), this, SLOT(ClickButton()));
	connect(_okBtn, SIGNAL(click()), this, SLOT(ClickButton()));
	connect(_cancelBtn, SIGNAL(click()), this, SLOT(ClickButton()));
	_textLabel = new CTextLabel(this, _content, QColor(221,221,221), Qt::FramelessWindowHint);
	_textLabel->SetRoundRadius(2);
	_textLabel->SetAlignFlag(Qt::AlignCenter | Qt::TextWrapAnywhere);
	_textLabel->SetSizer(25,30,25,40, -1,-1);

	ShowButtons( _buttonsFlag);
}

void CMessageDialog::ClickButton( void )
{
	if (QObject::sender() == _closeBtn)
	{
		close();
	}
	else if (QObject::sender() == _okBtn)
	{
		accept();
	}
	else if (QObject::sender() == _cancelBtn)
	{
		reject();
	}
}

void CMessageDialog::ShowButtons( StandardButton buttons )
{
	if (buttons == No_Button)
	{
		_closeBtn->show();
		_cancelBtn->hide();
		_okBtn->hide();
	}
	else if (buttons == Normal)
	{
		_closeBtn->show();
		_cancelBtn->show();
		_okBtn->show();
	}
	else
	{
		if ((buttons & Ok) && (buttons & Cancel) )
		{
			_okBtn->show();
			_cancelBtn->show();
		}
		else if(buttons & Ok)
		{
			_closeBtn->show();
			_okBtn->SetSizer((size().width()-_okBtn->size().width())/2,-1,-1,10,-1,-1);
			/*_okBtn->OnSizer();*/
			_okBtn->show();
		}
		else
		{
			_closeBtn->show();
			_cancelBtn->hide();
			_okBtn->hide();
		}
	}	
}

void CMessageDialog::SetAlignFlag( int flag )
{
	_textLabel->SetAlignFlag(flag);
}


