﻿
#include "RoundRadiusWnd.h"
#include <QtGui/qimage.h>


CRoundRadiusWnd::CRoundRadiusWnd(QWidget *parent, Qt::WindowFlags flags)
	: QWidget(parent, flags), _can_move_wnd(false)
{
	setMouseTracking(true);
	_drag_type = DragNone;

#ifdef MACOS
	setAttribute(Qt::WA_TranslucentBackground);
#endif

	_round_radius = 5;
}

CRoundRadiusWnd::~CRoundRadiusWnd()
{

}

void CRoundRadiusWnd::SetRoundRadius(int round)
{
	_round_radius = round;
}

void CRoundRadiusWnd::resizeEvent(QResizeEvent * event)
{
	if (_round_radius == 0) return;

	QBitmap bmp(this->size());
	bmp.fill();
	QPainter p(&bmp);

	//p.setRenderHint(QPainter::Antialiasing);    //抗锯齿

	QRect rc = this->rect();
	QPainterPath path;

	int ROUND = _round_radius;

	path.moveTo(rc.left(), rc.center().y());

	path.arcTo(rc.left(), rc.top(), ROUND*2, ROUND*2, 180.0f, -90.0f);
	path.arcTo(rc.right()+1 - ROUND*2, rc.top(), ROUND*2, ROUND*2, 90.0f, -90.0f);

	path.arcTo(rc.right()+1 - ROUND*2, rc.bottom()+1 - ROUND*2, ROUND*2, ROUND*2, 360.0f, -90.0f);
	path.arcTo(rc.left(), rc.bottom()+1 - ROUND*2, ROUND*2, ROUND*2, 270.0f, -90.0f);

	path.closeSubpath();

	p.fillPath(path, QBrush(QColor(255,0,0)));
	setMask(bmp);
}

void CRoundRadiusWnd::mousePressEvent(QMouseEvent *event)
{
	//按住左键可以托动窗口，按下右键关闭程序    
	if(event->button() == Qt::LeftButton && _can_move_wnd)    
	{
		//grabMouse();

		_down_pos = event->globalPos();
		_down_rc = frameGeometry();
		event->accept();

		activateWindow();
		setFocus(Qt::MouseFocusReason);

		_drag_type = Hittest(event->x(), event->y());

		if (_drag_type == DragClient)
		{
			setCursor(Qt::ArrowCursor);
		}
		else if (_drag_type == DragTopLeft || _drag_type == DragBottomRight)
		{
			setCursor(Qt::SizeFDiagCursor);
		}
		else if (_drag_type == DragTopRight || _drag_type == DragBottomLeft)
		{
			setCursor(Qt::SizeBDiagCursor);
		}
		else if (_drag_type == DragLeft || _drag_type == DragRight)
		{
			setCursor(Qt::SizeHorCursor);
		}
		else if (_drag_type == DragTop || _drag_type == DragBottom)
		{
			setCursor(Qt::SizeVerCursor);
		}
	}
}

void CRoundRadiusWnd::mouseMoveEvent(QMouseEvent *event)
{
	if (!_can_move_wnd)
	{
		return;
	}
	int min_width = minimumWidth();
	int min_height = minimumHeight();

	if (_drag_type == DragClient)
	{
		setCursor(Qt::ArrowCursor);
		if (event->buttons() == Qt::LeftButton)
		{
			move(_down_rc.topLeft() + event->globalPos() - _down_pos);
			event->accept();
		}
	}
	else if (_drag_type == DragTopLeft || _drag_type == DragBottomRight)
	{
		setCursor(Qt::SizeFDiagCursor);
		event->accept();

		if (_drag_type == DragTopLeft)
		{
			int x = _down_rc.x() + (event->globalX() - _down_pos.x());
			int y = _down_rc.y() + (event->globalY() - _down_pos.y());
			int w = _down_rc.width() - (event->globalX() - _down_pos.x());
			int h = _down_rc.bottom() - y;

			if (w < min_width)
			{
				x = _down_rc.right() - min_width;
				w = min_width;
			}
			if (h < min_height)
			{
				y = _down_rc.bottom() - min_height;
				h = min_height;
			}
			setGeometry(x,y,w,h);
		}
		else
		{
			int x = _down_rc.x();
			int y = _down_rc.y();
			int w = _down_rc.width() + (event->globalX() - _down_pos.x());
			int h = _down_rc.height() + (event->globalY() - _down_pos.y());
			setGeometry(x,y,w,h);
		}
	}
	else if (_drag_type == DragTopRight || _drag_type == DragBottomLeft)
	{
		setCursor(Qt::SizeBDiagCursor);

		if (_drag_type == DragTopRight)
		{
			int x = _down_rc.x();
			int y = _down_rc.y() + (event->globalY() - _down_pos.y());
			int w = _down_rc.width() + (event->globalX() - _down_pos.x());
			int h = _down_rc.bottom() - y;

			if (w < min_width)
			{
				x = _down_rc.x();
				w = min_width;
			}
			if (h < min_height)
			{
				y = _down_rc.bottom() - min_height;
				h = min_height;
			}
			setGeometry(x,y,w,h);
		}
		else
		{
			int x = _down_rc.x() + (event->globalX() - _down_pos.x());
			int y = _down_rc.y();
			int w = _down_rc.right() - x;
			int h = _down_rc.height() + (event->globalY() - _down_pos.y());

			if (w < min_width)
			{
				x = _down_rc.right() - min_width;
				w = min_width;
			}
			if (h < min_height)
			{
				y = _down_rc.y();
				h = min_height;
			}
			setGeometry(x,y,w,h);
		}
	}
	else if (_drag_type == DragLeft || _drag_type == DragRight)
	{
		setCursor(Qt::SizeHorCursor);
		if (_drag_type == DragLeft)
		{
			int x = _down_rc.x() + (event->globalX() - _down_pos.x());
			int y = _down_rc.y();
			int w = _down_rc.right() - x;
			int h = _down_rc.height();

			if (w < min_width)
			{
				x = _down_rc.right() - min_width;
				w = min_width;
			}
			setGeometry(x,y,w,h);
		}
		else
		{
			int x = _down_rc.x();
			int y = _down_rc.y();
			int w = _down_rc.width() + (event->globalX() - _down_pos.x());
			int h = _down_rc.height();

			if (w < min_width)
			{
				x = _down_rc.x();
				w = min_width;
			}
			setGeometry(x,y,w,h);
		}
	}
	else if (_drag_type == DragTop || _drag_type == DragBottom)
	{
		setCursor(Qt::SizeVerCursor);
		if (_drag_type == DragTop)
		{
			int x = _down_rc.x();
			int y = _down_rc.y() + (event->globalY() - _down_pos.y());
			int w = _down_rc.width();
			int h = _down_rc.bottom() - y;

			if (h < min_height)
			{
				y = _down_rc.bottom() - min_height;
				h = min_height;
			}
			setGeometry(x,y,w,h);
		}
		else
		{
			int x = _down_rc.x();
			int y = _down_rc.y();
			int w = _down_rc.width();
			int h = _down_rc.height() + (event->globalY() - _down_pos.y());

			if (h < min_height)
			{
				y = _down_rc.y();
				h = min_height;
			}
			setGeometry(x,y,w,h);
		}
	}
	else
	{
		DragType drag_type = Hittest(event->x(), event->y());

		if (drag_type == DragClient)
		{
			setCursor(Qt::ArrowCursor);
		}
		else if (drag_type == DragTopLeft || drag_type == DragBottomRight)
		{
			setCursor(Qt::SizeFDiagCursor);
		}
		else if (drag_type == DragTopRight || drag_type == DragBottomLeft)
		{
			setCursor(Qt::SizeBDiagCursor);
		}
		else if (drag_type == DragLeft || drag_type == DragRight)
		{
			setCursor(Qt::SizeHorCursor);
		}
		else if (drag_type == DragTop || drag_type == DragBottom)
		{
			setCursor(Qt::SizeVerCursor);
		}
	}
}

void CRoundRadiusWnd::mouseReleaseEvent(QMouseEvent *event)
{
	releaseMouse();
	_drag_type = DragNone;

	setCursor(Qt::ArrowCursor);
}

CRoundRadiusWnd::DragType CRoundRadiusWnd::Hittest(int x, int y)
{
	int width = rect().width();
	int height = rect().height();

	if (x < 12 && y < 12)
	{
		return DragTopLeft;
	}
	if (x < 12 && y > height - 12)
	{
		return DragBottomLeft;
	}
	if (x > width - 12 && y < 12)
	{
		return DragTopRight;
	}
	if (x > width - 12 && y > height - 12)
	{
		return DragBottomRight;
	}

	if (x < 6)
	{
		return DragLeft;
	}
	if (x > width - 6)
	{
		return DragRight;
	}
	if (y < 6)
	{
		return DragTop;
	}
	if (y > height - 6)
	{
		return DragBottom;
	}

	return DragClient;
}

void CRoundRadiusWnd::SetMoveWndEnable(bool enableToMove)
{
	_can_move_wnd = enableToMove;
}

void CRoundRadiusWnd::DrawBorder(QPainter& painterDc)
{
	int sideLength = _round_radius * 2;
	painterDc.setRenderHint(QPainter::Antialiasing);
	painterDc.drawArc(QRect(0, 0, sideLength, sideLength), 16 * 90, 16 * 90);
	painterDc.drawArc(QRect(rect().width() - sideLength, 0, sideLength, sideLength), 16 * 0, 16 * 90);
	painterDc.drawArc(QRect(0, rect().height() - sideLength, sideLength, sideLength), 16 * 180, 16 * 90);
	painterDc.drawArc(QRect(rect().width() - sideLength, rect().height() - sideLength, sideLength, sideLength), 16 * 270, 16 * 90);
	painterDc.drawRect(QRect(0, 0, rect().width(), rect().height()));
		
}


