﻿#include "TextEdit.h"
#include <QtGui/qpainter.h>

CTextEdit::CTextEdit( int id, QWidget* parent, QString text, QColor color)
	: QWidget(parent, Qt::FramelessWindowHint),_text(text),_id(id),_backGroundColor(color),_editType(SingleLineEditBox)
{
	_textMaxCount = -1;

	_textEditor = new QTextEdit( this);
	_textEditor->setContextMenuPolicy(Qt::NoContextMenu);
	_textEditor->setPlainText(_text);
	connect(_textEditor, SIGNAL(textChanged()), this, SLOT(ChangeText()));

	QPalette palette = _textEditor->palette();
	palette.setColor(QPalette::Base , color);
	palette.setColor(QPalette::Text , QColor( ));
	_textEditor->setPalette(palette);

	QFont fontEdit;
#ifdef _MSC_VER
    fontEdit.setFamily("Microsoft YaHei");
#endif
	fontEdit.setPixelSize(12);
	fontEdit.setBold(false);
	_textEditor->setFont(fontEdit);
	_textEditor->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	ResetTextRect(_rect);

	QString style = QString("QTextEdit{border-width:0;border-style:outset;background-color:rgb(%1,%2,%3);color:rgb(0x33,0x33,0x33);}").arg(color.red()).arg(color.green()).arg(color.blue());
	_textEditor->setStyleSheet(style);

	_is_show_frame = true;
}

CTextEdit::~CTextEdit()
{

}

void CTextEdit::SetText(QString text)
{
	_text = text;
	_textEditor->setPlainText(text);
}

QString CTextEdit::GetText()
{
	_text = _textEditor->toPlainText();
	return _text;
}

void CTextEdit::paintEvent( QPaintEvent *event )
{
	QPainter dc(this);

	if (_is_show_frame)
	{
		dc.setPen(QColor());
		dc.setBrush(QBrush(_backGroundColor));
		QRect rc(_rect.left(),_rect.top(),_rect.width()-1, _rect.height()-1);
		dc.drawRoundedRect(rc,2,2);
	}
	else
	{
		dc.fillRect(_rect, QBrush(_backGroundColor));
	}
}

void CTextEdit::resizeEvent( QResizeEvent *event )
{
	QWidget::resizeEvent(event);
	QRect rect = this->frameGeometry();
	_rect = QRect(0,0,rect.width(),rect.height());
	ResetTextRect(_rect);
	_textEditor->setGeometry(_textRect);
}



void CTextEdit::ResetTextRect(QRect& rc)
{
	_textEditor->document()->adjustSize();
	int h ;
	if (_editType == SingleLineEditBox)
	{
		h = _textEditor->document()->size().height();
	}
	else
	{
		h = _rect.height() - BOX_TEXT_LEFT_SPACE_VAL*2;
	}
	
	int top = (_rect.height() - h)/2;
	if (top < 0 || top >= _rect.height() || h >= _rect.height() || h <= 0)
	{
		top = 2;
		h =  _rect.height()-4;
	}
	_textRect = QRect(BOX_TEXT_LEFT_SPACE_VAL, top, _rect.width()-BOX_TEXT_LEFT_SPACE_VAL*2, h);
}

void CTextEdit::ChangeText()
{
	if (_textMaxCount == -1)
	{
		_text = _textEditor->toPlainText();
		emit textChanged();
	}
	else
	{
		if (LimitTextLenth(_textMaxCount))
		{
			//Text 的长度大于限制的长度
		}
		else
		{
			//Text的长度小于限制的长度
			_text = _textEditor->toPlainText();
			emit textChanged();
		}
	}
}

void CTextEdit::SetReadOnly( bool val)
{
	_textEditor->setReadOnly(val);
}

void CTextEdit::SetEditType( EditType type )
{
	_editType = type;
}

void CTextEdit::SetAlignFlag(int flag)
{
	_textAlignFlag =  flag;
	_textEditor->setAlignment((Qt::Alignment)flag);
}

bool CTextEdit::LimitTextLenth( int maxLength )
{
	QString textContent = _textEditor->toPlainText();
	int length = textContent.count();
	if(length > maxLength)
	{
		int position = _textEditor->textCursor().position();
		QTextCursor textCursor = _textEditor->textCursor();
		textContent.remove(position - (length - maxLength), length - maxLength);
		_textEditor->setText(textContent);
		textCursor.setPosition(position - (length - maxLength));
		_textEditor->setTextCursor(textCursor);
		return true;
	}
	return false;
}

void CTextEdit::SetTextLimitCount( int count )
{
	_textMaxCount = count;
}

void CTextEdit::InstallEventFilter( QObject *obj )
{
	_textEditor->installEventFilter(obj);
}

bool CTextEdit::EditHasFocus()
{
	return _textEditor->hasFocus();
}

void CTextEdit::SetEditFocus()
{
	_textEditor->setFocus();
}

void CTextEdit::HideFrame()
{
	_is_show_frame = false;
}

QTextEdit* CTextEdit::GetEditObj()
{
	return _textEditor;
}
