﻿

#ifndef __V_SCROLL_H__
#define __V_SCROLL_H__


#include <QtGui/qimage.h>
#include "QtPlatform.h"

#include "WidgetSizer.h"

class CVScroll : public QWidget, public CWidgetSizer<CVScroll>
{
public:
	Q_OBJECT
public:
	enum PosAt
	{
		InitAtTop,
		InitAtBottom,
		InitHoldOn,
	};
	int			_step;
	QColor		_bg_color1;
	QColor		_bg_color2;
	QColor		_color;
	int			_pos;
	int			_max;
	int			_page;
	QRect		_thumb_rc;
	int			_down_y;
	int			_last_y;
	bool		_is_down;
	bool		_is_init_at_end;

	CVScroll(QWidget *_parent, int scroll_step)
		:QWidget(_parent)
	{
		_bg_color1 = QColor(255,255,255);
		_bg_color2 = QColor(255,255,255);
		_color = QColor(157,167,176);
		_pos = 0;
		_max = 1;
		_page = 1;
		_is_down = false;

		QRect rc = _parent->rect();
		_step = scroll_step;
		this->setGeometry(rc.width()-8,0,rc.height(),8);
		this->SetSizer(-1,0,0,0,8,-1);
		setMouseTracking(true);
	}

	~CVScroll()
	{

	}

	void SetBgColor(QColor color1, QColor color2)
	{
		_bg_color1 = color1;
		_bg_color2 = color2;
	}

	void SetColor(QColor color)
	{
		_color = color;
	}

	void SetStep(int step)
	{
		_step = step;
	}

	void SetScroll(int max, int page, PosAt type/* = InitHoldOn*/)
	{
		if (max != _max || page != _page)
		{
			_max = max;
			if (_max <= 0)
				_max = 1;
			_page = page;
			if (_pos > max - page)
			{
				_pos = max - page;
				if (_pos < 0)
				{
					_pos = 0;
					emit scroll_change();
				}
			}
			if (type == InitAtBottom)
			{
				_pos = max - page;
			}
			else if (type == InitAtTop)
			{
				_pos = 0;
			}
			Layout();
		}
	}

	void SetScrollPos(int pos)
	{
		if (pos != _pos)
		{
			if (pos > _max - _page)
				pos = _max - _page;
			if (pos < 0)
				pos = 0;
			if (_pos != pos)
			{
				_pos = pos;
				Layout();
				this->parentWidget()->repaint();
				emit scroll_change();
			}
			else
			{
				Layout();
			}
		}
	}

	void ScrollToEnd()
	{
		SetScrollPos(_max);
	}

	int GetScrollPos()
	{
		return _pos;
	}

	int GetMax()
	{
		return _max;
	}

	int GetPage()
	{
		return _page;
	}

	void Layout()
	{
		QRect rc = rect();
		float unit_pixel = (float)(rc.height()) / _max;
		int thum_height = int(_page * rc.height() / _max);
		int yy = (int)(_pos * unit_pixel);

		int thum_width = rc.width();
		int xx = rc.right() - thum_width;

		_thumb_rc.setRect(xx,yy,thum_width,thum_height);
	}

	void wheelEvent(QWheelEvent * event)
	{
#if(QT_VERSION < 0x500000)
		int off = event->delta() / -120;
#else
		int off = event->angleDelta().y() / -120;	
#endif
		SetScrollPos(_pos + (off * _step));
		event->accept();
	}

signals:

	void click();
	void scroll_change();

protected:

	void resizeEvent(QResizeEvent * event)
	{
		Layout();
	}

	void paintEvent(QPaintEvent *_event)
	{
		QPainter dc(this);
		QRect rc = rect();
		{
			QLinearGradient gradient(0,0,0,rc.height());
			gradient.setColorAt(0, _bg_color1);
			gradient.setColorAt(1, _bg_color2);
			dc.fillRect(QRectF(0,0,rc.width(),rc.height()), QBrush(gradient));
		}

		if (_max > _page)
		{
			dc.setRenderHint(QPainter::Antialiasing);

			rc = _thumb_rc;
			QPainterPath path;

			int ROUND = 3;

			path.moveTo(rc.left()+1, rc.center().y());

			path.arcTo(rc.left()+1, rc.top(), ROUND*2, ROUND*2, 180.0f, -90.0f);
			path.arcTo(rc.right() - ROUND*2, rc.top(), ROUND*2, ROUND*2, 90.0f, -90.0f);

			path.arcTo(rc.right() - ROUND*2, rc.bottom() - ROUND*2, ROUND*2, ROUND*2, 360.0f, -90.0f);
			path.arcTo(rc.left()+1, rc.bottom() - ROUND*2, ROUND*2, ROUND*2, 270.0f, -90.0f);

			path.closeSubpath();

			dc.fillPath(path, QBrush(_color));
		}
	}

	void leaveEvent(QEvent *event)
	{

	}

	void mousePressEvent(QMouseEvent* event)
	{
		if (event->button() == Qt::LeftButton)
		{
			//this->grabMouse();

			_is_down = true;
			_down_y = event->y();
			_last_y = _down_y;
		}
		event->accept();
	}

	void mouseMoveEvent(QMouseEvent* event)
	{
		event->accept();
		setCursor(Qt::ArrowCursor);

		if (_page > _max)
		{
			return;
		}

		QRect rc = rect();
		int y = event->y();

		if (y < 0)
		{
			y = 0;
		}
		else if (y > rc.height())
		{
			y = rc.height();
		}

		if (_is_down)
		{
			int off = y - _last_y;
			float unit_pixel = (float)(rc.height()) / _max;
			_thumb_rc.translate(0,off);
			if (_thumb_rc.top() < 0)
			{
				_thumb_rc.translate(0, -_thumb_rc.top());
			}
			else if (_thumb_rc.bottom() > rc.height())
			{
				_thumb_rc.translate(0, rc.height() - _thumb_rc.bottom());
			}
			int pos = _thumb_rc.top() / unit_pixel;
			if (_pos != pos)
			{
				_pos = pos;
				emit scroll_change();
			}
			_last_y = y;
			this->parentWidget()->repaint();
		}
	}

	void mouseReleaseEvent(QMouseEvent *event)
	{
		if (event->button() == Qt::LeftButton)
		{

			event->accept();
		}
		this->releaseMouse();
		_is_down = false;
	}

	void mouseDoubleClickEvent(QMouseEvent * event)
	{
		event->accept();
	}

private:

};


#endif


