﻿

#ifndef __WIDGET_SIZER_H__
#define __WIDGET_SIZER_H__

#include <QtGui/qimage.h>
#include "QtPlatform.h"

#ifdef _MSC_VER
#define QSTRING(s) QString((const QChar*)L##s)
#else
#define QSTRING(s) QString(s)
#endif 

template <typename T>
class CWidgetSizer
{
protected:
	int		_left_sizer;
	int		_top_sizer;
	int		_right_sizer;
	int		_bottom_sizer;
	int		_width_sizer;
	int		_height_sizer;
public:
	CWidgetSizer()
	{
		_left_sizer = -1;
		_top_sizer = -1;
		_right_sizer = -1;
		_bottom_sizer = -1;
		_width_sizer = -1;
		_height_sizer = -1;
	}

	void SetSizer(int _left, int _top, int _right, int _bottom, int _width, int _height)
	{
		_left_sizer = _left;
		_top_sizer = _top;
		_right_sizer = _right;
		_bottom_sizer = _bottom;
		_width_sizer = _width;
		_height_sizer = _height;
/*
#ifdef MACOS
        QWidget* w = ((T*)this)->parentWidget();
         if (w && w->parentWidget() == NULL)
         {
              if ((w->windowFlags() & Qt::Window) == Qt::Window)
              {
                   if (_top_sizer >= 0)
                   {
                       _top_sizer -= 20;
                       if (_top_sizer < 0)
                       {
                           _top_sizer = 0;
                       }
                   }
              }
         }
#endif
*/
	}

    void SetSizerTop(int _top)
    {
        _top_sizer = _top;
    }

    void SetSizerLeft(int _left)
    {
        _left_sizer = _left;
    }

	void SetSizerRight(int _right)
	{
		_right_sizer = _right;
	}

	void SetSizerBottom(int _bottom)
	{
		_bottom_sizer = _bottom;
	}

	void SetSizerWidth(int _width)
	{
		_width_sizer = _width;
	}

	void SetSizerHeight(int _height)
	{
		_height_sizer = _height;
	}

	virtual void OnSizer()
	{
		if (_left_sizer < 0 && _right_sizer < 0 
			&& _top_sizer < 0 && _bottom_sizer < 0
			&& _width_sizer < 0 && _height_sizer < 0)
		{
			return;
		}

        bool is_framw_window = false;
#ifdef MACOS
        QWidget* w = ((T*)this)->parentWidget();
        QWidget* w2 = NULL;
        if (w)
        {
            w2 = w->parentWidget();
        }
        if (w2 == NULL && (w->windowFlags() & Qt::Window) == Qt::Window)
        {
            if ((w->windowFlags() & Qt::FramelessWindowHint) != Qt::FramelessWindowHint)
            {
                is_framw_window = true;
            }
        }
#endif

		QWidget* parent = ((T*)this)->parentWidget();
		if (parent == NULL) return;

		QRect rcParent = parent->frameGeometry();
		QRect rc = ((T*)this)->frameGeometry();

		int parent_width = rcParent.width();
		int parent_height = rcParent.height();

		int left = 0;
		int right = 0;
		int top = 0;
		int bottom = 0;

		if (_left_sizer >= 0)
		{
			if (_right_sizer >= 0)
			{
				left = 0 + _left_sizer;
				right = 0 + parent_width - _right_sizer;
			}
			else
			{
				left = 0 + _left_sizer;
				right = left + ((_width_sizer>0 )? _width_sizer : rc.width());
			}
		}
		else
		{
			if (_right_sizer >= 0)
			{
				right = 0 + parent_width - _right_sizer;
				left = right - ((_width_sizer>0 )? _width_sizer : rc.width());
			}
			else
			{
				left = rc.x();
				right = rc.x() + rc.width();
			}
		}
		
		if (_top_sizer >= 0)
		{
			if (_bottom_sizer >= 0)
			{
                if (is_framw_window)
                {
                    top = 0 + _top_sizer;
                    bottom = 0 + parent_height - _bottom_sizer - 20;
                }
                else
                {
                    top = 0 + _top_sizer;
                    bottom = 0 + parent_height - _bottom_sizer;
                }
			}
			else
			{
				top = 0 + _top_sizer;
				bottom = top + ((_height_sizer>0 )? _height_sizer : rc.height());
			}
		}
		else
		{
			if (_bottom_sizer >= 0)
			{
                if (is_framw_window)
                {
                    bottom = 0 + parent_height - _bottom_sizer - 20;
                    top = bottom - ((_height_sizer>0 )? _height_sizer : rc.height());
                }
                else
                {
                    bottom = 0 + parent_height - _bottom_sizer;
                    top = bottom - ((_height_sizer>0 )? _height_sizer : rc.height());
                }
			}
			else
			{
				top = rc.y();
				bottom = rc.y() + rc.height();
			}
		}

		if (_width_sizer >= 0)
		{
			right = left + _width_sizer;
		}
		if (_height_sizer >= 0)
		{
			bottom = top + _height_sizer;
		}
		
		int width = right - left;
		int height = bottom - top;

		((T*)this)->setGeometry(left,top,width,height);
	}

	QRect GetGlobalRect()
	{
		QRect rc = ((T*)this)->frameGeometry();

		QWidget* w = ((T*)this);
		QWidget* root = ((T*)this);
		while (w)
		{
			root = w;
			w = w->parentWidget();
		}

		QRect rc2 = root->frameGeometry();

		return QRect(rc.x() + rc2.x(), rc.y() + rc2.y(), rc.width(), rc.height());
	}
};

#endif
