#include "AudioList.h"
#include "Button2.h"

bool CDeviceListView::selectCurrentDevice(tee3::avd::IAudioDevice* device, tee3::avd::Device& item, bool isSpeaker)
{
	CDeviceListView* listDialog = new CDeviceListView(device, item, QColor(38, 37, 42, 255), isSpeaker);
	listDialog->setCurrent(item);
	int rv = listDialog->exec();
	bool selectnew = false;
	if      (QDialog::Rejected == rv)
	{
		selectnew = false;
	}
	else if (QDialog::Accepted == rv)
	{
		item = listDialog->GetDevice();
		selectnew = true;
	}
	delete listDialog;
	return selectnew;
}

bool CDeviceListView::selectCurrentDevice(QPoint moveTo, tee3::avd::IAudioDevice* device, tee3::avd::Device& item, bool isSpeaker) {
	CDeviceListView* listDialog = new CDeviceListView(device, item, QColor(38, 37, 42, 255), isSpeaker);
	listDialog->setCurrent(item);
	moveTo.setX(moveTo.x() - g_width);
	listDialog->move(moveTo);
	int rv = listDialog->exec();
	bool selectnew = false;
	if (QDialog::Rejected == rv) {
		selectnew = false;
	}
	else if (QDialog::Accepted == rv) {
		item = listDialog->GetDevice();
		selectnew = true;
	}
	delete listDialog;
	return selectnew;
}
int CDeviceListView::g_width = 350;
int CDeviceListView::g_height = 460;

CDeviceListView::CDeviceListView(IAudioDevice* sdk, tee3::avd::Device& current, QColor backColor, bool isSpeaker)
{
	SetRoundRadius(8);
	setMaximumSize(g_width, g_height);
	setMinimumSize(g_width, g_height);
	_back_ground_color = backColor;
	_device_list = new CDeviceList(this, _back_ground_color);
	_device_list->SetSizer(5, 40, 5, 10, -1, -1);
	_wnd_close_btn = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/close_btn.png");
	_wnd_close_btn->SetSizer(-1, 10, 10, -1, 13, 13);
	_wnd_close_btn->show();
	_sdk = NULL;
	connect(_device_list, SIGNAL(doubleClick(CListItemBase*)), this, SLOT(HandleDoubleClick(CListItemBase*)));
	connect(_wnd_close_btn, SIGNAL(click()), this, SLOT(HandleCloseClick()));

	if (sdk) {
		_sdk = sdk;
		if (isSpeaker)
		{
			tee3::avd::SpeakersType devicelist;
			sdk->getSpeakers(devicelist);
			for (size_t i = 0; i < devicelist.size(); i++)
			{
				const tee3::avd::Device& item = devicelist[i];
				_device_list->AddDeviceItem(item, item.id == current.id);
			}
		}
		else
		{
			tee3::avd::MicrophonesType devicelist;
			sdk->getMicrophones(devicelist);
			for (size_t i = 0; i < devicelist.size(); i++)
			{
				const tee3::avd::Device& item = devicelist[i];
				_device_list->AddDeviceItem(item, item.id == current.id);
			}
		}
	}
	_isSpeaker = isSpeaker;
}
void CDeviceListView::setCurrent(tee3::avd::Device& current){
	_current = current;
}
void CDeviceListView::resizeEvent(QResizeEvent * event)
{
	_device_list->OnSizer();
	_wnd_close_btn->OnSizer();
	FrameWindow::resizeEvent(event);
}

void CDeviceListView::paintEvent(QPaintEvent *event)
{
	QPainter dc(this);
	// draw User Name
	dc.fillRect(rect(), _back_ground_color);
	dc.setPen(QColor(140, 144, 153));
	QFont font;
	font.setPixelSize(12);
	font.setFamily(QSTRING("Microsoft YaHei"));
	dc.setFont(font);
	QString titleStr = _isSpeaker ? QSTRING("") : QSTRING("˷");
	dc.drawText(QRect(12, 0, rect().width() - 12, 30), Qt::AlignVCenter | Qt::AlignLeft, titleStr);
}

void CDeviceListView::HandleDoubleClick(CListItemBase* item)
{
	CDeviceListItem* myitem = static_cast<CDeviceListItem*>(item);
	_current = myitem->GetDevice();
	accept();
	//close();
}

void CDeviceListView::HandleCloseClick()
{
	reject();
	//close();
}

CDeviceListItem::CDeviceListItem(QWidget* parent, const tee3::avd::Device& item, QColor bg_color, QColor bg_hit_color)
: CListItemBase(parent, bg_color, bg_hit_color), _item(item), m_iscurrent(false)
{
}

void CDeviceListItem::pain()
{
	QPainter dc(this);
	if (_isMouseIn == false)
	{
		dc.fillRect(QRect(0, 0, _rect.width(), _rect.height() - 1), QBrush(_backGroundColor));
	}
	else
	{
		dc.fillRect(QRect(0, 0, _rect.width(), _rect.height() - 1), QBrush(_hitBackGroundColor));
	}
	// draw User Name
	dc.setPen(QColor(140, 144, 153));
	QFont font;
	font.setPixelSize(12);
	font.setFamily(QSTRING("Microsoft YaHei"));
	if (m_iscurrent)
	{
		font.setBold(true);
		dc.setPen(QColor(255, 144, 153));
	}
	dc.setFont(font);
	QString nameStr = QString::fromUtf8(_item.name.c_str());
	dc.drawText(QRect(12, 0, rect().width() - 12, rect().height()), Qt::AlignVCenter | Qt::AlignLeft, nameStr);
}

void CDeviceListItem::setCurrentFlag(bool iscurrent)
{
	m_iscurrent = iscurrent;
}

CDeviceList::CDeviceList(QWidget* parent, QColor bg_color)
: CListBase(parent, bg_color)
{
}
void CDeviceList::AddDeviceItem(const tee3::avd::Device& item, bool isCurrent)
{
	CDeviceListItem* lstItem = new CDeviceListItem(this, item, QColor(68, 68, 68), QColor(68, 68, 68));;
	lstItem->setCurrentFlag(isCurrent);

	PushBack(lstItem);
}
