#include "AudioPage.h"
#include "UserPage.h"
#include "MessageBoxFactory.h"

///
CAudioPage::CAudioPage(RoomWindow* parent) : _room_wnd(parent), _ui_users(parent->GetUserPage()), m_audio(NULL), m_audioDevice(NULL) {
}
CAudioPage::~CAudioPage() {
}

void CAudioPage::RegistAudioHandle(IMAudio* audioHandle) {
	m_audio = audioHandle;
	m_audio->setListener(this);
	m_audioDevice = m_audio->getAudioDevice();
	m_audioDevice->setListener(this);
}

bool CAudioPage::OpenAudio(const UserId& userId) {
	if (m_audio->isSelfUser(userId)) {
		return OpenAudio();
	}
		return RT_SUCCEEDED(m_audio->remotecmdOpenMicrophone(userId));
}
bool CAudioPage::OpenAudio() {
	m_userId = m_audio->getUserManager()->getSelfUserId();
	return RT_SUCCEEDED(m_audio->openMicrophone());
}
bool CAudioPage::CloseAudio() {
	return RT_SUCCEEDED(m_audio->closeMicrophone());
}
bool CAudioPage::CloseAudio(const UserId& userId) {
	if (m_audio->isSelfUser(userId)) {
		return CloseAudio();
	}
		return RT_SUCCEEDED(m_audio->remotecmdCloseMicrophone(userId));
}
void CAudioPage::MonitorAudioLevel(bool isMonitor) {
	if (isMonitor) {
		m_audio->monitorAudioLevel();
	}
	else {
		m_audio->unmonitorAudioLevel();
	}
}
///
void CAudioPage::onMicrophoneStatusNotify(MicrophoneStatus status, const UserId& fromId) {
}
void CAudioPage::onAudioLevelMonitorNotify(const tee3::avd::AudioInfo& info) {
	if (_ui_users) {
		std::vector<std::pair<UserId, int> > userid2level = info.active_streams;
		if (info.input_level > 0) { // add myself in level map
			userid2level.push_back(
				std::make_pair(m_userId, info.input_level));
		}
		if (userid2level.empty()) {
			return;
		}
		_ui_users->UpdateAudioLevels(userid2level);
	}
}
void CAudioPage::onOpenMicrophoneResult(Result result) {
	_room_wnd->SetAudioState(RT_SUCCEEDED(result));
}
void CAudioPage::onCloseMicrophoneResult(Result result) {
	_room_wnd->SetAudioState(!RT_SUCCEEDED(result));
}
///
void CAudioPage::onMicrophonePlugin(const Microphone& item) {
}
void CAudioPage::onMicrophonePlugout(const Microphone& item) {
}
void CAudioPage::onSpeakerPlugin(const Speaker& item) {
}
void CAudioPage::onSpeakerPlugout(const Speaker& item) {
}
void CAudioPage::onMicrophoneVolumeChanged(uint32 volume) {
}
void CAudioPage::onSpeakerVolumeChanged(uint32 volume) {
}

