#include "ChatPage.h"
#include "ChatHistoryBoard.h"
#include "UIManager.h"
#include "Button1.h"
#include "RoomWindow.h"
#include "MainToolWnd.h"
#include "UIDefine.h"

#include <QtGui/qpainter.h>


CChatPage::CChatPage(RoomWindow* parent)
	: CRoundRadiusWnd(parent), _room_wnd(parent)
{
	SetRoundRadius(8);
	_history_board = new CChatHistoryBoard(this, QColor(38, 37, 42, (int)(255 * 0.95)));
	_history_board->SetSizer(1,50,1,50,-1,-1);
	_history_board->show();

	_input_wnd = new CChatInput(this, QColor(38, 37, 42, (int)(255 * 0.95)));
	_input_wnd->SetSizer(1,-1,1,1,-1,50);

	_close_btn = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/close_box_btn.png");
	_close_btn->SetSizer(-1, 11, 10, -1, 13, 13);


	connect(_input_wnd, SIGNAL(sendMsg(QString)), this, SLOT(HandleSendMsg(QString)));
	connect(_close_btn, SIGNAL(click()), this, SLOT(HideWnd()));

}



void CChatPage::paintEvent( QPaintEvent *event )
{
	QPainter dc(this);
	//dc.fillRect(rect(),QBrush(QColor(55,55,55,88)));
	dc.fillRect(rect(), QColor(38, 37, 42, (int)(255 * 0.95)));
	
	//tittle splite
	QBrush brsh = dc.brush();
	dc.setPen(QColor(38, 37, 42));
	QLinearGradient linearGradient(1, 35, 1, 40);
	linearGradient.setColorAt(0, QColor(1, 1, 1));
	linearGradient.setColorAt(1.0, QColor(38, 37, 42));
	dc.setBrush(QBrush(linearGradient));
	dc.drawRect(0, 35, rect().width(), 5);
	dc.setBrush(brsh);

	dc.setPen(QColor(140, 144, 153));
	QFont font = dc.font();
	font.setPixelSize(14);
//	font.setBold(true);
	font.setFamily(QSTRING("Microsoft Yahei"));
	dc.setFont(font);
	dc.drawText(10,23, QSTRING(""));
	//border
	dc.setPen(QColor(1, 1, 1));
	DrawBorder(dc);
}

void CChatPage::resizeEvent( QResizeEvent *event )
{
	CRoundRadiusWnd::resizeEvent(event);
	_history_board->OnSizer();
	_input_wnd->OnSizer();
	_close_btn->OnSizer();
}

CChatPage::~CChatPage()
{

}

void CChatPage::HandleSendMsg( QString msg )
{
	_history_board->AddMineMsg("xx", msg);
	
	_history_board->repaint();

	m_chat->sendPublicMessage( msg.toUtf8().data());
}

void CChatPage::OnReceiveMsg( QString userName, QString msg )
{
	if (!isVisible())
	{
		_msg_num->Add(1);
		_msg_num->repaint();
	}
	_history_board->AddOtherMsg(userName, msg);
	//_history_board->repaint();
}

void CChatPage::HideWnd()
{
	hide();
}

void CChatPage::RegistChatHandle(IMChat* chat)
{
	m_chat = chat;
	m_chat->setListener(this);
}

void CChatPage::onPublicMessage(const tee3::avd::Message& message)
{
	const tee3::avd::User& aUser = m_chat->getUserManager()->getUser(message.fromId);
	if (aUser.isValid())
	{
		_history_board->AddOtherMsg(QString::fromUtf8(aUser.userName.c_str()), QString::fromUtf8(message.message.c_str()));
		if (!isVisible())
			_room_wnd->GetMainToolWnd()->_chat_btn->Add(1);
	}
}

void CChatPage::onPrivateMessage(const tee3::avd::Message& message)
{

}


CChatButton::~CChatButton()
{

}

CChatButton::CChatButton(QWidget *_parent, int x, int y, const char* image_file)
:CButton2(_parent, x, y, image_file), msg_num(0)
{

}

void CChatButton::paintEvent(QPaintEvent *_event)
{
	CButton2::paintEvent(_event);
	if (msg_num >0)
	{
		QPainter dc(this);
		QImage image(":/nice2meet/Resources/image/chat_notice_num_bg.png");
		QRect msgNumRc(rect().width() - image.width(), /*rect().height()-image.height()*/5, image.width(), image.height());
		dc.drawImage(msgNumRc, image, QRect(0, 0, image.width(), image.height()));
		dc.setPen(QColor(226, 192, 165));
		QFont font;
		font.setPixelSize(10);
		font.setFamily(YAHEI);
		dc.setFont(font);
		dc.drawText(msgNumRc, Qt::AlignCenter, QString::number(msg_num));
	}
	
}

void CChatButton::Add(int index)
{
	msg_num += index;
	repaint();
}

int CChatButton::GetMsgNum()
{
	return msg_num;
}

void CChatButton::SetMsgNum(int num)
{
	msg_num = num;
	repaint();
}
