#include "InviteDlg.h"
#include "qlogging.h"
#include "QFormLayout"
#include "MessageBoxFactory.h"
#include "api/musermanager.h"
#include "../roomprofile.h"
#include <QtGui/QDesktopServices>
#include <QtGui/QIcon>
#include <QUrl>

///
CInviteDlg::CInviteDlg(QWidget* parent)
: FrameWindow(parent, Qt::Dialog | Qt::WindowCloseButtonHint) {
	SetRoundRadius(4);
	InitUIStyle();
}
CInviteDlg::~CInviteDlg() {
}

void CInviteDlg::InitUIStyle() {
	ui.setupUi(this);
	QFile stylesheetfile(":/nice2meet/Qss/SettingStyleSheet.qss");
	if (stylesheetfile.open(QFile::ReadOnly)) {
		QString stylesheetstr = QString::fromLatin1(stylesheetfile.readAll());
		setStyleSheet(stylesheetstr);
	}
	else {
		qDebug() << (stylesheetfile.errorString()) << endl;
	}
	ui.lbl_wintitle->setAttribute(Qt::WA_TranslucentBackground);
	QPalette palette = ui.line_title->palette();
	palette.setColor(QPalette::Base, QColor(0, 255, 0));
	palette.setColor(QPalette::WindowText, QColor(0, 255, 0));
	palette.setColor(QPalette::Text, QColor(0, 250, 0));
	ui.line_title->setPalette(palette);
	ui.lbl_roomIdname->setAttribute(Qt::WA_TranslucentBackground);
	ui.lbl_roomId->setAttribute(Qt::WA_TranslucentBackground);
	ui.lbl_url->setAttribute(Qt::WA_TranslucentBackground);
	ui.lbl_joinningUrl->setAttribute(Qt::WA_TranslucentBackground);
	ui.lbl_recordtitle->setAttribute(Qt::WA_TranslucentBackground);
	ui.lbl_optip->setAttribute(Qt::WA_TranslucentBackground);
	QIcon iconclose(QString(":/nice2meet/Resources/image/close_single_normal.png"));
	iconclose.addFile(QString(":/nice2meet/Resources/image/close_single_normal.png"), QSize(13, 13), QIcon::Normal, QIcon::On);
	ui.btn_close->setIcon(iconclose);

	ui.tb_recordLive->setColumnCount(7);
	ui.tb_recordLive->horizontalHeader()->setStretchLastSection(true);
	ui.tb_recordLive->setSelectionBehavior(QAbstractItemView::SelectRows);
	ui.tb_recordLive->horizontalHeader()->setFixedHeight(35);
	ui.tb_recordLive->verticalHeader()->hide();
	QStringList header;
	header << QSTRING(" ") << QSTRING(" ") << QSTRING("״ ̬") << QSTRING(" ") << QSTRING(" ") << QSTRING("rtmp") << QSTRING("hls");
	ui.tb_recordLive->setHorizontalHeaderLabels(header);
	ui.tb_recordLive->hideColumn(5);
	ui.tb_recordLive->hideColumn(6);
	ui.tb_recordLive->setEditTriggers(QAbstractItemView::NoEditTriggers);
	ui.tb_recordLive->show();
	ui.tb_recordLive->setColumnWidth(1, 50);
	ui.tb_recordLive->setColumnWidth(2, 60);
	ui.tb_recordLive->setColumnWidth(3, 100);
	ui.tb_recordLive->setColumnWidth(4, 50);
	ui.tb_recordLive->setColumnWidth(0, ui.tb_recordLive->width() - 270);
	enableLiveRecordOp(false);

	connect(ui.tb_recordLive, SIGNAL(itemDoubleClicked(QTableWidgetItem*)), this, SLOT(playItem(QTableWidgetItem*)));
	connect(ui.tb_recordLive, SIGNAL(itemSelectionChanged()), this, SLOT(changedOperate()));
	connect(ui.btn_close, SIGNAL(clicked()), this, SLOT(hideWnd()));
	connect(ui.btn_copyJoinningLink, SIGNAL(clicked()), this, SLOT(setClipboard()));
	connect(ui.btn_play, SIGNAL(clicked()), this, SLOT(playRecord()));
	connect(ui.btn_downlink, SIGNAL(clicked()), this, SLOT(downloadRecord()));
	connect(ui.btn_copylink, SIGNAL(clicked()), this, SLOT(copylink2Clipboard()));
}
void CInviteDlg::enableLiveRecordOp(bool enable) {
	ui.btn_play->setEnabled(enable);
	ui.btn_downlink->setEnabled(enable);
	ui.btn_copylink->setEnabled(enable);
}
void CInviteDlg::InitUIData() {
	_roomId = CRoomProfile::Instance()->GetRoomID();
	_inviteLink = CRoomProfile::Instance()->GetInviteUrl() + QSTRING("?roomId=") + _roomId;
	ui.lbl_roomId->setText(_roomId);
	ui.lbl_joinningUrl->setText(_inviteLink);

	_live = IAVDLive::getLive(IAVDEngine::Instance());
	_record = IAVDRecord::getRecord(IAVDEngine::Instance());

	clearRecordLiveInfos();
	uint32 callid = 0;
	String filter = "{\"roomId\":\"" + String(_roomId.toUtf8().data()) +"\"}";
	_live->findLiveInfos(callid, 0, 100, filter);
	_record->findRecordInfos(callid, 0, 100, filter);
	String surlbase;
	IAVDEngine::Instance()->getOption(eo_demo_urlbase_liverecord, &surlbase);
	_urlbase = QSTRING("http://")+QString::fromUtf8(surlbase.c_str());
}
///
void CInviteDlg::setVisible(bool visible) {
	if (visible) {
		InitUIData();
	}
	FrameWindow::setVisible(visible);
}
void CInviteDlg::hideWnd() {
	hide();
}
void CInviteDlg::setClipboard() {
	QClipboard *clipboard = QApplication::clipboard();
	clipboard->setText(_inviteLink);
	MessageBoxFactory::Get()->CreateMessageBox(
		MessageBoxFactory::ChildAutoBox, this, QColor(38, 37, 42, (int)(255 * 0.95)), "", QSTRING("ѳɹƵа壡"),
		QRect(10, rect().height() - 50, rect().width() - 20, 50), 5000);
}
void CInviteDlg::clearRecordLiveInfos() {
	while (ui.tb_recordLive->rowCount() > 0) {
		ui.tb_recordLive->removeRow(0);
	}
	ui.tb_recordLive->clearContents();
}
void CInviteDlg::updateRecordInfos(const RecordInfosType& infos) {
	IRoom* room = IRoom::obtain(_roomId.toUtf8().data());
	if (NULL == room) { return; }
	IMUserManager* usrs = IMUserManager::getUserManager(room);
	if (NULL == usrs) { return; }
	int curRow = ui.tb_recordLive->rowCount();
	RecordInfosType::const_iterator it = infos.begin();
	for (; infos.end() != it; ++it) {
		ui.tb_recordLive->insertRow(curRow);
		ui.tb_recordLive->setItem(curRow, 0, new QTableWidgetItem(QString::fromUtf8(it->name.c_str())));
		ui.tb_recordLive->setItem(curRow, 1, new QTableWidgetItem(QSTRING("¼ ")));
		ui.tb_recordLive->setItem(curRow, 2, new QTableWidgetItem(status2String(it->status)));
		ui.tb_recordLive->setItem(curRow, 3, new QTableWidgetItem(QString::fromUtf8(usrs->getUser(it->userId).userName.c_str())));
		ui.tb_recordLive->setItem(curRow, 4, new QTableWidgetItem(QSTRING(" ")));
		ui.tb_recordLive->setItem(curRow, 5, new QTableWidgetItem(QString::fromUtf8(it->url.c_str())));
		ui.tb_recordLive->setItem(curRow, 6, new QTableWidgetItem(QString::fromUtf8(it->url.c_str())));
	}
}
QString CInviteDlg::status2String(RecordStatus status) const {
	static StatusName record[] = {
		{ rs_created, QSTRING("")},
		{ rs_recording, QSTRING("¼") },
		{ rs_availble, QSTRING("") },
		{ rs_removed, QSTRING("ɾ") },
		{ rs_exception, QSTRING("쳣") },
	};
	return record[status].name;
}
QString CInviteDlg::status2String(LiveStatus status) const {
	static StatusName record[] = {
		{ ls_created, QSTRING("") },
		{ ls_living, QSTRING("ֱ") },
		{ ls_stopped, QSTRING("ͣ") },
		{ ls_removed, QSTRING("ɾ") },
		{ ls_exception, QSTRING("쳣") },
	};
	return record[status].name;
}

void CInviteDlg::updateLiveInfos(const LiveInfosType& infos) {
	IRoom* room = IRoom::obtain(_roomId.toUtf8().data());
	if (NULL == room) { return; }
	IMUserManager* usrs = IMUserManager::getUserManager(room);
	if (NULL == usrs) { return; }
	int curRow = ui.tb_recordLive->rowCount();
	LiveInfosType::const_iterator it = infos.begin();
	for (; infos.end() != it; ++it) {
		ui.tb_recordLive->insertRow(curRow);
		ui.tb_recordLive->setItem(curRow, 0, new QTableWidgetItem(QString::fromUtf8(it->name.c_str())));
		ui.tb_recordLive->setItem(curRow, 1, new QTableWidgetItem(QSTRING("ֱ ")));
		ui.tb_recordLive->setItem(curRow, 2, new QTableWidgetItem(status2String(it->status)));
		ui.tb_recordLive->setItem(curRow, 3, new QTableWidgetItem(QString::fromUtf8(usrs->getUser(it->userId).userName.c_str())));
		ui.tb_recordLive->setItem(curRow, 4, new QTableWidgetItem(QSTRING("ά")));
		ui.tb_recordLive->setItem(curRow, 5, new QTableWidgetItem(QString::fromUtf8(it->rtmpurl.c_str())));
		ui.tb_recordLive->setItem(curRow, 6, new QTableWidgetItem(QString::fromUtf8(it->hlsurl.c_str())));
	}
}
void CInviteDlg::changedOperate() {
	bool isLive = false;
	QTableWidgetItem* item = getSelectedItem(isLive);
	if (NULL == item) {
		enableLiveRecordOp(false);
		return;
	}
		enableLiveRecordOp(true);
	if (isLive) {
		ui.btn_play->setText(QSTRING("RTMP"));
		ui.btn_downlink->setText(QSTRING("HLSά"));
		ui.btn_copylink->setText(QSTRING("ֱ"));
	}
	else {
		ui.btn_play->setText(QSTRING(""));
		ui.btn_downlink->setText(QSTRING(""));
		ui.btn_copylink->setText(QSTRING(""));
	}
}
void CInviteDlg::playRecord() {
	playdownloadcopyItem(1);
}
void CInviteDlg::downloadRecord() {
	playdownloadcopyItem(2);
}
void CInviteDlg::copylink2Clipboard() {
	playdownloadcopyItem(3);
}
QTableWidgetItem* CInviteDlg::getSelectedItem(bool& isLive) {
	QList<QTableWidgetItem*> items = ui.tb_recordLive->selectedItems();
	if (items.isEmpty()) {
		return NULL;
	}
	QTableWidgetItem* item = items.front();
	if (items.isEmpty()) {
		return NULL;
	}
	isLive = (QSTRING("ֱ ") == ui.tb_recordLive->item(item->row(), 1)->text());
	return item;
}
void CInviteDlg::playItem(QTableWidgetItem* item) {
	if (NULL == item) {
		return;
	}
	if (4 == item->column()) {
		playdownloadcopyItem(2);
	}
	else {
		playdownloadcopyItem(1);
	}
}

void CInviteDlg::playdownloadcopyItem(int mode) {
	bool isLive = false;
	QTableWidgetItem* item = getSelectedItem(isLive);
	if (NULL == item) {
		return;
	}
	QString	status = ui.tb_recordLive->item(item->row(), 2)->text();
	bool	isErrorStatus = false;
	if (isLive) {
		if (QSTRING("ֱ") != status) {
			isErrorStatus = true;
		}
	}
	else {
		if (QSTRING("") != status) {
			isErrorStatus = true;
		}
	}
	if (isErrorStatus) {
		MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, QColor(38, 37, 42, (int)(255 * 0.95)), "", QSTRING("¼Ч״ֱֻ̬¼ƿ״̬ɲ鿴"),
			QRect(10, rect().height() - 50, rect().width() - 20, 50), 3000);
		return;
	}

	QString url = ui.tb_recordLive->item(item->row(), (2 == mode) ? 6 : 5)->text();
	if (url.isEmpty()) {
		MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, QColor(38, 37, 42, (int)(255 * 0.95)), "", QSTRING("¼ЧUrl"),
			QRect(10, rect().height() - 50, rect().width() - 20, 50), 3000);
		return;
	}
	QString playurl;
	if (isLive) {
		if		(1 == mode) { // play rtmp live
			playurl = _urlbase + QString("/admin/live/rtmpPlayer.html?src=") + url;
		}
		else if (2 == mode) { // play hls live
			playurl = _urlbase + QString("/admin/live/qrcode.html?src=") + _urlbase + QString("/admin/live/hlsPlayer.html?src=") + url;
		}
		else if (3 == mode) {
			url = _urlbase + QString("/admin/live/rtmpPlayer.html?src=") + url;
		}
	}
	else {
		if		(1 == mode) { // play mp4 record
			playurl = _urlbase + QString("/admin/record/recordPlayer.html?src=") + url;
		}
		else if (2 == mode) { // download mp4
			playurl = url;
		}

	}
	if (3 == mode) {
		QClipboard *clipboard = QApplication::clipboard();
		clipboard->setText(url);
		MessageBoxFactory::Get()->CreateMessageBox(
			MessageBoxFactory::ChildAutoBox, this, QColor(38, 37, 42, (int)(255 * 0.95)), "", QSTRING("ѳɹƵа壡"),
			QRect(10, rect().height() - 50, rect().width() - 20, 50), 4000);
	}
	else {
		qDebug() << "playurl=" << playurl;
		QDesktopServices::openUrl(QUrl(playurl, QUrl::StrictMode));
	}
}
