#include <QtGui/QKeyEvent>
#include <QtGui/qpainter.h>
#include <QtWidgets/QRadioButton>
#include "LoginWnd.h"
#include "TextLabel.h"
#include "TextEdit.h"
#include "LineEdit.h"
#include "TextButton1Disable.h"
#include "Button1.h"
#include "RoomWindow.h"
#include "UIDefine.h"
#include "MessageBoxFactory.h"
#include "string"
#include "UIManager.h"
#include "../roomprofile.h"

///
LoginWindow* g_login_wnd = NULL;
LoginWindow::LoginWindow( QWidget* parent /*= NULL*/ ) {
	QRect rc = QApplication::desktop()->availableGeometry();
	SetRoundRadius(8);
	int width = 350;
	int height = 560;
	setMaximumSize(width, height);
	//setMinimumSize(width, height);
	_back_ground_color = QColor(38, 37, 42, 255);
	setGeometry((rc.width()-width)/2, (rc.height()-height)/2, width, height);

	_close_btn = new CButton1(this, -1, -1, ":/nice2meet/Resources/image/close_btn1.png");
	_close_btn->SetSizer(-1, 0, 0, -1, 42, 39);

	_logo_btn = new CButton1(this, -1, -1, CRoomProfile::Instance()->GetLogo().toUtf8().constData());
	_logo_btn->SetSizer((width - 153) / 2, (175 - 82) / 2, -1, -1, 153, 82);

	_default_url = QSTRING("Ļұţ");
	_default_user_name = QSTRING("");
	_default_user_password = QSTRING("루ѡ");

	_user_name_edit = new CLineEdit(1,this, "", QColor(255,255,255));
	_conf_url_edit = new CLineEdit(2, this, "", QColor(255, 255, 255));
	_password_edit = new CLineEdit(3, this, "", QColor(255, 255, 255));
	_user_name_edit->SetRoundRadius(8);
	_conf_url_edit->SetRoundRadius(8);
	_password_edit->SetRoundRadius(8);
	_user_name_edit->SetSizer(22,175,22,-1,-1,42);
	_conf_url_edit->SetSizer(22, 235, 22, -1, -1, 42);
	_password_edit->SetSizer(22, 295, 22, -1, -1, 42);
	//_conf_url_edit->SetSizer(22,240,22,-1,-1,42);
	//_password_edit->SetSizer(22, 305, 22, -1, -1, 42);
	_user_name_edit->SetEmptyText(_default_user_name);
	_conf_url_edit->SetEmptyText(_default_url);
	_password_edit->SetEmptyText(_default_user_password);

	QPalette palette /*= _input_edit->GetEditObj()->palette()*/;
	palette.setColor(QPalette::Base, QColor(71, 71, 71, 255));
	palette.setColor(QPalette::Text, QColor(255, 255, 255));
	_user_name_edit->GetEditObj()->setPalette(palette);
	_user_name_edit->GetEditObj()->setStyleSheet("border-width:0; border-style:outset; background-color:rgb(71,71,71,0); color:rgb(0,0,0)");
	_conf_url_edit->GetEditObj()->setPalette(palette);
	_conf_url_edit->GetEditObj()->setStyleSheet("border-width:0; border-style:outset; background-color:rgb(71,71,71,0); color:rgb(0,0,0)");
	_conf_url_edit->SetText(CRoomProfile::Instance()->GetRoomID());
	_password_edit->GetEditObj()->setPalette(palette);
	_password_edit->GetEditObj()->setStyleSheet("border-width:0; border-style:outset; background-color:rgb(71,71,71,0); color:rgb(0,0,0)");

	_group_mode = new QButtonGroup(this);
	_mode_mcu = new QRadioButton(QSTRING("MCU"), this);
	_mode_p2p = new QRadioButton(QSTRING("P2P"), this);
	_group_mode->addButton(_mode_mcu, rm_mcu);
	_group_mode->addButton(_mode_p2p, rm_p2p);
	QString groupStyle = "color:rgb(169,171,180);background-color:transparent;";
	_mode_mcu->setStyleSheet(groupStyle);
	_mode_p2p->setStyleSheet(groupStyle);
	QPalette paletteradio;
	palette.setColor(QPalette::Base, QColor(0, 255, 0));
	palette.setColor(QPalette::WindowText, QColor(0, 255, 0));
	palette.setColor(QPalette::Text, QColor(0, 250, 0));
	_mode_mcu->setPalette(paletteradio);
	_mode_p2p->setPalette(paletteradio);
	_mode_mcu->setGeometry(75, 335, 75, 42);
	_mode_p2p->setGeometry(200, 335, 75, 42);
	if (CRoomProfile::Instance()->GetRoomMode() == "mcu") {
		_mode_mcu->setChecked(true);
	}
	else {
		_mode_p2p->setChecked(true);
	}
	_join_or_create_conf = new CTextButton1Disable(this, -1, -1, QColor(16, 102, 219),QColor(67,67,67), QSTRING(""));
	_join_or_create_conf->SetRoundRadius(8);
	_join_or_create_conf->SetTextParams(QColor(255, 255, 255), 14, QSTRING("Microsoft YaHei"), false);
	_join_or_create_conf->SetSizer(22, 390, 22, -1, -1, 40);
	_join_or_create_conf->setEnabled(false);

	connect(_join_or_create_conf, SIGNAL(click()), this, SLOT(joinOrCreateConference()));
	connect(_close_btn, SIGNAL(click()), this, SLOT(Close()));
	g_login_wnd = this;

	UIManager::GetUIManager()->InitAVDEngine();
	QPropertyAnimation *animation = new QPropertyAnimation(this, "windowOpacity");
	animation->setDuration(500);
	animation->setStartValue(0);
	animation->setEndValue(1);
	animation->start();
	setWindowIcon(CRoomProfile::Instance()->GetAppIcon());
}

LoginWindow::~LoginWindow() {
	if (_user_name_edit) {
		delete _user_name_edit;
	}
	if (_conf_url_edit) {
		delete _conf_url_edit;
	}
	if (_password_edit) {
		delete _password_edit;
	}
	if (_join_or_create_conf) {
		delete _join_or_create_conf;
	}
	if (_group_mode) {
		delete _group_mode;
	}
	if (_mode_mcu) {
		delete _mode_mcu;
	}
	if (_mode_p2p) {
		delete _mode_p2p;
	}
	if (_close_btn) {
		delete _close_btn;
	}
	if (_logo_btn) {
		delete _logo_btn;
	}
}

void LoginWindow::resizeEvent( QResizeEvent * event ){
	_join_or_create_conf->OnSizer();
	_conf_url_edit->OnSizer();
	_user_name_edit->OnSizer();
	_password_edit->OnSizer();
	_close_btn->OnSizer();
	_logo_btn->SetSizer((rect().width() - 153) / 2, (175 - 82) / 2, -1, -1, 153, 82);
	_logo_btn->OnSizer();
	FrameWindow::resizeEvent(event);
}
void LoginWindow::paintEvent( QPaintEvent *event ) {
	QPainter dc(this);
	dc.fillRect(rect(), QBrush(_back_ground_color));
	//QImage image(":/nice2meet/Resources/image/main_bg.png");
	//dc.drawImage(rect(), image, QRect(0, 0, image.width(), image.height()));
	dc.setPen(QColor(92, 169, 184));
	DrawBorder(dc);
}
void LoginWindow::keyPressEvent( QKeyEvent *event ) {
	if (event->key() == Qt::Key_Return) {
	}
}

void LoginWindow::createConference() {
	tee3::avd::RoomInfo room;
	memset(&room, 0, sizeof(tee3::avd::RoomInfo));
	room.roomName = "TestRoom";

	room.appRoomId = QTime::currentTime().toString().toUtf8().data();
	room.ownerId = _user_name_edit->GetText().toUtf8().data();
	room.hostId = _user_name_edit->GetText().toUtf8().data();
	room.hostPassword = _password_edit->GetText().toUtf8().data();
	room.startTime = 3600;
	room.duration = 3600;
	room.maxAttendee = 10;
	room.maxAudio = 8;
	room.maxVideo = 5;
	if (_mode_mcu->isChecked()) {
		room.setRoomMode(tee3::avd::rm_mcu);
	}
	else {
		room.setRoomMode(tee3::avd::rm_p2p);
	}
	uint32 callId = 0;
	IAVDEngine::Instance()->scheduleRoom(callId, room);
}
void LoginWindow::joinOrCreateConference() {
	if (_user_name_edit->GetText() == "") {
		MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, _back_ground_color,"", QSTRING(""),
			QRect(10, rect().height()-50, rect().width()-20, 50), 5000);
		return;
	}

	if (_conf_url_edit->GetText() == "") {
		QString password = _password_edit->GetText();
		if (password.isEmpty()) {
			MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, _back_ground_color, "", QSTRING("һ飬룡"),
				QRect(10, rect().height() - 50, rect().width() - 20, 50), 5000);
			return;
		}
		else {
			createConference();
		}
	}
	else {
		QString url = _conf_url_edit->GetText();
		if (!url.isEmpty()) {
			joinConference(url);
		}
		else {
			MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, _back_ground_color, "", QSTRING("ȷĻţ"),
				QRect(10, rect().height() - 50, rect().width() - 20, 50), 5000);
			return;
		}
	}
}

void LoginWindow::joinConference( QString room_id ) {
	RoomWindow * roomwnd = UIManager::GetUIManager()->CreateRoomWnd(room_id);
	CRoomProfile::Instance()->SetRoomID(room_id);
	if (NULL != roomwnd) {
		roomwnd->Join(_user_name_edit->GetText(), _password_edit->GetText());
	}
}

void LoginWindow::setEnabled( bool enable ) {
	if (!enable) {
		MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, _back_ground_color, "", QSTRING("ʧȥӣ"),
			QRect(10, rect().height() - 50, rect().width() - 20, 50), 50000);
	}
	_join_or_create_conf->setEnabled(enable);	
}

void LoginWindow::Close() {
	close();
}

void LoginWindow::ArrangeRoomError(int result) {
	MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, _back_ground_color, "", QSTRING("δɹŻ飬ϵͷ"),
		QRect(10, rect().height() - 50, rect().width() - 20, 50), 5000);
}

void LoginWindow::closeEvent(QCloseEvent *event) { 
	UIManager::GetUIManager()->RemoveRoomWnd();
	IAVDEngine::Instance()->release();
	GetApplication()->quit();
}

void LoginWindow::SetRoomID(const char* roomID) {
	_conf_url_edit->SetText(QString::fromUtf8(roomID));
}

LoginWindow* GetLoginWnd() {
	return g_login_wnd;
}
