#include "MessageBoxFactory.h"

#include <QtCore/QTimer>

ChildMessageNoticeBox::~ChildMessageNoticeBox()
{
	if (NULL != _move_icon)
	{
		delete _move_icon;
	}
}



////////////////////////////////////////////////////////////////
ChildMessageNoticeBox::ChildMessageNoticeBox(QWidget* parent, QRect showRectInParent, QColor backColor)
	: CRoundRadiusWnd(parent), _move_icon(NULL), _back_color(backColor)
{
	SetRoundRadius(8);

	/*int width = showRectInParent.width();
	this->resize(width, showRectInParent.height());*/
	//ñ
	this->setWindowFlags(Qt::FramelessWindowHint);
	//ñɫ͸
	QPalette palette;
	//QColor color(190, 230, 250);
	//color.setAlphaF(0.6);
	palette.setBrush(this->backgroundRole(), backColor);
	this->setPalette(palette);
	//QWidgetֱshowбɫģŵһWidgetʱûЧ´Ϳˣ
	this->setAutoFillBackground(true);

	//رհť
	//close_button = new QToolButton(this);
	//QPixmap close_pix = style()->standardPixmap(QStyle::SP_TitleBarCloseButton);
	//close_button->setIcon(close_pix);
	//close_button->setStyleSheet("QToolButton{background-color: transparent;}");

	
	//ʾͼƬ
	msg_label = new QLabel(this);
	msg_label->setStyleSheet("background-color: transparent;");
	msg_label->setScaledContents(true);
	//ʾϢ
	ask_label = new QLabel(this);
	ask_label->setStyleSheet("background-color: transparent; color: red;");
	ask_label->setAlignment(Qt::AlignCenter);
	//close_button->setCursor(Qt::PointingHandCursor);

	show_timer = new QTimer(this);
	show_timer->setSingleShot(true);

	connect(show_timer, SIGNAL(timeout()), this, SLOT(HandleTimeOut()));
	//connect(close_button, SIGNAL(clicked()), this, SLOT(CloseWidget()));
}

void ChildMessageNoticeBox::SetTipInfo(QString info)
{
	//ʾϢ
	ask_label->setText(info);
}

void ChildMessageNoticeBox::SetTipStaticIcon(QPixmap pixmap)
{
	msg_label->setPixmap(pixmap);
}

void ChildMessageNoticeBox::SetTipMoveIcon(const QString iconfile)
{
	_move_icon = new QMovie(this);
	_move_icon->setFileName(iconfile);
	_move_icon->start();
	msg_label->setMovie(_move_icon);
}
//رհťҪʾ
bool ChildMessageNoticeBox::CloseWidget()
{
	this->hide();
	return true;
}

void ChildMessageNoticeBox::HandleTimeOut()
{
	show_timer->stop();
	close();
	this->destroy(true);
	delete this;
}

void ChildMessageNoticeBox::SetDestroyTimer(int time)
{
	show_timer->start(time);
}

void ChildMessageNoticeBox::resizeEvent(QResizeEvent *event)
{
	CRoundRadiusWnd::resizeEvent(event);
	//close_button->setGeometry(width() - 20, 0, 20, 20);
	msg_label->setGeometry(QRect(5, (height()-20)/2, 20, 20));
	ask_label->setGeometry(QRect(30, 0, width() - 60, height()));
}

void ChildMessageNoticeBox::paintEvent(QPaintEvent *event)
{

}

MessageBoxFactory* g_messageBoxFactory = NULL;
MessageBoxFactory* MessageBoxFactory::Get()
{
	if (g_messageBoxFactory == NULL)
	{
		g_messageBoxFactory = new MessageBoxFactory();
	}
	return g_messageBoxFactory;
}

ChildMessageNoticeBox* MessageBoxFactory::CreateChildMessageNoticeBox(MessageBoxType type, QWidget* parent, QColor backColor, const QString tipIcon, const QString content, QRect showRectInParent, int showTime /*= -1*/)
{
	ChildMessageNoticeBox* box = new ChildMessageNoticeBox(parent, showRectInParent, backColor);
	switch (type)
	{
	case MessageBoxFactory::ChildAutoBox:
	{
		box->SetTipStaticIcon(QPixmap(tipIcon));
		break;
	}
	case MessageBoxFactory::ChildAutoBoxWithMoveIcon:
	{
		box->SetTipMoveIcon(tipIcon);
		break;
	}
	default:
		break;
	}
	box->SetSizer(showRectInParent.left(), showRectInParent.top(), -1, -1, showRectInParent.width(), showRectInParent.height());
	box->OnSizer();
	box->SetDestroyTimer(showTime);
	box->show();
	return box;
}

bool MessageBoxFactory::CreateMessageBox(MessageBoxType type, QWidget* parent, QColor backColor, const QString tipIcon, const QString content, QRect showRectInParent, int showTime /*= -1 */)
{
	switch (type)
	{
	case MessageBoxFactory::ChildAutoBox:
	{
		ChildMessageNoticeBox* box = new ChildMessageNoticeBox(parent, showRectInParent, backColor);
		box->SetTipStaticIcon(QPixmap(tipIcon));
		box->SetTipInfo(content);
		box->SetSizer(showRectInParent.left(), showRectInParent.top(), -1, -1, showRectInParent.width(), showRectInParent.height());
		box->OnSizer();
		box->SetDestroyTimer(showTime);
		box->show();
		break;
	}
	case MessageBoxFactory::ChildAutoBoxWithMoveIcon:
	{
		ChildMessageNoticeBox* box = new ChildMessageNoticeBox(parent, showRectInParent, backColor);
		box->SetTipMoveIcon(tipIcon);
		box->SetTipInfo(content);
		box->SetSizer(showRectInParent.left(), showRectInParent.top(), -1, -1, showRectInParent.width(), showRectInParent.height());
		box->OnSizer();
		box->SetDestroyTimer(showTime);
		box->show();
		break;
	}
	default:
		break;
	}
	
	return true;
}

MessageBoxFactory::~MessageBoxFactory()
{

}

MessageBoxFactory::MessageBoxFactory()
{

}
