
#include "RoomWindow.h"
#include "VideoRenderWnd.h"
#include "UIManager.h"
#include "VideoPage.h"
#include "AudioPage.h"
#include "UserList.h"
#include "ChatPage.h"
#include "UserReport.h"
#include "LoginWnd.h"
#include "MainToolWnd.h"
#include "InviteLinkWnd.h"
#include "UserPage.h"
#include "MessageBoxFactory.h"
#include "MessageBox.h"
#include "SettingWindow.h"
#include "InviteDlg.h"

#include "api/room.h"
#include "api/maudio.h"
#include "AudioList.h"
#include "api/avdrecord.h"
#include "../roomprofile.h"

#include <QtGui/qpainter.h>
#include "ScreenPage.h"

RoomWindow::RoomWindow(QWidget* parent /*= NULL*/, const QString roomId)
:FrameWindow(parent), m_room(NULL), _audio_active(false), _chat_page(NULL), _video_page(NULL), _screen_page(NULL), _setting_dlg(NULL) {
	_room_id = roomId;
	SetRoundRadius(8);
	InitUIControl();
	CreatAndRegistRoom(roomId);
}
RoomWindow::RoomWindow(QWidget* parent)
:_audio_active(false), m_room(NULL), _chat_page(NULL), _video_page(NULL), _screen_page(NULL), _setting_dlg(NULL) {
	SetRoundRadius(8);
	InitUIControl();
}

//#define __USE_OLD_INVITE__
void RoomWindow::InitUIControl() {
	QRect rc = QApplication::desktop()->availableGeometry();
	int height = 600;
	int width = 800;
	int x = rc.width() - width - 100;
	int y = (rc.height() - height) / 2;
	setGeometry(QRect(x, y, width, height));
	setMinimumSize(width, height);

	_wnd_close_btn = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/close_btn.png");
	_wnd_close_btn->SetSizer(-1, 10, 10, -1, 13, 13);
	_wnd_close_btn->show();

	_wnd_normal_max = new CButton2M2(this, -1, -1, ":/nice2meet/Resources/image/window_max.png", ":/nice2meet/Resources/image/window_normal.png");
	_wnd_normal_max->SetSizer(-1, 10, 30, -1, 13, 13);

	_wnd_normal_max->show();

	_wnd_mini = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/mini_btn.png");
	_wnd_mini->SetSizer(-1, 10, 50, -1, 13, 13);
	_wnd_mini->show();

	_wnd_setting = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/window_setting.png");
	_wnd_setting->SetSizer(-1, 10, 70, -1, 13, 13);
	_wnd_setting->show();

	_main_btn_tool = new MainToolWnd(this);
	_main_btn_tool->SetSizer(1, -1, 1, 1, -1, 43);

	_video_page = new CVideoPage(this);
	_video_page->SetSizer(1, 35, 1, 45, -1, -1);
	_main_btn_tool->raise();
	_chat_page = new CChatPage(this);
	_chat_page->SetSizer((width - 340) / 2, -1, -1, 120, 340, 480);
	_chat_page->hide();

#ifdef __USE_OLD_INVITE__
	_invite_wnd = new CInviteLinkWnd(this);
	_invite_wnd->SetSizer((width - 400) / 2, -1, -1, 120, 400, 250);
	_invite_wnd->SetRoomId(_room_id);
	_invite_wnd->hide();
	_invite_dlg = NULL;
#else
	_invite_dlg = new CInviteDlg(this);
	_invite_dlg->SetSizer((width - 400) / 2, -1, -1, 120, 500, 400);
	_invite_dlg->hide();
#endif

	_user_page = new CUserPage(this);
	_user_page->SetSizer((width - 310) / 2, -1, -1, 50, 310, 400);
	_user_page->hide();

	_screen_page = new ScreenSharePage(this);
	_screen_page->hide();

	_audio_page = new CAudioPage(this);

	//setting
	//˵˵
	_pop_menu = new QMenu();
	_menu_sel_microphone = new QAction(this);
	_menu_sel_speaker = new QAction(this);
	_menu_switch_camera = new QAction(this);
	_menu_sel_resolution = new QAction(this);

	_menu_sel_microphone->setText(QSTRING("ѡ˷"));
	_menu_sel_speaker->setText(QSTRING("ѡ"));
	_menu_switch_camera->setText(QSTRING("лͷ"));
	_menu_sel_resolution->setText(QSTRING("ѡֱ"));
	//ź
	connect(_menu_sel_microphone, SIGNAL(triggered()), this, SLOT(SelectMicrophone()));
	connect(_menu_sel_speaker, SIGNAL(triggered()), this, SLOT(SelectSpeaker()));
	connect(_menu_switch_camera, SIGNAL(triggered()), this, SLOT(SwitchCamrea()));
	connect(_menu_sel_resolution, SIGNAL(triggered()), this, SLOT(SelectResolution()));
	connect(_wnd_close_btn, SIGNAL(click()), this, SLOT(CloseWnd()));
	connect(_main_btn_tool->_audioBtn, SIGNAL(state2Click()), this, SLOT(OpenAudio()));
	connect(_main_btn_tool->_audioBtn, SIGNAL(state1Click()), this, SLOT(CloseAudio()));
	connect(_main_btn_tool->_videoBtn, SIGNAL(state2Click()), this, SLOT(OpenVideo()));
	connect(_main_btn_tool->_videoBtn, SIGNAL(state1Click()), this, SLOT(CloseVideo()));
	connect(_main_btn_tool->_help_btn, SIGNAL(click()), this, SLOT(HelpPageShow()));
	connect(_main_btn_tool->_screenBtn, SIGNAL(state2Click()), this, SLOT(OpenScreen()));
	connect(_main_btn_tool->_screenBtn, SIGNAL(state1Click()), this, SLOT(CloseScreen()));
	connect(_main_btn_tool->_screenBtn, SIGNAL(mouseIn(bool)), this, SLOT(MouseInScreenBtn(bool)));
	connect(_main_btn_tool->_invite_btn, SIGNAL(click()), this, SLOT(InviteMemberPageShow()));
	connect(_main_btn_tool->_attendee_btn, SIGNAL(click()), this, SLOT(AttendeePageShow()));
	connect(_main_btn_tool->_chat_btn, SIGNAL(click()), this, SLOT(ChatShow()));
	connect(_wnd_mini, SIGNAL(click()), this, SLOT(MiniClick()));
	connect(_wnd_normal_max, SIGNAL(state1Click()), this, SLOT(ShowMax()));
	connect(_wnd_normal_max, SIGNAL(state2Click()), this, SLOT(ShowNormal()));
	connect(_wnd_setting, SIGNAL(click()), this, SLOT(ShowSetting()));

	//Tray icon
	setWindowTitle(QSTRING(" Nice2Meet"));
	setWindowIcon(CRoomProfile::Instance()->GetAppIcon());

#ifdef __USE_TRAYICON__
	_tray_icon = new TrayIcon(this);
	_tray_icon->setIcon(CRoomProfile::Instance()->GetAppIcon());
	_tray_icon->show();
	{
		_tray_menu = new QMenu();

		QFont font;
		font.setPixelSize(12);
#ifdef _MSC_VER
		font.setFamily("Microsoft YaHei");
#endif
		_open_action = new QAction(QSTRING(""), _tray_menu);
		_open_action->setFont(font);
		connect(_open_action, SIGNAL(triggered()), this, SLOT(trayicon_open_main_window()));
		_tray_menu->addAction(_open_action);

		_setting_action = new QAction(QSTRING(""), _tray_menu);
		_setting_action->setFont(font);
		_setting_action->setIcon(QIcon(QString(":/mOffice/Resources/image/icon_setting_16.png")));
		connect(_setting_action, SIGNAL(triggered()), this, SLOT(trayicon_setting()));
		_tray_menu->addAction(_setting_action);

		_change_account_action = new QAction(QSTRING("л˺"), _tray_menu);
		_change_account_action->setFont(font);
		_change_account_action->setIcon(QIcon(QString(":/mOffice/Resources/image/icon_user_16.png")));
		connect(_change_account_action, SIGNAL(triggered()), this, SLOT(trayicon_change_account()));
		_tray_menu->addAction(_change_account_action);

		// add by hang.zhou
		_check_update_action = new QAction(QSTRING(""), _tray_menu);
		_check_update_action->setFont(font);
		_check_update_action->setIcon(QIcon(QString(":/mOffice/Resources/image/icon_update_16.png")));
		connect(_check_update_action, SIGNAL(triggered()), this, SLOT(trayicon_check_update()));
		_tray_menu->addAction(_check_update_action);

		_email_action = new QAction(QSTRING(""), _tray_menu);
		_email_action->setFont(font);
		_email_action->setIcon(QIcon(QString(":/mOffice/Resources/image/icon_message_16.png")));
		connect(_email_action, SIGNAL(triggered()), this, SLOT(trayicon_email()));
		_tray_menu->addAction(_email_action);

		_about_action = new QAction(QSTRING(""), _tray_menu);
		_about_action->setFont(font);
		_about_action->setIcon(QIcon(QString(":/mOffice/Resources/image/icon_about_16.png")));
		connect(_about_action, SIGNAL(triggered()), this, SLOT(trayicon_about()));
		_tray_menu->addAction(_about_action);

		_quit_action = new QAction(QSTRING("˳"), _tray_menu);
		_quit_action->setFont(font);
		_quit_action->setIcon(QIcon(QString(":/mOffice/Resources/image/icon_out_16.png")));
		connect(_quit_action, SIGNAL(triggered()), this, SLOT(trayicon_quit()));
		_tray_menu->addAction(_quit_action);

		_tray_icon->setContextMenu(_tray_menu);
	}
	connect(_tray_icon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(trayicon_activated(QSystemTrayIcon::ActivationReason)));
#endif //__USE_TRAYICON__
}

RoomWindow::~RoomWindow() {
	m_room->release();
	if (_chat_page) {
		delete _chat_page;
		_chat_page = NULL;
	}
	if (_audio_page) {
		delete _audio_page;
		_audio_page = NULL;
	}
	if (_video_page) {
		delete _video_page;
		_video_page = NULL;
	}
	if (_screen_page) {
		delete _screen_page;
		_screen_page = NULL;
	}
}
void RoomWindow::resizeEvent(QResizeEvent * event) {
	_video_page->OnSizer();
	_chat_page->SetSizer((rect().width() - 340) / 2, -1, -1, 50, 340, 480);
	_chat_page->OnSizer();
	_wnd_mini->OnSizer();
	_wnd_close_btn->OnSizer();
	_wnd_normal_max->OnSizer();
	_wnd_setting->OnSizer();
	_main_btn_tool->OnSizer();
#ifdef __USE_OLD_INVITE__
	_invite_wnd->SetSizer((rect().width() - 400) / 2, -1, -1, 120, 400, 250);
	_invite_wnd->OnSizer();
#else
	_invite_dlg->SetSizer((rect().width() - 400) / 2, -1, -1, 120, 500, 400);
	_invite_dlg->OnSizer();
#endif
	_user_page->SetSizer((rect().width() - 310) / 2, -1, -1, 50, 310, 400);
	_user_page->OnSizer();

	FrameWindow::resizeEvent(event);
}

void RoomWindow::paintEvent(QPaintEvent *event) {
	QPainter dc(this);
	//dc.fillRect(rect(), QBrush(QColor(56,59,68)));
	////title
	QRect titleRect(0, 0, rect().width(), 35);
	dc.fillRect(rect(), QBrush(QColor(51, 51, 51)));

	QBrush brsh = dc.brush();
	dc.setPen(QColor(38, 37, 42));
	QLinearGradient linearGradient(1, 30, 1, 35);
	linearGradient.setColorAt(0, QColor(1, 1, 1));
	linearGradient.setColorAt(1.0, QColor(38, 37, 42));
	dc.setBrush(QBrush(linearGradient));
	dc.drawRect(0, 30, rect().width(), 6);
	dc.setBrush(brsh);

	QImage logoImage = CRoomProfile::Instance()->GetTitleBarLogo();
	dc.drawImage(QRect(10, (30 - logoImage.height()) / 2, logoImage.width(), logoImage.height()), logoImage, QRect(0, 0, logoImage.width(), logoImage.height()));

	dc.setPen(QColor(92, 169, 184));
	DrawBorder(dc);

	dc.setPen(QColor(169, 171, 180));
	QFont font;
	font.setPixelSize(12);
	font.setFamily(QSTRING("Microsoft Yahei"));
	dc.setFont(font);
	int roomIdRectLeft = 10 + logoImage.width() + 10;
	dc.drawText(QRect(roomIdRectLeft, titleRect.top(), titleRect.width() - roomIdRectLeft, titleRect.height()), Qt::AlignVCenter | Qt::AlignLeft, QSTRING("ţ") + _room_id);
}

void RoomWindow::FreshMainWnd() {
}

void RoomWindow::ChatShow() {
	if (_chat_page->isVisible()) {
		_chat_page->hide();
	}
	else {
		_chat_page->show();
		_main_btn_tool->_chat_btn->SetMsgNum(0);
	}
}
void RoomWindow::InviteMemberPageShow() {
#ifdef __USE_OLD_INVITE__
	if (_invite_wnd->isVisible()) {
		_invite_wnd->hide();
	}
	else {
		_invite_wnd->show();
	}
#else
	if (_invite_dlg->isVisible()) {
		_invite_dlg->setVisible(false);
	}
	else {
		_invite_dlg->setVisible(true);
		_invite_dlg->move(this->x() + (this->width() - _invite_dlg->width()) / 2, this->y() + (this->height() - _invite_dlg->height()) / 2);
	}
#endif
}
void RoomWindow::HelpPageShow() {
	QString helpurl = CRoomProfile::Instance()->GetRootUrl() + "download/n2m_help_win.pdf";
	QDesktopServices::openUrl(QUrl(helpurl, QUrl::TolerantMode));
}
void RoomWindow::AttendeePageShow() {
	if (_user_page->isVisible()) {
		_audio_page->MonitorAudioLevel(false);
		_user_page->hide();
	}
	else {
		_audio_page->MonitorAudioLevel(true);
		_user_page->show();
	}
}
void RoomWindow::CloseWnd() {
	if (_video_page) {
		_video_page->StopLiveRecord();
	}
	if (_isJoin) {
		if (m_room->leave(0) == 0) {
			onLeaveIndication(0, "");
		}
	}
}
void RoomWindow::SelectMicrophone() {
	tee3::avd::Device item = _audio_page->getAudioDevice()->getRecordingMicrophone();
	if (CDeviceListView::selectCurrentDevice(QCursor::pos(), _audio_page->getAudioDevice(), item, false)) {
		_audio_page->getAudioDevice()->setRecordingMicrophone(item.id);
	}
}
void RoomWindow::SelectSpeaker() {
	tee3::avd::Device item = _audio_page->getAudioDevice()->getPlayoutSpeaker();
	if (CDeviceListView::selectCurrentDevice(QCursor::pos(), _audio_page->getAudioDevice(), item, true)) {
		_audio_page->getAudioDevice()->setPlayoutSpeaker(item.id);
	}
}
void RoomWindow::SelectResolution() {
	throw std::logic_error("The method or operation is not implemented.");
}
void RoomWindow::CreatAndRegistRoom(const QString roomId) {
	m_room = IRoom::obtain(roomId.toUtf8().data());
	m_room->setListener(this);
	_chat_page->RegistChatHandle(IMChat::getChat(m_room));
	IMLivecast* livcast = NULL;
//	if (CRoomProfile::Instance()->HasLivecast()) {
//		livcast = IMLivecast::getLivecast(m_room);
//	}
	_video_page->RegistVideoHandle(IMVideo::getVideo(m_room), livcast);
	_user_page->RegistUserManagerHandle(IMUserManager::getUserManager(m_room));
	_audio_page->RegistAudioHandle(IMAudio::getAudio(m_room));
	_screen_page->RegistScreenHandle(IMScreen::getScreen(m_room));
}

void RoomWindow::OpenVideo() {
	_video_page->OpenVideo();
}
void RoomWindow::CloseVideo() {
	_video_page->CloseVideo();
}
void RoomWindow::CloseAudio() {
	_audio_page->CloseAudio();
}
void RoomWindow::OpenAudio() {
	_audio_page->OpenAudio();
}
IRoom* RoomWindow::GetRoomHandle() {
	return m_room;
}
void RoomWindow::SetVideoState(bool isOpen) {
	if (isOpen) {
		_main_btn_tool->_videoBtn->SetState(CButton2M2::State1);
	}
	else {
		_main_btn_tool->_videoBtn->SetState(CButton2M2::State2);

	}
}
void RoomWindow::SetAudioState(bool isOpen) {
	if (isOpen) {
		_main_btn_tool->_audioBtn->SetState(CButton2M2::State1);
	}
	else {
		_main_btn_tool->_audioBtn->SetState(CButton2M2::State2);
	}
}
void RoomWindow::closeEvent(QCloseEvent *event) {
	CloseWnd();
}
int RoomWindow::Join(QString name, QString password) {
	_self_name = name;
	_conf_pass = password;
	QString userid = CRoomProfile::Instance()->GetUserID();

	User user;
	user.userId = userid.toUtf8().data();
	user.userName = _self_name.toUtf8().data();

	bool usedtls = CRoomProfile::Instance()->IsDtls();
	m_room->setOption(tee3::avd::ro_media_use_dtls, &usedtls);
	m_room->setOption(tee3::avd::ro_audio_option_codec, CRoomProfile::Instance()->GetAudioCodec());
	return m_room->join(user, password.toUtf8().data(), NULL
#ifdef __USE_QINIU__
		//, "FiteyApnqH602g1ndBo9aqIf-OzwQnuRLpXH2ABM:k5_PQfeaoPet9Y3oikPHlJ8CIVI=");
		, "FiteyApnqH602g1ndBo9aqIf-OzwQnuRLpXH2ABM:kS83xRGttNs6tUH93j5mJxnng44=");
#else
		);
#endif
}
void RoomWindow::MiniClick() {
	//hide();
	showMinimized();
}
void RoomWindow::trayicon_activated(QSystemTrayIcon::ActivationReason reason) {
	// ͼ¼
	if (reason == QSystemTrayIcon::Context) {
		/*if (GetMOffice()->IsConnected())
		{
		_open_action->setEnabled(true);
		_setting_action->setEnabled(true);
		_change_account_action->setEnabled(true);
		}
		else
		{
		_open_action->setEnabled(false);
		_setting_action->setEnabled(false);
		_change_account_action->setEnabled(false);
		}*/
	}
	else if (reason == QSystemTrayIcon::Trigger) {
		/*if (!GetMOffice()->IsConnected() && GetMOfficeManager()->IsNeedRelogin())
		{
		ReLoginPopup* relogin_window = GetReLoginWindow();

		QRect rc = _tray_icon->geometry();
		relogin_window->show_popup(rc.x(), rc.y());
		}
		else
		{
		trayicon_open_main_window();
		}*/
	}
	else if (reason == QSystemTrayIcon::DoubleClick) {
		show();
	}
}
void RoomWindow::ShowMax() {
	showMaximized();
	_wnd_normal_max->SetState(CButton2M2::State2);
}
void RoomWindow::ShowNormal() {
	showNormal();
	_wnd_normal_max->SetState(CButton2M2::State1);
}
void RoomWindow::SwitchCamrea() {
	_video_page->SwitchCamera();
}
void RoomWindow::onJoinResult(Result result) {
	if (RT_SUCCEEDED(result)) {
#ifndef DEBUG
		GetLoginWnd()->hide();
#endif // DEBUG
		show();
		_isJoin = true;
		_user_page->AddSelf();
		if (CRoomProfile::Instance()->IsAutoAudio()) {
			OpenAudio();
		}
		if (CRoomProfile::Instance()->IsAutoVideo()) {
			OpenVideo();
		}
		m_room->enableStats(true);
	}
	else {
		if (404 == result) {
			MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, GetLoginWnd(), QColor(20, 0, 20), "", QSTRING("ʧ[ķ]"),
				QRect(10, GetLoginWnd()->rect().height() - 60, GetLoginWnd()->rect().width() - 20, 50), 5000);
			UIManager::GetUIManager()->RemoveRoomWnd(_room_id.toUtf8().data());
		}
	}
}
void RoomWindow::onLeaveIndication(Result reason, const UserId& fromId) {
	if (RT_SUCCEEDED(reason)) {
		_isJoin = false;
		close();
		UIManager::GetUIManager()->RemoveRoomWnd(_room_id);

		GetLoginWnd()->show();
	}
	else if (reason == 1) {
		_isJoin = false;
		close();
		QString roomId = QString::fromUtf8(m_room->getRoomId().c_str());
		//m_room->getAudio()->closeMicrophone(m_room->getUserManager()->getSelf()->nodeID);
		//m_room->getVideo()->closeVideo(m_room->getUserManager()->getSelf()->nodeID, m_room->getVideo()->getVideoDevice()->getLocalCamera(0)->m_id);
		UIManager::GetUIManager()->RemoveRoomWnd(roomId);

		GetLoginWnd()->show();
	}
}

void RoomWindow::onPublicData(const char* data, uint32 len, const UserId& fromId) {
}
void RoomWindow::onPrivateData(const char* data, uint32 len, const UserId& fromId) {
}
void RoomWindow::onAppDataNotify(const String& key, const String& value) {
}
void RoomWindow::onRoomStatusNotify(RoomStatus status) {
}
void RoomWindow::onConnectionStatus(ConnectionStatus status) {
	switch (status) {
	case tee3::avd::cs_ready:
		break;
	case tee3::avd::cs_connecting:
		break;
	case tee3::avd::cs_connected:
		break;
	case tee3::avd::cs_connectFailed:
	{
										CMessageDialog* dialog = new CMessageDialog(this, QSTRING("ʾ"), QSTRING("ýʧܣرջ"), CMessageDialog::Ok);
										dialog->setAttribute(Qt::WA_DeleteOnClose);
										dialog->exec();
										CloseWnd();
										break;
	}
	default:

		break;
	}
}

void RoomWindow::enterEvent(QEvent *) {
	_main_btn_tool->showAnimation();
}
void RoomWindow::leaveEvent(QEvent *) {
	_main_btn_tool->hideAnimation();
}

void RoomWindow::MouseInScreenBtn(bool in) {
	if (in && _screen_page && _screen_page->hasScreenShared()) {
		_screen_page->ShowScreenWnd();
	}
	else if (!in && _screen_page && _screen_page->hasMyScreenShared()) {
		_screen_page->hide();
	}
}

void RoomWindow::OpenScreen() {
	//MediaStats stats = m_room->getMediaStats(_video_page->getLocalVideoId());
	//return;
	if (_screen_page != NULL && _screen_page->hasScreenShared()) {
		_screen_page->ShowScreenWnd();
	}
	else {
		if (_screen_page->creatScreenChooseList(_shared_wnd)) {
			if (!_shared_wnd.id.empty()) {
				if (_screen_page == NULL) {
					_screen_page = new ScreenSharePage(this);
				}
				_screen_page->ShareScreen(_shared_wnd);
			}
		}
	}
}

void RoomWindow::CloseScreen() {
	if (_screen_page) {
		_screen_page->StopShareScreen();
	}
}

void RoomWindow::SetScreenState(bool isOpen) {
	if (isOpen) {
		_main_btn_tool->_screenBtn->SetState(CButton2M2::State1);
	}
	else {
		_main_btn_tool->_screenBtn->SetState(CButton2M2::State2);
	}
}
void RoomWindow::ShowSetting() {
	tee3::avd::IVideoRenderView::ScalingType oldtype = CRoomProfile::Instance()->GetRenderType();

	if (!_setting_dlg) {
		_setting_dlg = new SettingWindow(m_room);
		_setting_dlg->move(this->x() + (this->width() - _setting_dlg->width()) / 2, this->y() + (this->height() - _setting_dlg->height()) / 2);
		connect(_setting_dlg, SIGNAL(videoOptionChanged()), _video_page, SLOT(videoOptionChanged()));
	}

	if (QDialog::Accepted == _setting_dlg->exec()) {
		tee3::avd::IVideoRenderView::ScalingType newType = CRoomProfile::Instance()->GetRenderType();
		qDebug() << "accepted" << endl;
		if (newType != oldtype) {
			if (_video_page) {
				_video_page->UpdateAllRenderScalingType(newType);
			}
			if (_screen_page) {
				_screen_page->UpdateAllRenderScalingType(newType);
			}
		}
	}
	else {
		CRoomProfile::Instance()->SetRenderType(oldtype);
	}
	if (_setting_dlg) {
		delete _setting_dlg;
		_setting_dlg = NULL;
	}
}
bool RoomWindow::event(QEvent *e) {
	if (e->type() == QEvent::KeyPress) {
		QKeyEvent * ke = (QKeyEvent*)e;
		if (ke && ke->key() == Qt::Key_Escape) {
			return true;
		}
	}
	return QWidget::event(e);
}
