#include "ScreenPage.h"
#include "VideoShowWnd.h"
#include "RoomWindow.h"
#include "Button2.h"
#include "MessageBox.h"
#include "VideoLayoutFormat.h"
#include "VideoRenderWnd.h"
#include "MainToolWnd.h"
#include "api/rtclogic.h"
#include "../roomprofile.h"
#include "UIManager.h"

ScreenSharePage::ScreenSharePage(RoomWindow* parent /*= NULL*/, int width /*= 800*/, int height /*= 600*/)
:FrameWindow(NULL), _room_wnd(parent) {
	SetRoundRadius(8);
	setMinimumSize(400, 300);
	_render_layout_wnd = new ScreenLayoutWnd(this, _room_wnd, width, height);
	_render_layout_wnd->SetSizer(5, 40, 5, 10, -1, -1);

	_wnd_close_btn = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/close_btn.png");
	_wnd_close_btn->SetSizer(-1, 10, 10, -1, 13, 13);
	_wnd_close_btn->show();

	_wnd_normal_max = new CButton2M2(this, -1, -1, ":/nice2meet/Resources/image/window_max.png", ":/nice2meet/Resources/image/window_normal.png");
	_wnd_normal_max->SetSizer(-1, 10, 30, -1, 13, 13);
	_wnd_normal_max->show();

	_wnd_mini = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/mini_btn.png");
	_wnd_mini->SetSizer(-1, 10, 50, -1, 13, 13);
	_wnd_mini->show();

	_room_wnd = parent;
	_back_ground_color = QColor(38, 37, 42, 255);

	connect(_wnd_close_btn, SIGNAL(click()), this, SLOT(CloseWnd()));
	connect(_wnd_mini, SIGNAL(click()), this, SLOT(MiniClick()));

	setWindowIcon(CRoomProfile::Instance()->GetAppIcon());

#ifdef __BUILD_USE_FAKECAPTURER__
	_fakecapturer = NULL;
	_timer_fakecapturer = 0;
	_buf_frame = NULL;
	_buf_size = 0;
	_time_stamp_ns = 0;
	_ui_main_thread = QThread::currentThreadId();
	connect(this, SIGNAL(onstart_s()), this, SLOT(OnStart()));
	connect(this, SIGNAL(onstop_s()), this, SLOT(OnStop()));
#endif

}

ScreenSharePage::~ScreenSharePage() {
#ifdef __BUILD_USE_FAKECAPTURER__
	if (_timer_fakecapturer != 0) {
		killTimer(_timer_fakecapturer);
		_timer_fakecapturer = 0;
	}
	if (_fakecapturer) {
		FakeVideoCapturer::Destroy(_fakecapturer);
		_fakecapturer = NULL;
	}
#endif
}

void ScreenSharePage::paintEvent(QPaintEvent *event) {

	QPainter dc(this);
	//dc.fillRect(rect(), QBrush(QColor(56,59,68)));
	////title
	QRect titleRect(0, 0, rect().width(), 35);
	dc.fillRect(rect(), QBrush(QColor(51, 51, 51)));

	QBrush brsh = dc.brush();
	dc.setPen(QColor(38, 37, 42));
	QLinearGradient linearGradient(1, 30, 1, 35);
	linearGradient.setColorAt(0, QColor(1, 1, 1));
	linearGradient.setColorAt(1.0, QColor(38, 37, 42));
	dc.setBrush(QBrush(linearGradient));
	dc.drawRect(0, 30, rect().width(), 6);
	dc.setBrush(brsh);

	dc.setPen(QColor(169, 171, 180));
	QFont font;
	font.setPixelSize(12);
	font.setFamily(QSTRING("Microsoft Yahei"));
	dc.setFont(font);
	int roomIdRectLeft = 10;
	dc.drawText(QRect(roomIdRectLeft, titleRect.top(), titleRect.width() - roomIdRectLeft, titleRect.height()), Qt::AlignVCenter | Qt::AlignLeft, QSTRING(""));

}

void ScreenSharePage::resizeEvent(QResizeEvent *event) {
	_render_layout_wnd->OnSizer();
	_wnd_mini->OnSizer();
	_wnd_close_btn->OnSizer();
	_wnd_normal_max->OnSizer();

	FrameWindow::resizeEvent(event);
}

bool ScreenSharePage::creatScreenChooseList(ScreenWindow& wnd) {

	ScreenShareList* listDialog = new ScreenShareList(m_screen, QColor(38, 37, 42, 255));
	CButton2M2* obj = (CButton2M2*)_room_wnd->_main_btn_tool->_screenBtn;
	QRect objRect1 = obj->rect();
	QRect objRect(obj->mapToGlobal(QPoint(0, 0)), obj->mapToGlobal(QPoint(objRect1.right(), objRect1.bottom())));
	QRect roomWindowRect = _room_wnd->geometry();
	QPoint movePt(objRect.right() - listDialog->width(), roomWindowRect.bottom() - objRect.height() - listDialog->height());
	if (roomWindowRect.width() > listDialog->width() && movePt.x() < roomWindowRect.left()) {
		movePt.setX(objRect.left());
	}
	if (roomWindowRect.height() > listDialog->height() && movePt.y() < roomWindowRect.top()) {
		movePt.setY(objRect.bottom());
	}
	listDialog->move(movePt);
	bool isShareScreen = false;
	int rv = listDialog->exec();
	if (Rejected == rv) {
		isShareScreen = false;
	}
	else if (Accepted == rv) {
		wnd = listDialog->getScreenWnd();
		isShareScreen = true;
	}
	delete listDialog;
	return isShareScreen;
}
void ScreenSharePage::UpdateScreenData(int level, const std::string& description) {
	m_screen->updateScreenData(m_wnd.id, level, description);
}

#ifdef __BUILD_USE_FAKECAPTURER__
bool ScreenSharePage::isMainThread() const {
	return _ui_main_thread == QThread::currentThreadId();
}

bool ScreenSharePage::OnStart() {
	if (!isMainThread())
	{
		emit onstart_s();
		return true;
	}
	_time_stamp_ns = 0;
	_timer_fakecapturer = startTimer(150);
	return true;
}
void ScreenSharePage::OnStop() {
	if (!isMainThread())
	{
		emit onstop_s();
		return;
	}
	if (_timer_fakecapturer != 0) {
		killTimer(_timer_fakecapturer);
		_timer_fakecapturer = 0;
	}
}
void ScreenSharePage::timerEvent(QTimerEvent * event)
{
	if (NULL != _fakecapturer && _fakecapturer->isRunning()) {
		uint32 format = tee3::avd::FOURCC_I420;
		int w = 1920;
		int h = 1080;
		if (_buf_size != tee3::avd::IVideoFrame::SizeOf(w, h)) {
			_buf_size = tee3::avd::IVideoFrame::SizeOf(w, h);
			delete[] _buf_frame;
			_buf_frame = new uint8[_buf_size];
			FillMemory(_buf_frame, w * h, 128);
		}

		static int red = 0;
		if (red > 255) {
			red = 0;
		}
		FillMemory(_buf_frame + w * h, w * h / 4, red);
		FillMemory(_buf_frame + w * h + w * h / 4, w * h / 4, red);
		++red;

		int64 elapsed_time_ns = 0;
		_time_stamp_ns += 55;
		/*rtcapi::IVideoFrame* frame = rtcapi::IVideoFrame::Create(format,
		w, h, w, h, _buf_frame, _buf_size, 1, 1, elapsed_time_ns, _time_stamp_ns, 0);
		if (NULL != frame) {
		_fakecapturer->inputCapturedFrame(*frame);
		}*/
		_fakecapturer->inputCapturedFrame(0, format, w, h, _buf_frame, _buf_size, 0);
	}
}
#endif

bool ScreenSharePage::ShareScreen(ScreenWindow& wnd) {
	//std::string hhe("0001");
	//UserId toId("ios");
	//_room_wnd->GetRoomHandle()->sendPrivateData(hhe.c_str(), hhe.length(), toId);
	//return false;
	if (m_screen != NULL) {
		//CMessageDialog* dialog = new CMessageDialog(this, QSTRING("ʾ"), QSTRING("ģûбء"), CMessageDialog::Ok);
		//dialog->setAttribute(Qt::WA_DeleteOnClose);
		//dialog->exec();
		//delete dialog;
#ifdef __BUILD_USE_FAKECAPTURER__
		m_wnd.id = "screen-fake-id";
		m_wnd.name = "screen_fake_test";
		//m_wnd.publishedQualities.setStreamOptions(tee3::avd::stream_main, tee3::avd::CameraCapability(1280, 720, 15), tee3::avd::codec_vp8);
		//m_wnd.publishedQualities.setStreamOptions(tee3::avd::stream_assist1, tee3::avd::CameraCapability(640, 360, 15), tee3::avd::codec_vp8);
		if (_fakecapturer) {
			FakeVideoCapturer::Destroy(_fakecapturer);
			_fakecapturer = NULL;
		}
		_fakecapturer = FakeVideoCapturer::Create(this,true);
		m_screen->publishScreen(m_wnd, _fakecapturer);
		return true;
#else
		int fps = 25;
		m_screen->getRoom()->setOption(RoomOption::ro_screen_share_fps, &fps);
		RoomWindow* roomwin = UIManager::GetUIManager()->CreateRoomWnd(QString("r1000"));
		if (roomwin) {
			std::vector<WindowId> excludeds;
			excludeds.push_back(WindowId((WindowId::WindowT)(roomwin->winId())));
			m_screen->setExcludedWindows(excludeds);
		}
		wnd.level = 100;
		wnd.description = "haha";
		m_wnd = wnd;
		m_screen->publishScreen(wnd);

		m_screen->updateScreenData(wnd.id, 101, "haha1");
		return true;
#endif
	}
	else {
		CMessageDialog* dialog = new CMessageDialog(this, QSTRING("ʾ"), QSTRING("ģûбء"), CMessageDialog::Ok);
		dialog->setAttribute(Qt::WA_DeleteOnClose);
		dialog->exec();
		return false;
	}
}

bool ScreenSharePage::StopShareScreen() {
	if (m_wnd.status == tee3::avd::ds_published) {
		m_screen->unpublishScreen();
	}
	else {
		hide();
		if (_room_wnd != NULL) {
			_room_wnd->show();
			_room_wnd->activateWindow();
		}

	}
	return true;
}

void ScreenSharePage::AddScreenRender(const UserId& nodeId, const QString deviceId) {
	_render_layout_wnd->AddScreenRender(nodeId, deviceId, "");
}
void ScreenSharePage::UpdateAllRenderScalingType(tee3::avd::IVideoRenderView::ScalingType type) {
	_render_layout_wnd->UpdateAllRenderScalingType(type);
}
void ScreenSharePage::RemoveScreenRender(const UserId& nodeId, const QString deviceId) {
	_render_layout_wnd->RemoveScreenRender(nodeId, deviceId);
}

void ScreenSharePage::MiniClick() {
	hide();
}

void ScreenSharePage::CloseWnd() {
	if (m_wnd.status == tee3::avd::ds_published) {
		CMessageDialog* dialog = new CMessageDialog(this, QSTRING("ʾ"), QSTRING("رոôڽ"));
		dialog->setAttribute(Qt::WA_DeleteOnClose);
		int rv = dialog->exec();
		if (QDialog::Accepted == rv) {
			m_screen->unpublishScreen();
		}
	}
	else {
		hide();
		if (_room_wnd != NULL) {
			_room_wnd->show();
			_room_wnd->activateWindow();
		}

	}

}

bool ScreenSharePage::hasScreenShared() {
	return _render_layout_wnd->hasScreenWnd();
}

void ScreenSharePage::RegistScreenHandle(IMScreen* screen) {
	m_screen = screen;
	m_screen->setListener(this);
}

void ScreenSharePage::onScreenStatusNotify(ScreenStatus status, const DeviceId& fromId) {
}
void ScreenSharePage::onScreenDataNotify(int level, const String& description, const DeviceId& fromId) {
}

void ScreenSharePage::onPublishScreenNotify(const ScreenWindow& screen) {
	if (screen.status == tee3::avd::ds_published) {
		m_screen->subscribe(screen.id);
	}
}

void ScreenSharePage::onUnpublishScreenNotify(const ScreenWindow& screen) {
	if (m_screen->isSelfUser(m_screen->getOwnerId(screen.id))) {

	}
	else if (screen.status == tee3::avd::ds_none || screen.status == tee3::avd::ds_ready) {
		m_screen->unsubscribe(screen.id);
	}

	RemoveScreenRender(m_screen->getOwnerId(screen.id), QString::fromUtf8(screen.id.c_str()));
	if (m_wnd.id == screen.id) {
		m_wnd.status = ds_none;
	}
	if (!hasScreenShared()) {
		hide();
	}
}

void ScreenSharePage::onSubscribeScreenResult(Result result, const DeviceId& fromId) {
	if (RT_SUCCEEDED(result)) {
		if (m_wnd.id == fromId) {
			m_wnd.status = tee3::avd::ds_published;
		}
		AddScreenRender(m_screen->getOwnerId(fromId), QString::fromUtf8(fromId.c_str()));

	}
	if (hasScreenShared()) {
		ShowScreenWnd();
	}
}

void ScreenSharePage::onUnsubscribeScreenResult(Result result, const DeviceId& fromId) {

}

void ScreenSharePage::onPublishScreenResult(Result result, const DeviceId& fromId) {
	if (RT_SUCCEEDED(result)) {
		if (m_wnd.id == fromId) {
			m_wnd.status = tee3::avd::ds_published;
		}
		AddScreenRender(m_screen->getOwnerId(fromId), QString::fromUtf8(fromId.c_str()));
		_room_wnd->SetScreenState(true);
		ShowScreenWnd();
	}
	if (hasScreenShared()) {
		ShowScreenWnd();
		hide();
	}
}

void ScreenSharePage::onUnpublishScreenResult(Result result, const DeviceId& fromId) {
	if (RT_SUCCEEDED(result)) {
		RemoveScreenRender(m_screen->getOwnerId(fromId), QString::fromUtf8(fromId.c_str()));
		if (m_wnd.id == fromId) {
			m_wnd.status = tee3::avd::ds_none;
		}
		_room_wnd->SetScreenState(false);
	}
	if (!hasScreenShared()) {
		hide();
	}
}

bool ScreenSharePage::hasMyScreenShared() {
	if (m_wnd.status == tee3::avd::ds_published) {
		return true;
	}
	return false;
}

void ScreenSharePage::ShowScreenWnd() {
	CButton2M2* obj = (CButton2M2*)_room_wnd->_main_btn_tool->_screenBtn;
	QRect objRect1 = obj->rect();
	QRect objRect(obj->mapToGlobal(QPoint(0, 0)), obj->mapToGlobal(QPoint(objRect1.right(), objRect1.bottom())));
	QRect roomWindowRect = _room_wnd->geometry();
	QPoint movePt(objRect.right() - this->width(), roomWindowRect.bottom() - objRect.height() - this->height());
	if (roomWindowRect.width() > this->width() && movePt.x() < roomWindowRect.left()) {
		movePt.setX(objRect.left());
	}
	if (roomWindowRect.height() > this->height() && movePt.y() < roomWindowRect.top()) {
		movePt.setY(objRect.bottom());
	}
	this->move(movePt);
	this->activateWindow();
	this->show();
}

ScreenShareList::ScreenShareList(IMScreen* screen, QColor backColor) {
	SetRoundRadius(8);
	int width = 350;
	int height = 460;
	setMaximumSize(width, height);
	setMinimumSize(width, height);
	_back_ground_color = backColor;
	_screen_list = new CScreenList(this, _back_ground_color);
	_screen_list->SetSizer(5, 40, 5, 10, -1, -1);
	_screen = screen;
	_wnd_close_btn = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/close_btn.png");
	_wnd_close_btn->SetSizer(-1, 10, 10, -1, 13, 13);
	_wnd_close_btn->show();

	connect(_screen_list, SIGNAL(doubleClick(CListItemBase*)), this, SLOT(HandleDoubleClick(CListItemBase*)));
	connect(_wnd_close_btn, SIGNAL(click()), this, SLOT(HandleCloseClick()));
	tee3::avd::ScreensType screenwndlist;
	screen->getScreenWindows(screenwndlist);

	for (tee3::avd::ScreensType::iterator it = screenwndlist.begin(); it != screenwndlist.end(); it++) {
		_screen_list->AddScreenWndItem(*it);
	}
}

void ScreenShareList::resizeEvent(QResizeEvent * event) {
	_screen_list->OnSizer();
	_wnd_close_btn->OnSizer();
	FrameWindow::resizeEvent(event);
}

void ScreenShareList::paintEvent(QPaintEvent *event) {
	QPainter dc(this);
	// draw User Name
	dc.fillRect(rect(), _back_ground_color);
	dc.setPen(QColor(140, 144, 153));
	QFont font;
	font.setPixelSize(12);
	font.setFamily(QSTRING("Microsoft YaHei"));
	dc.setFont(font);
	QString titleStr = QSTRING("湲б");
	dc.drawText(QRect(12, 0, rect().width() - 12, 30), Qt::AlignVCenter | Qt::AlignLeft, titleStr);
}

void ScreenShareList::HandleDoubleClick(CListItemBase* item) {
	CScreenListItem* screenItem = static_cast<CScreenListItem*>(item);
	_screenWnd = screenItem->getScreenWnd();
	accept();
	//close();
}

void ScreenShareList::HandleCloseClick() {
	reject();
	//close();
}

CScreenListItem::CScreenListItem(QWidget* parent, ScreenWindow& win, QColor bg_color, QColor bg_hit_color)
: CListItemBase(parent, bg_color, bg_hit_color), _screenWnd(win) {

}

void CScreenListItem::pain() {

	QPainter dc(this);
	if (_isMouseIn == false) {
		dc.fillRect(QRect(0, 0, _rect.width(), _rect.height() - 1), QBrush(_backGroundColor));
	}
	else {
		dc.fillRect(QRect(0, 0, _rect.width(), _rect.height() - 1), QBrush(_hitBackGroundColor));
	}
	// draw User Name
	dc.setPen(QColor(140, 144, 153));
	QFont font;
	font.setPixelSize(12);
	font.setFamily(QSTRING("Microsoft YaHei"));
	dc.setFont(font);
	QString nameStr = QString::fromUtf8(_screenWnd.name.c_str());
	dc.drawText(QRect(12, 0, rect().width() - 12, rect().height()), Qt::AlignVCenter | Qt::AlignLeft, nameStr);
}

CScreenList::CScreenList(QWidget* parent, QColor bg_color)
: CListBase(parent, bg_color)

{

}

void CScreenList::AddScreenWndItem(ScreenWindow& win) {
	CScreenListItem* screenWin = new CScreenListItem(this, win, QColor(68, 68, 68), QColor(68, 68, 68));
	PushBack(screenWin);
}



ScreenLayoutWnd::ScreenLayoutWnd(ScreenSharePage* parent, RoomWindow* roomHandle, int width, int height)
: QWidget(parent), _room_wnd(roomHandle), _parent(parent) {

}

void ScreenLayoutWnd::RemoveScreenRender(const UserId& nodeId, const QString deviceId) {
	_parent->GetScreenShareHandle()->detachRender(deviceId.toUtf8().data());

	QString id = deviceId;
	RemoveRenderWnd(id);
	CVideoRenderWnd* render = _user_render_wnds[deviceId];
	_user_render_wnds.erase(deviceId);
	if (render != NULL) {
		delete render;
	}
	Layout();
	if (_user_render_wnds.size() == 0) {
		hide();
	}
}

void ScreenLayoutWnd::AddScreenRender(const UserId& nodeId, const QString deviceId, const QString userName) {
	std::map<QString, CVideoRenderWnd*>::iterator it = _user_render_wnds.find(deviceId);
	if (_user_render_wnds.end() == it) {
		CVideoRenderWnd* render_wnd = new CVideoRenderWnd(deviceId, userName, this);
		//connect(render_wnd, SIGNAL(muteClick(int, QString)), this, SLOT(muteVideo(int, QString)));
		//connect(render_wnd, SIGNAL(unmuteClick(int, QString)), this, SLOT(unmuteVideo(int, QString)));
		render_wnd->show();
		render_wnd->SetScalingType(CRoomProfile::Instance()->GetRenderType());
		if (render_wnd->GetRender() != NULL) {
			_parent->GetScreenShareHandle()->attachRender(deviceId.toUtf8().data(), render_wnd->GetRender());
		}

		if (_user_render_wnds.empty()/* || auser->role == User::host*/) {
			SetCenter(render_wnd);
			render_wnd->setBorder(false);
			render_wnd->setCenter(true);
		}
		_user_render_wnds[deviceId] = render_wnd;
		AddRenderWnd(deviceId, render_wnd);
		Layout();
	}
	if (_user_render_wnds.size() > 0) {
		show();
	}
}

void ScreenLayoutWnd::resizeEvent(QResizeEvent *event) {
	int width = rect().width();
	int height = rect().height();
	if (width != _black_boad_width || height != _black_boad_heigth) {
		_black_boad_heigth = height;
		_black_boad_width = width;
		ChangeSizeNotice(_black_boad_width, _black_boad_heigth);
	}
}

ScreenLayoutWnd::~ScreenLayoutWnd() {
	for (std::map<QString, CVideoRenderWnd*>::iterator i = _user_render_wnds.begin(); i != _user_render_wnds.end();) {
		if (NULL != i->second) {
			_parent->GetScreenShareHandle()->detachRender(i->second->GetRender());
			delete i->second;
			_user_render_wnds.erase(i++);
		}
		else {
			i++;
		}
	}
}

bool ScreenLayoutWnd::hasScreenWnd() {
	return _user_render_wnds.size() != 0;
}

void ScreenLayoutWnd::UpdateAllRenderScalingType(tee3::avd::IVideoRenderView::ScalingType type) {
	for (std::map<QString, CVideoRenderWnd*>::iterator i = _user_render_wnds.begin(); i != _user_render_wnds.end(); i++) {
		if (NULL != i->second) {
			i->second->SetScalingType(type);
		}
	}
}
