#pragma  once

#include "FrameWindow.h"
#include "ListItemBase.h"
#include "ListBase.h"
#include "VideoLayoutFormat.h"

#include "api/mscreen.h"
#include "api/videocapture.h"

using namespace tee3::avd;
class CVideoShowWnd;
class RoomWindow;
class CVideoRenderWnd;
class CVideoLayoutFormatBase;
class CButton2;
class CButton2M2;
class ScreenSharePage;


using tee3::avd::IMScreen;
using tee3::avd::ScreenWindow;
using tee3::avd::User;
using tee3::UserId;
using tee3::DeviceId;
using tee3::avd::FakeVideoCapturer;

class ScreenLayoutWnd : public QWidget, public CVideoOneByOneLayoutFormat, public CWidgetSizer<ScreenLayoutWnd>
{
public:
	ScreenLayoutWnd(ScreenSharePage* parent, RoomWindow* roomHandle, int width, int height);

	~ScreenLayoutWnd();
	void AddScreenRender(const UserId& nodeId, const QString deviceId, const QString userName);
	void RemoveScreenRender(const UserId& nodeId, const QString deviceId);
	void UpdateAllRenderScalingType(tee3::avd::IVideoRenderView::ScalingType type);
	bool hasScreenWnd();

	void resizeEvent(QResizeEvent *event);
private:
	std::map<QString, CVideoRenderWnd*> _user_render_wnds;
	int		_black_boad_width;
	int		_black_boad_heigth;


	RoomWindow* _room_wnd;
	ScreenSharePage* _parent;
};

//#define __BUILD_USE_FAKECAPTURER__

class ScreenSharePage : public FrameWindow, public IMScreen::IListener
#ifdef __BUILD_USE_FAKECAPTURER__
	, public FakeVideoCapturer::Listener
#endif
{
public:
	Q_OBJECT
public:

	ScreenSharePage(RoomWindow* parent = NULL, int width = 800, int height = 600);
	~ScreenSharePage();
	
	bool creatScreenChooseList(ScreenWindow& wnd);

	void paintEvent(QPaintEvent *event);
	void resizeEvent(QResizeEvent * event);

	void RemoveScreenRender(const UserId& nodeId, const QString deviceId);
	void RegistScreenHandle(IMScreen* screen);
	void AddScreenRender(const UserId& nodeId, const QString deviceId);
	void UpdateAllRenderScalingType(tee3::avd::IVideoRenderView::ScalingType type);
	bool hasScreenShared();
	bool hasMyScreenShared();

	bool ShareScreen(ScreenWindow& wnd);
	bool StopShareScreen();
	IMScreen* GetScreenShareHandle(){ return m_screen; }
	void UpdateScreenData(int level, const std::string& description);
	void ShowScreenWnd();
	//

	// notify when publishScreen
	virtual void onScreenStatusNotify(ScreenStatus status, const DeviceId& fromId);
	virtual void onScreenDataNotify(int level, const tee3::String& description, const DeviceId& fromId);
	virtual void onPublishScreenNotify(const ScreenWindow& screen);
	virtual void onUnpublishScreenNotify(const ScreenWindow& screen);
	// result for subscribe
	virtual void onSubscribeScreenResult(Result result, const DeviceId& fromId);
	virtual void onUnsubscribeScreenResult(Result result, const DeviceId& fromId);
	// result for publishScreen
	virtual void onPublishScreenResult(Result result, const DeviceId& fromId);
	virtual void onUnpublishScreenResult(Result result, const DeviceId& fromId);
	protected slots:
	void MiniClick();
	void CloseWnd();
#ifdef __BUILD_USE_FAKECAPTURER__
public:
	virtual bool OnStart();
	virtual void OnStop();
protected:
	void timerEvent(QTimerEvent *event);
	bool isMainThread() const;
signals:
	bool onstart_s();
	void onstop_s();
#endif
private:
	
	ScreenLayoutWnd*						_render_layout_wnd;



	IMScreen* m_screen;
	RoomWindow* _room_wnd;
	QColor _back_ground_color;

	CButton2*	_wnd_close_btn;
	CButton2M2*	_wnd_normal_max;
	CButton2*	_wnd_mini;
	ScreenWindow m_wnd;
private:
#ifdef __BUILD_USE_FAKECAPTURER__
	FakeVideoCapturer* _fakecapturer;
	int _timer_fakecapturer;
	int64 _time_stamp_ns;
	uint8* _buf_frame;
	int _buf_size;
	Qt::HANDLE _ui_main_thread;
#endif
	
};
class CScreenListItem : public CListItemBase
{
public:
	Q_OBJECT
public:
	CScreenListItem(QWidget* parent, ScreenWindow& win, QColor bg_color, QColor bg_hit_color);
	ScreenWindow& getScreenWnd(){ return _screenWnd; }
protected:
	void pain();
protected slots:
private:
	ScreenWindow		_screenWnd;
};

class CScreenList : public CListBase
{
public:
	Q_OBJECT
public:

	CScreenList(QWidget* parent, QColor bg_color);

	void AddScreenWndItem(ScreenWindow& win);
};


class ScreenShareList : public FrameWindow
{
public:
	Q_OBJECT
public:
	ScreenShareList(IMScreen* screen, QColor backColor);
	~ScreenShareList(){};

	void resizeEvent(QResizeEvent * event);
	void paintEvent(QPaintEvent *event);
	ScreenWindow& getScreenWnd(){ return _screenWnd; }
	public slots:
	
	void HandleDoubleClick(CListItemBase* item);
	void HandleCloseClick();
private:
	IMScreen* _screen;
	CScreenList*  _screen_list;
	QColor _back_ground_color;
	CButton2*	_wnd_close_btn;
	ScreenWindow		_screenWnd;

};

