#include "SettingWindow.h"
#include "VideoShowWnd.h"
#include "qlogging.h"
#include "QFormLayout"
#include "..\roomprofile.h"
#include "MessageBoxFactory.h"

///
SettingWindow::SettingWindow(tee3::avd::IRoom* roomHandle) :_roomHandle(roomHandle) {
	InitUIStyle();
	if (roomHandle) {
		_audioHandle = tee3::avd::IMAudio::getAudio(roomHandle);
		if (_audioHandle) {
			_audioDeviceHandle = _audioHandle->getAudioDevice();
		}
		_videoHandle = tee3::avd::IMVideo::getVideo(roomHandle);
		if (_videoHandle) {
			_cameraDeviceHandle = _videoHandle->getCameraDevice();
		}

	}
	LoadSettings();
}

SettingWindow::SettingWindow()
: _roomHandle(NULL)
, _cameraDeviceHandle(NULL)
, _audioDeviceHandle(NULL)
, _audioHandle(NULL)
, _videoHandle(NULL) {
	InitUIStyle();
}

SettingWindow::~SettingWindow() {
	if (_isPreviewed && _videoHandle) {
		_videoHandle->unpreviewLocalCamera(_selectCameraBak.id);
	}
	if (_showWnd) {
		delete _showWnd;
		_showWnd = NULL;
	}
}

void SettingWindow::InitUIStyle() {
	ui.setupUi(this);
	QFile stylesheetfile(":/nice2meet/Qss/SettingStyleSheet.qss");
	if (stylesheetfile.open(QFile::ReadOnly)) {
		QString stylesheetstr = QString::fromLatin1(stylesheetfile.readAll());
		setStyleSheet(stylesheetstr);
	}
	else {
		qDebug() << (stylesheetfile.errorString()) << endl;
	}
	connect(ui.buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
	connect(ui.buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
	connect(ui.selectMicCombox, SIGNAL(activated(const QString &)), this, SLOT(on_selectMicCombox_activated(const QString&)));
	connect(ui.selectSpeakerCombox, SIGNAL(activated(const QString &)), this, SLOT(on_selectSpeakerCombox_activated(const QString&)));
	connect(ui.selectResolutionCombox, SIGNAL(activated(const QString &)), this, SLOT(on_selectResolutionCombox_activated(const QString&)));
	connect(ui.selectRenderModeCombox, SIGNAL(activated(const QString &)), this, SLOT(on_selectRenderModeCombox_activated(const QString&)));

	_showWnd = new CVideoShowWnd("", "previewWnd", ui.videoTab);
	_showWnd->SetScalingType(CRoomProfile::Instance()->GetRenderType());
	QGridLayout* mainLayout = new QGridLayout;
	mainLayout->addWidget(_showWnd);
	ui.videoWnd->setLayout(mainLayout);
	QString groupStyle = "color:rgb(169,171,180);background-color:transparent;";
	ui.gpb_video_codec->setStyleSheet(groupStyle);
	ui.gpb_video_radio->setStyleSheet(groupStyle);
	ui.gpb_video_pub->setStyleSheet(groupStyle);
	ui.gpb_data_dtls->setStyleSheet(groupStyle);
	ui.groupBox->setStyleSheet(groupStyle);
	ui.groupBox_2->setStyleSheet(groupStyle);
	ui.groupBox_3->setStyleSheet(groupStyle);
	ui.ck_auto_video->setStyleSheet(groupStyle);
	ui.ck_auto_audio->setStyleSheet(groupStyle);
	ui.ck_has_livecast->setStyleSheet(groupStyle);
	//ui.videoWnd->addWidget(_showWnd);
	//if (!_videoRenderView) {
	//	_videoRenderView = tee3::avd::IVideoRenderView::CreateVideoRenderView(0, (void*)(_showWnd->winId()), false, false);
	//}
	//if (!_videoRender && _videoRenderView) {
	//	_videoRender = _videoRenderView->CreateVideoRender(0, 0, 0, 0, 0, 0);
	//}
}
QString Resolution2QString(uint32 w, uint32 h, uint32 rate) {
	return QString::number(w) + "*" + QString::number(h) + " " + QString::number(rate);
}

QString Resolution2QString(tee3::avd::CameraCapability& cap) {
	return Resolution2QString(cap.width, cap.height, cap.maxFPS);
}

tee3::avd::CameraCapability ResolutionFromQString(QString resolutionStr) {
	tee3::avd::CameraCapability cap;
	int resolutionSpliterIndex = resolutionStr.indexOf("*", 0);
	if (resolutionSpliterIndex > 0) {
		cap.width = resolutionStr.left(resolutionSpliterIndex).toUInt();
	}
	int rateSpliterIndex = resolutionStr.indexOf(" ");
	if (rateSpliterIndex > 0) {
		cap.height = resolutionStr.mid(resolutionSpliterIndex + 1, rateSpliterIndex - resolutionSpliterIndex - 1).toUInt();
		cap.maxFPS = resolutionStr.mid(rateSpliterIndex + 1).toUInt();
	}
	return cap;
}

QString RenderMode2QString(IVideoRenderView::ScalingType scalingType) {
	QString scalingTypeStr;
	switch (scalingType) {
	case tee3::avd::IVideoRenderView::Scale_Fill:
		//scalingTypeStr = QSTRING("");
		scalingTypeStr = QSTRING("Scale_Fill");
		break;
	case tee3::avd::IVideoRenderView::Scale_Aspect_Fit:
		//scalingTypeStr = QSTRING("Ӧͼ");
		scalingTypeStr = QSTRING("Scale_Aspect_Fit");
		break;
	case tee3::avd::IVideoRenderView::Scale_Aspect_Full:
		//scalingTypeStr = QSTRING("Ӧ");
		scalingTypeStr = QSTRING("Scale_Aspect_Full");
		break;
	default:
		scalingTypeStr = QSTRING("Error");
		break;
	}
	return scalingTypeStr;
}

IVideoRenderView::ScalingType RenderModeFromQString(QString scalingTypeStr) {
	IVideoRenderView::ScalingType scalingType;
	//if (scalingTypeStr == QSTRING("")) {
	if (scalingTypeStr == QSTRING("Scale_Fill")) {
		scalingType = tee3::avd::IVideoRenderView::Scale_Fill;
	}
	//else if (scalingTypeStr == QSTRING("Ӧͼ")) {
	else if (scalingTypeStr == QSTRING("Scale_Aspect_Fit")) {
		scalingType = tee3::avd::IVideoRenderView::Scale_Aspect_Fit;
	}
	//else if (scalingTypeStr == QSTRING("Ӧ")) {
	else if (scalingTypeStr == QSTRING("Scale_Aspect_Full")) {
		scalingType = tee3::avd::IVideoRenderView::Scale_Aspect_Full;
	}
	else {

	}
	return scalingType;
}
void SettingWindow::LoadSettings() {
	if (_audioDeviceHandle) {
		//fill mics
		tee3::avd::MicrophonesType mics;
		_audioDeviceHandle->getMicrophones(mics);
		for (size_t i = 0; i < mics.size(); i++) {
			ui.selectMicCombox->addItem(QString::fromUtf8(mics[i].name.c_str()), QString::fromUtf8(mics[i].id.c_str()));
		}
		if (!_currentMic.isValid()) {
			_currentMic = _audioDeviceHandle->getRecordingMicrophone();
			_selectMicBak = _currentMic;
		}
		ui.selectMicCombox->setCurrentIndex(ui.selectMicCombox->findData(QString::fromUtf8(_currentMic.id.c_str())));
		//fill speak
		tee3::avd::SpeakersType speakers;
		_audioDeviceHandle->getSpeakers(speakers);
		for (size_t i = 0; i < speakers.size(); i++) {
			ui.selectSpeakerCombox->addItem(QString::fromUtf8(speakers[i].name.c_str()), QString::fromUtf8(speakers[i].id.c_str()));
		}
		if (!_currentSpeaker.isValid()) {
			_currentSpeaker = _audioDeviceHandle->getPlayoutSpeaker();
			_selectSpeakerBak = _currentSpeaker;
		}
		ui.selectSpeakerCombox->setCurrentIndex(ui.selectSpeakerCombox->findData(QString::fromUtf8(_currentSpeaker.id.c_str())));
	}

	if (_cameraDeviceHandle) {
		//fill video devices
		tee3::avd::CamerasType cameras;
		_cameraDeviceHandle->getLocalCameras(cameras);
		for (size_t i = 0; i < cameras.size(); i++) {
			ui.selectCameraCombox->addItem(QString::fromUtf8(cameras[i].name.c_str()), QString::fromUtf8(cameras[i].id.c_str()));
		}
		if (!_currentCamera.isValid()) {
			_currentCamera = _cameraDeviceHandle->getDefaultCamera();
			_selectCameraBak = _currentCamera;
		}
		ui.selectCameraCombox->setCurrentIndex(ui.selectCameraCombox->findData(QString::fromUtf8(_currentCamera.id.c_str())));

		if (!_currentCapability.isValid()) {
			tee3::avd::IAVDEngine::Instance()->getOption(tee3::avd::eo_camera_capability_default, &_currentCapability);
			_selectCapabilityBak = _currentCapability;
		}
		LoadResolutions(_currentCamera, _currentCapability);
	}
	on_selectCameraCombox_activated(QString::fromUtf8(_currentCamera.name.c_str()));

	LoadRenderModes();

	if (codec_vp8 == CRoomProfile::Instance()->GetVideoCodec()) {
		ui.opt_video_codec_vp8->setChecked(true);
	}
	else {
		ui.opt_video_codec_h264->setChecked(true);
	}
	if (ratio_4_3 == CRoomProfile::Instance()->GetVideoRatio()) {
		ui.opt_video_ratio_4_3->setChecked(true);
	}
	else {
		ui.opt_video_ratio_16_9->setChecked(true);
	}
	if (CRoomProfile::vp_single == CRoomProfile::Instance()->GetVideoPubType()) {
		ui.opt_video_pub_single->setChecked(true);
	}
	else if (CRoomProfile::vp_2_codec == CRoomProfile::Instance()->GetVideoPubType()) {
		ui.opt_video_pub_2codec->setChecked(true);
	}
	else {
		ui.opt_video_pub_3resolution->setChecked(true);
	}
	if (CRoomProfile::Instance()->IsDtls()) {
		ui.opt_security_yes->setChecked(true);
	}
	else {
		ui.opt_security_no->setChecked(true);
	}
	if (CRoomProfile::Instance()->IsDataTcp()) {
		ui.opt_datachannel_tcp->setChecked(true);
	}
	else {
		ui.opt_datachannel_udp->setChecked(true);
	}
	ui.ck_auto_video->setChecked(CRoomProfile::Instance()->IsAutoVideo());
	ui.ck_auto_audio->setChecked(CRoomProfile::Instance()->IsAutoAudio());
	ui.ck_has_livecast->setChecked(CRoomProfile::Instance()->HasLivecast());

	QString sdkVersion = QString::fromUtf8(IAVDEngine::Instance()->getVersion().c_str());
	ui.lbl_sdk_version->setText(sdkVersion);
}

void SettingWindow::LoadResolutions(const tee3::avd::Camera& camera, tee3::avd::CameraCapability& capability) {
	tee3::avd::CapabilitiesType caps = camera.deviceQualities.getCapabilities();
	ui.selectResolutionCombox->clear();
	for (size_t i = 0; i < caps.size(); i++) {
		ui.selectResolutionCombox->addItem(Resolution2QString(caps[i]), Resolution2QString(caps[i]));
	}
	int index = ui.selectResolutionCombox->findData(Resolution2QString(capability));
	if (index >= 0) {
		ui.selectResolutionCombox->setCurrentIndex(index);
	}
}

void SettingWindow::on_selectMicCombox_activated(const QString &micName) {
	QString deviceId = ui.selectMicCombox->itemData(ui.selectMicCombox->findText(micName)).toString();
	if (_audioDeviceHandle) {
		_selectMicBak = _audioDeviceHandle->getMicrophone(deviceId.toUtf8().data());
		_audioDeviceHandle->setRecordingMicrophone(deviceId.toUtf8().data());
	}

}

void SettingWindow::on_selectSpeakerCombox_activated(const QString &speakerName) {
	QString deviceId = ui.selectSpeakerCombox->itemData(ui.selectSpeakerCombox->findText(speakerName)).toString();
	if (_audioDeviceHandle) {
		_selectSpeakerBak = _audioDeviceHandle->getSpeaker(deviceId.toUtf8().data());
		_audioDeviceHandle->setPlayoutSpeaker(deviceId.toUtf8().data());
	}

}

void SettingWindow::on_selectCameraCombox_activated(const QString &cameraName) {
	QString deviceId = ui.selectCameraCombox->itemData(ui.selectCameraCombox->findText(cameraName)).toString();
	if (_cameraDeviceHandle && _videoHandle) {
		if (_selectCameraBak.isValid() && _selectCameraBak.id != deviceId.toUtf8().data()) {
			_videoHandle->unpreviewLocalCamera(_selectCameraBak.id);
			_showWnd->repaint();
			_isPreviewed = false;
			LoadResolutions(_cameraDeviceHandle->getCamera(deviceId.toUtf8().data()), _currentCapability);
		}
		_videoHandle->previewLocalCamera(deviceId.toUtf8().data(), _showWnd->GetRender());
		_isPreviewed = true;
		_selectCapabilityBak = _currentCapability;
		_selectCameraBak = _cameraDeviceHandle->getCamera(deviceId.toUtf8().data());
	}
}

void SettingWindow::on_selectResolutionCombox_activated(const QString & resolutionStr) {
	tee3::avd::CameraCapability quality = ResolutionFromQString(resolutionStr);
	if (_videoHandle && !_isPreviewed && _selectCameraBak.isValid()) {
		_videoHandle->previewLocalCamera(_selectCameraBak.id, _showWnd->GetRender());
		_isPreviewed = true;
	}

	if (_videoHandle && _isPreviewed && _selectCameraBak.isValid()) {
		_videoHandle->updateCaptureCapability(_selectCameraBak.id, quality);
		_selectCapabilityBak = quality;
	}
}

void SettingWindow::on_selectRenderModeCombox_activated(const QString & renderModeStr) {
	tee3::avd::IVideoRenderView::ScalingType renderType = RenderModeFromQString(renderModeStr);
	CRoomProfile::Instance()->SetRenderType(renderType);
	if (_showWnd) {
		_showWnd->SetScalingType(renderType);
	}
}

void SettingWindow::accept() {
	bool videooptionchanged = false;
	if (_cameraDeviceHandle && _videoHandle) {
		tee3::avd::IAVDEngine::Instance()->setOption(tee3::avd::eo_camera_capability_default, &_selectCapabilityBak);
		_cameraDeviceHandle->setDefaultCamera(_selectCameraBak.id);
		videooptionchanged = true;
	}

	bool bset = CRoomProfile::Instance()->SetVideoCodec(ui.opt_video_codec_vp8->isChecked() ? codec_vp8 : codec_h264);
	if (bset) {
		videooptionchanged = true;
	}

	//bset = CRoomProfile::Instance()->SetVideoRatio(ui.opt_video_ratio_4_3->isChecked() ? ratio_4_3 : ratio_16_9);
	//if (bset) {
	//	videooptionchanged = true;
	//}

	CRoomProfile::VideoPubType pubtype = CRoomProfile::vp_single;
	if (ui.opt_video_pub_2codec->isChecked()) {
		pubtype = CRoomProfile::vp_2_codec;
	}
	else if (ui.opt_video_pub_3resolution->isChecked()) {
		pubtype = CRoomProfile::vp_3_svc;
	}
	bset = CRoomProfile::Instance()->SetVideoPubType(pubtype);
	if (bset) {
		videooptionchanged = true;
	}
	if (videooptionchanged) {
		emit videoOptionChanged();
	}

	CRoomProfile::Instance()->SetDtls(ui.opt_security_yes->isChecked());
	CRoomProfile::Instance()->SetDataTcp(ui.opt_datachannel_tcp->isChecked());
	CRoomProfile::Instance()->SetAutoAudio(ui.ck_auto_audio->isChecked());
	CRoomProfile::Instance()->SetAutoVideo(ui.ck_auto_video->isChecked());
	CRoomProfile::Instance()->SetLivecast(ui.ck_has_livecast->isChecked());
	QDialog::accept();
}

void SettingWindow::reject() {
	if (_audioDeviceHandle) {
		_audioDeviceHandle->setPlayoutSpeaker(_currentSpeaker.id);
		_audioDeviceHandle->setRecordingMicrophone(_currentMic.id);
	}
	if (_videoHandle) {
		_videoHandle->updateCaptureCapability(_currentCamera.id, _currentCapability);
	}
	QDialog::reject();
}

void SettingWindow::LoadRenderModes() {
	ui.selectRenderModeCombox->addItem(RenderMode2QString(tee3::avd::IVideoRenderView::Scale_Fill), RenderMode2QString(tee3::avd::IVideoRenderView::Scale_Fill));
	ui.selectRenderModeCombox->addItem(RenderMode2QString(tee3::avd::IVideoRenderView::Scale_Aspect_Fit), RenderMode2QString(tee3::avd::IVideoRenderView::Scale_Aspect_Fit));
	ui.selectRenderModeCombox->addItem(RenderMode2QString(tee3::avd::IVideoRenderView::Scale_Aspect_Full), RenderMode2QString(tee3::avd::IVideoRenderView::Scale_Aspect_Full));
	int index = ui.selectRenderModeCombox->findData(RenderMode2QString(CRoomProfile::Instance()->GetRenderType()));
	if (index >= 0) {
		ui.selectRenderModeCombox->setCurrentIndex(index);
	}

}
