#include "UIManager.h"
#include "LoginWnd.h"
#include "RoomWindow.h"
#include "../roomprofile.h"

//#define __USE_C_API__
///
UIManager* g_ui_manager = NULL;
UIManager* UIManager::GetUIManager() {
	if (g_ui_manager == NULL) {
		g_ui_manager = new UIManager();
	}
	return g_ui_manager;
}
UIManager::UIManager(): _ui_main_thread(QThread::currentThreadId()) {
}
UIManager::~UIManager() {
	IAVDEngine::Instance()->release();
}
bool UIManager::IsMainThread() {
	return _ui_main_thread == QThread::currentThreadId();
}

RoomWindow* UIManager::CreateRoomWnd(QString room_id) {
	RoomWindow* roomwnd = NULL;
	if (mainWindows.end() == mainWindows.find(room_id)) {
		roomwnd = new RoomWindow(NULL, room_id);
		mainWindows[room_id] = roomwnd;
	}
	else {
		roomwnd = mainWindows.find(room_id)->second;
	}
	return roomwnd;
}
void UIManager::RemoveRoomWnd(QString room_id) {
	RoomWindowsType::iterator it = mainWindows.find(room_id);
	if (mainWindows.end() != mainWindows.find(room_id)) {
		RoomWindow* temp = it->second;
		mainWindows.erase(it);
		delete temp;
	}
}
void UIManager::RemoveRoomWnd() {
	RoomWindowsType::iterator it = mainWindows.begin();
	while (mainWindows.end() != it) {
		RoomWindow* temp = it->second;
		mainWindows.erase(it++);
		delete temp;
	}
}

void UIManager::onInitResult(Result result) {
	Q_ASSERT(IsMainThread());
	GetLoginWnd()->setEnabled(RT_SUCCEEDED(result));

	bool align = true;
	IAVDEngine::Instance()->setOption(eo_video_resolution_16balign, &align);

	//CameraCapability cap(960,720,30);
	//CameraCapability cap(1920, 1080, 30);
	CameraCapability cap(640, 480, 10);
	IAVDEngine::Instance()->setOption(eo_camera_capability_default, &cap);
}

void UIManager::onUninitResult(Result result) {
}

void UIManager::onGetRoomResult(uint32 callId, Result result, const RoomInfo& room) {
}

void UIManager::onFindRoomsResult(uint32 callId, Result result, const RoomInfosType& rooms) {
}
void UIManager::onScheduleRoomResult(uint32 callId, Result result, const RoomId& roomId) {
	Q_ASSERT(IsMainThread());
	if (RT_SUCCEEDED(result)) {
		GetLoginWnd()->joinConference(QString::fromUtf8(roomId.c_str()));
		GetLoginWnd()->SetRoomID(roomId.c_str());
	}
	else {
		GetLoginWnd()->ArrangeRoomError(result);
	}
}

void UIManager::onCancelRoomResult(uint32 callId, Result result, const RoomId& roomId){
}

#ifdef __USE_C_API__
#include "../../api_c/avdengine_c.h"
#include "../../api_c/room_c.h"
#include "../../api_c/maudio_c.h"
#include "../../api_c/mvideo_c.h"
#include "../../api_c/mchat_c.h"

char				g_roomid[] = "r6998";
AVDEngine_Callback	g_cb_engine;
Room_Callback		g_cb_room;
AVDHandle			g_room;
Audio_Callback		g_cb_m_audio;
AVDHandle			g_m_audio;
Video_Callback		g_cb_m_video;
AVDHandle			g_m_video;
AVDHandle			g_m_video_device;
Chat_Callback		g_cb_m_chat;
AVDHandle			g_m_chat;
UserManager_Callback g_cb_users;
AVDHandle			g_m_users;

///
void cb_audio_onMicrophoneStatusNotify(AVDHandle handle, AVDMicrophoneStatus status, const char* deviceId) {

}
void cb_audio_onOpenMicrophoneResult(AVDHandle handle, Result result) {
	if (RT_SUCCEEDED(result)) {

	}
}
void cb_audio_onCloseMicrophoneResult(AVDHandle handle, Result result) {

}
void cb_audio_onAudioLevelMonitorNotify(AVDHandle audioHandle, AVDAudioInfo& info) {
	LOG_INFO_F("info: in=" << info.input_level << ",out=" << info.output_level << ",actives:" << info.ncount);
	for (uint32 i = 0; i < info.ncount; ++i) {
		LOG_INFO_F("item: id=" << info.userids[i] << ",level=" << info.levels[i]);
	}
}

void cb_chat_onPublicMessage(AVDHandle handle, const AVDMessage& message) {
	string name = message.fromName;
	string msg = message.message;

	uint32 ncount = avd_usermanager_getParticipantsCount(g_m_users);
	if (ncount > 0) {
		AVDUser* users = new AVDUser[ncount];
		avd_usermanager_getParticipants(g_m_users, 0, ncount, users);
		delete[] users;
	}

	avd_videodevice_init(g_m_video, NULL, g_m_video_device);
	uint32 ncamera = avd_videodevice_getLocalCameraCount(g_m_video_device);
	for (uint32 i = 0; i < ncamera; ++i) {
		AVDCamera item;
		avd_videodevice_getLocalCamera(g_m_video_device, i, item);

		char buff[Max_UserData] = { 0 };
		uint32 bsize = Max_UserData;
		avd_usermanager_getOwnerId(g_m_users, item.id, buff, bsize);

		bool hhe = avd_usermanager_isSelfDevice(g_m_users, item.id);

		uint32 ncameracaps = avd_videodevice_getCameraCapabilitiesCount(g_m_video_device, item.id);
		AVDCameraCapability* caps = new AVDCameraCapability[ncameracaps];
		avd_videodevice_getCameraCapabilities(g_m_video_device, item.id, ncameracaps, caps);
		delete[] caps;
	}

	ncamera = avd_videodevice_getRemoteCameraCount(g_m_video_device, message.fromId);
	AVDCamera* cameras = new AVDCamera[ncamera];
	avd_videodevice_getRemoteCameras(g_m_video_device, message.fromId, ncamera, cameras);
	for (uint32 i = 0; i < ncamera; ++i) {
		bool hhe = avd_usermanager_isSelfDevice(g_m_users, cameras[i].id);
		if (hhe) {
			break;
		}
	}
	delete[] cameras;
}
void cb_chat_onPrivateMessage(AVDHandle handle, const AVDMessage& message) {
}

///
void cb_room_onJoinResult(AVDHandle room, int result) {
	if (RT_SUCCEEDED(result)) {

		avd_usermanager_init(g_room, &g_cb_users, g_m_users);

		g_cb_m_audio.audio_onMicrophoneStatusNotify = cb_audio_onMicrophoneStatusNotify;
		g_cb_m_audio.audio_onOpenMicrophoneResult = cb_audio_onOpenMicrophoneResult;
		g_cb_m_audio.audio_onCloseMicrophoneResult = cb_audio_onCloseMicrophoneResult;
		g_cb_m_audio.audio_onAudioLevelMonitorNotify = cb_audio_onAudioLevelMonitorNotify;
		avd_audio_init(g_room, &g_cb_m_audio, g_m_audio);

		avd_video_init(g_room, &g_cb_m_video, g_m_video);

		g_cb_m_chat.chat_onPublicMessage = cb_chat_onPublicMessage;
		g_cb_m_chat.chat_onPrivateMessage = cb_chat_onPrivateMessage;
		avd_chat_init(g_room, &g_cb_m_chat, g_m_chat);

		//avd_audio_openMicrophone(g_m_audio);
		string msg("hello i'm c api.");
		avd_chat_sendPublicMessage(g_m_chat, msg.c_str(), msg.size());

		char roomid[Max_UserData] = { 0 };
		uint32 rsize = Max_UserData;
		avd_room_getRoomId(g_room, roomid, rsize);
		rsize = Max_UserData;
		avd_room_getAppRoomId(g_room, roomid, rsize);
		rsize = Max_UserData;
		avd_room_getRoomName(g_room, roomid, rsize);
		rsize = Max_UserData;
		avd_room_getRoomTopic(g_room, roomid, rsize);

		avd_audio_monitorAudioLevel(g_m_audio);
	}
}

///
void cb_engine_onInitResult(int result) {
	if (RT_SUCCEEDED(result)) {
		g_cb_room.room_onJoinResult = cb_room_onJoinResult;

		avd_room_init(g_roomid, "test_ss", &g_cb_room, g_room);
		AVDUser user;
		strcpy_s(user.userId, "test_ss");
		strcpy_s(user.userName, "test_name");
		avd_room_join(g_room, user, NULL, NULL);

		uint32 callid = 111;
		std::string cancelroomid = "r6998";
		avd_engine_cancelRoom(callid, cancelroomid.c_str());

		AVDCameraCapability cap;
		avd_engine_getOption(eo_camera_capability_default, &cap);
	}
}

void cb_engine_onCancelRoomResult(uint32 callId, Result result, const char* roomId) {
	if (RT_SUCCEEDED(result)) {
		int i = 0;
		++i;
	}
}
#endif //__USE_C_API__

bool UIManager::InitAVDEngine() {

#ifdef __USE_C_API__
	g_cb_engine.onInitResult = cb_engine_onInitResult;
	g_cb_engine.onCancelRoomResult = cb_engine_onCancelRoomResult;
	return RT_SUCCEEDED(avd_engine_init(&g_cb_engine, m_siteurl.toUtf8().data(), CRoomProfile::getAppKey().toUtf8().data(), CRoomProfile::getSecretKey().toUtf8().data()));
#endif //__USE_C_API__
#ifdef __USE_QINIU__
	Result ret = IAVDEngine::Instance()->initWithOEM(this,
		CRoomProfile::Instance()->GetSiteUrl().toUtf8().data(), "qiniu.com");
	if (0 == ret) {
		onInitResult(ret);
	}
	return ret;
#else
	return RT_SUCCEEDED(IAVDEngine::Instance()->init(this, 
		CRoomProfile::Instance()->GetSiteUrl().toUtf8().data(), 
		CRoomProfile::getAppKey().toUtf8().data(), CRoomProfile::getSecretKey().toUtf8().data()));
#endif //__USE_C_API__
}
bool UIManager::UninitAEngine() {
	return RT_SUCCEEDED(IAVDEngine::Instance()->uninit());
}
