#include "UserPage.h"
#include "Button2.h"
#include "UserList.h"
#include "VideoPage.h"
#include "AudioPage.h"
#include "api/mchat.h"
#include "api/room.h"

///
CUserPage::CUserPage(RoomWindow* parent)
	: CRoundRadiusWnd(parent), _roomWnd (parent), _userManager(NULL) {
	SetRoundRadius(8);
	_close_btn = new CButton2(this, -1, -1, ":/nice2meet/Resources/image/close_box_btn.png");
	_close_btn->SetSizer(-1, 11, 10, -1, 13, 13);
	_user_list = new CUserList(this, QColor(38,37,42));
	_user_list->SetSizer(1, 70, 1, 15, -1, -1);
	connect(_close_btn, SIGNAL(click()), this, SLOT(hideWnd()));
	connect(_user_list, SIGNAL(clickBtn(CListItemBase*, unsigned int)), this, SLOT(HandleUserOperate(CListItemBase*, unsigned int)));
}
CUserPage::~CUserPage() {
}

void CUserPage::paintEvent(QPaintEvent *event) {
	QPainter dc(this);
	//title
	dc.fillRect(rect(), QColor(38, 37, 42, (int)(255 * 0.95)));
	dc.setPen(QColor(140, 144, 153));
	QFont font = dc.font();
	font.setPixelSize(14);
	font.setFamily(QSTRING("Microsoft YaHei"));
	dc.setFont(font);
	dc.drawText(QRect(10, 0, rect().width(), 35), Qt::AlignLeft | Qt::AlignVCenter, QSTRING("Աб"));

	QBrush brsh = dc.brush();
	dc.setPen(QColor(38, 37, 42));
	QLinearGradient linearGradient(1, 35, 1, 40);
	linearGradient.setColorAt(0, QColor(1,1,1));
	linearGradient.setColorAt(1.0, QColor(38, 37, 42));
	dc.setBrush(QBrush(linearGradient));
	dc.drawRect(0, 35, rect().width(), 5);
	dc.setBrush(brsh);

	//border
	dc.setPen(QColor(19, 19, 19));
	DrawBorder(dc);
	//userListTitle
	QRect userListRect(1, 40, rect().width() - 2, 30);
	dc.fillRect(userListRect, QColor(34, 34, 34));
	dc.drawText(12, 59, QSTRING("λ"));
	if (_userManager) {
		QString userscount = QString::number(_userManager->getParticipantsCount());
		userscount += QSTRING("");
		dc.drawText(150, 59, userscount);
	}
	dc.drawText(236, 59, QSTRING("Ƶ"));
	dc.drawText(276, 59, QSTRING("Ƶ"));
}

void CUserPage::resizeEvent(QResizeEvent *event) {
	_close_btn->OnSizer();
	_user_list->OnSizer();
	CRoundRadiusWnd::resizeEvent(event);
}

void  CUserPage::HandleUserOperate(CListItemBase* item, unsigned int opt) {
	CUserListItem* userItem = static_cast<CUserListItem*>(item);
	const User& aUser = _userManager->getUser(userItem->GetUserId());
	const User& self = _userManager->getSelfUser();
	if (!aUser.isValid() || !self.isValid()) {
		return;
	}
	switch (opt) {
	case OperatorOpenVideo:
		_roomWnd->GetVideoPage()->OpenVideo(aUser.userId);
		break;
	case OperatorCloseVideo:
		_roomWnd->GetVideoPage()->CloseVideo(aUser.userId);
		break;
	case OperatorOpenAudio:
		//_roomWnd->GetRoomHandle()->kickoutUser(0, aUser.userId);
		_roomWnd->GetAudioPage()->OpenAudio(aUser.userId);
		break;
	case OperatorCloseAudio:
		_roomWnd->GetAudioPage()->CloseAudio(aUser.userId);
		break;
	default:
		break;
	}
}

void CUserPage::RegistUserManagerHandle(IMUserManager* userManager) {
	userManager->setListener(this);
	_userManager = userManager;
}

void CUserPage::AddSelf() {
	_user_list->AddUser(_userManager->getSelfUser());
	UsersType users;
	uint32 max = 10000;
	_userManager->getParticipants(0, max, users);
	for (UsersType::iterator it = users.begin(); users.end() != it; it++) {
		_user_list->AddUser(*it);
	}
}

void CUserPage::RequestOpenVideo(const QString userId, const QString deviceId) {
	CUserListItem * item = static_cast<CUserListItem *>(_user_list->GetItemByItemId(userId));
	if (NULL == item) {
		return;
	}
	item->ShowRequestOpenVideoBtn(20*1000);
}
void CUserPage::RequestOpenAudio(const QString userId) {
	CUserListItem * item = static_cast<CUserListItem *>(_user_list->GetItemByItemId(userId));
	if (NULL == item) {
		return;
	}
	item->ShowRequestOpenAudioBtn(20 * 1000);
}

void CUserPage::hideWnd() {
	hide();
}

bool CUserPage::GetUserID(vector<int>& ids) {
	for (int i = 0; i < _user_list->GetSize(); i++) {
		CListItemBase * item = _user_list->GetItemByIndex(i);
		ids.push_back(item->GetId());
	}
	return true;
}
void  CUserPage::UpdateAudioLevels(std::vector<std::pair<UserId, int> >& id2levels) {
	for (int i = 0; i < _user_list->GetSize(); i++) {
		CUserListItem * item = static_cast<CUserListItem *>(_user_list->GetItemByIndex(i));
		if (NULL == item) {
			continue;
		}
		if (id2levels.empty()) {
			break;
		}
		int level = 0;
		std::vector<std::pair<UserId, int> >::iterator it = id2levels.begin();
		for (; id2levels.end() != it; ++it) {
			if (item->GetUserId() == it->first) {
				level = it->second;
				id2levels.erase(it);
				break;
			}
		}
		if (level > 0) {
			item->SetAudioLevel(level);
		}
	}
}
void CUserPage::onUserJoinNotify(const User& user) {
	_user_list->AddUser(user);
	repaint();
}
void CUserPage::onUserLeaveNotify(const User& user) {
	_user_list->DeleteUser(user.nodeId);
	_roomWnd->GetVideoPage()->RemoveVideoRender(user);
}
void CUserPage::onUserUpdateNotify(const User& user){
	_user_list->UpdateUser(user);
}
void CUserPage::onUserStatusNotify(UserStatus status, const UserId& fromId) {
	User u = _userManager->getUser(fromId);
	_user_list->UpdateUser(u);
}
void CUserPage::onUserDataNotify(const String& userData, const UserId& fromId) {
}

