#include "VideoLayoutFormat.h"
#include "VideoRenderWnd.h"
#include "Button1.h"
#include <QtCore/qmath.h>

///
CVideoLayoutFormatBase::CVideoLayoutFormatBase( int width, int heigth )
	: _heigth(heigth), _width(width), _center_wnd(NULL){
}
CVideoLayoutFormatBase::CVideoLayoutFormatBase()
	: _heigth(0), _width(0),_center_wnd(NULL){
}
CVideoLayoutFormatBase::CVideoLayoutFormatBase( const CVideoLayoutFormatBase& aRight ) {
	*this = aRight;
}
CVideoLayoutFormatBase::~CVideoLayoutFormatBase(){
}
const CVideoLayoutFormatBase& CVideoLayoutFormatBase::operator=( const CVideoLayoutFormatBase& aRight ){
	if (this != &aRight){
		_heigth = aRight._heigth;
		_width = aRight._width;
		_wnd_list = aRight._wnd_list;
		_center_wnd_id = aRight._center_wnd_id;
		_center_wnd = aRight._center_wnd;
	}
	return *this;
}
//
void CVideoLayoutFormatBase::AddRenderWnd( QString id, CVideoRenderWnd* render_wnd ) {
	_wnd_list.push_back(render_wnd);
	connect(render_wnd, SIGNAL(doubleClick()), this, SLOT(ChangeCenterVideo()));
}
void CVideoLayoutFormatBase::SetCenter( const QString id ) {
	VideoViewsType::iterator it = _wnd_list.begin();
	for (; it != _wnd_list.end(); it++) {
		if ((*it)->getId() == id) {
			SetCenter(*it);
			break;
		}
	}
}
void CVideoLayoutFormatBase::SetCenter( CVideoRenderWnd* render_wnd ) {
	_center_wnd_id = render_wnd->getId();
	_center_wnd = render_wnd;
}
void CVideoLayoutFormatBase::ChangeSizeNotice( int width, int heigth ) {
	_width = width;
	_heigth = heigth;
	Layout();
}
void CVideoLayoutFormatBase::RemoveRenderWnd(QString id) {
	VideoViewsType::iterator it = _wnd_list.begin();
	for (; it != _wnd_list.end(); it++) {
		if ((*it)->getId() == id) {
			_wnd_list.erase(it);
			break;
		}
	}
	if (_center_wnd_id == id && _wnd_list.size() > 0) {
		SetCenter(_wnd_list.front());
	}
	if (_wnd_list.empty()) {
		_center_wnd_id = "";
		_center_wnd = NULL;
	}
}
void CVideoLayoutFormatBase::ChangeCenterVideo() {
	CVideoRenderWnd* nextCenterWnd = static_cast<CVideoRenderWnd*>(QObject::sender());
	if (nextCenterWnd == NULL || nextCenterWnd == _center_wnd) {
		return;
	}
	_center_wnd->setBorder(true);
	_center_wnd->setCenter(false);
	nextCenterWnd->setBorder(false);
	nextCenterWnd->setCenter(true);
	_center_wnd = nextCenterWnd;
	_center_wnd_id = nextCenterWnd->getId();
	Layout();
}
///
CVideoOneByOneLayoutFormat::CVideoOneByOneLayoutFormat()
	: CVideoLayoutFormatBase(0,0) {
}
CVideoOneByOneLayoutFormat::CVideoOneByOneLayoutFormat( int width, int heigth )
	: CVideoLayoutFormatBase(width, heigth) {
}
CVideoOneByOneLayoutFormat::~CVideoOneByOneLayoutFormat() {
}
void CVideoOneByOneLayoutFormat::Layout() {
	int size = _wnd_list.size();
	int size_sqrt = (int)qSqrt(size);
	int width_num = (size_sqrt==size)?1:size_sqrt+1;
	int heigth_num = size/width_num + (size%width_num?1:0);
	int item_width = _width/width_num;
	int item_heigth = heigth_num?_heigth/heigth_num:0;
	VideoViewsType::iterator it = _wnd_list.begin();
	for (int cout = 0; it != _wnd_list.end(); it++, cout++) {
		if (!(*it)->isFullScreen()) {
			(*it)->SetSizer(cout%width_num * item_width, cout / width_num * item_heigth, -1, -1, item_width, item_heigth);
			(*it)->OnSizer();
		}
	}
}
///
CVideoOneCenterLayoutFormat::~CVideoOneCenterLayoutFormat() {
}
CVideoOneCenterLayoutFormat::CVideoOneCenterLayoutFormat() : _current_page_num(1), _max_page_num(0) {
	_arrow_right_btn = new CButton1(NULL, -1, -1, ":/nice2meet/Resources/image/arrow_right.png");
	_arrow_right_btn->SetSizer(-1, -1, 10, 63, 31, 53);
	_arrow_right_btn->hide();
	_arrow_left_btn = new CButton1(NULL, -1, -1, ":/nice2meet/Resources/image/arrow_left.png");
	_arrow_left_btn->SetSizer(10, -1, -1, 63, 31, 53);
	_arrow_left_btn->hide();
	connect(_arrow_left_btn, SIGNAL(click()), this, SLOT(LeftPage()));
	connect(_arrow_right_btn, SIGNAL(click()), this, SLOT(RightPage()));
}
CVideoOneCenterLayoutFormat::CVideoOneCenterLayoutFormat(int width, int heigth) 
	: CVideoLayoutFormatBase(width, heigth),_current_page_num(1), _max_page_num(0) {
	_arrow_right_btn = new CButton1(NULL, -1, -1, ":/nice2meet/Resources/image/arrow_right.png");
	_arrow_right_btn->SetSizer(-1, -1, 10, 63, 31 , 53);
	_arrow_right_btn->hide();
	_arrow_left_btn = new CButton1(NULL, -1, -1, ":/nice2meet/Resources/image/arrow_left.png");
	_arrow_left_btn->SetSizer(10, -1, -1, 63, 31, 53);
	_arrow_left_btn->hide();
	connect(_arrow_left_btn, SIGNAL(click()), this, SLOT(LeftPage()));
	connect(_arrow_right_btn, SIGNAL(click()), this, SLOT(RightPage()));
}
void CVideoOneCenterLayoutFormat::Layout() {
	int size = _wnd_list.size();
	if (_width == 0 || _heigth == 0 || size == 0) {
		return;
	}
	int bottomWith = _width - 100;
	int itemWidth = 200;
	int itemHeigth = 105;
	int numPeerPage = bottomWith / (itemWidth + 10);
	if (numPeerPage == 0) {
		return;
	}
	int miniVideoSize = size - 1;
	_max_page_num = GetMiniVideoPage(miniVideoSize, numPeerPage);
	if (_current_page_num>_max_page_num) {
		_current_page_num = ((_max_page_num==0)? 1: _max_page_num);
	}
	if (_center_wnd != NULL) {
		_arrow_left_btn->setParent(static_cast<QWidget*>(_center_wnd->parent()));
		_arrow_right_btn->setParent(static_cast<QWidget*>(_center_wnd->parent()));
		_arrow_right_btn->OnSizer();
		_arrow_left_btn->OnSizer();
	}
	if (_current_page_num > 1) {
		_arrow_left_btn->show();
		_arrow_left_btn->raise();
	}
	else {
		_arrow_left_btn->hide();
	}
	if (_current_page_num < _max_page_num) {
		_arrow_right_btn->raise();
		_arrow_right_btn->show();
	}
	else {
		_arrow_right_btn->hide();
	}
	VideoViewsType::iterator it = _wnd_list.begin();
	for (int cout = 0; it != _wnd_list.end(); it++) {
		if ((*it)->getId() == _center_wnd_id ) {
			if (_wnd_list.size() == 1) {
				(*it)->SetSizer(0, 0, 0, 0, -1, -1);
			}
			else {
				(*it)->SetSizer(0, 115, 0, 0, -1, -1);
			}
			if (!(*it)->isFullScreen()) {
				(*it)->lower();
				(*it)->OnSizer();
			} 
		}
		else {
			if (GetMiniVideoPage(cout + 1, numPeerPage) == _current_page_num) {
				(*it)->show();
				(*it)->raise();
			}
			else {
				(*it)->hide();
			}
			int rightSize = 0;
			if (GetMiniVideoPage(cout + 1, numPeerPage) == _max_page_num) {
				int numOfLastPage = (miniVideoSize%numPeerPage) + ((miniVideoSize%numPeerPage == 0) ? numPeerPage : 0);
				rightSize = (bottomWith - (numOfLastPage)*(itemWidth + 10)) / 2 + (cout%numPeerPage)*(itemWidth + 10) + 50;
			}
			else {
				rightSize = (bottomWith - numPeerPage*(itemWidth + 10)) / 2 + (cout%numPeerPage)*(itemWidth + 10) + 50;
			}
			(*it)->SetSizer(rightSize, 5, -1, -1, itemWidth, itemHeigth);
			(*it)->OnSizer();
			cout++;
		}
	}
}
int CVideoOneCenterLayoutFormat::GetMiniVideoPage(int miniVideoIndex, int numPeerPage){
	return miniVideoIndex / numPeerPage + ((miniVideoIndex%numPeerPage) > 0 ? 1 : 0);
}
void CVideoOneCenterLayoutFormat::RightPage(){
	if (_current_page_num < _max_page_num){
		++_current_page_num;
		Layout();
	}
}
void CVideoOneCenterLayoutFormat::LeftPage() {
	if (1<_current_page_num) {
		--_current_page_num;
		Layout();
	}
}
void CVideoOneCenterLayoutFormat::SetCenter(CVideoRenderWnd* render_wnd) {
	if (_center_wnd&& _center_wnd != render_wnd) {
		_center_wnd->setBorder(true);
		_center_wnd->setCenter(false);
	}
	if (render_wnd) {
		render_wnd->setBorder(false);
		render_wnd->setCenter(true);
	}
	return CVideoLayoutFormatBase::SetCenter(render_wnd);
}
