#ifndef __OPEN_AVD_N2M_VideoPage_H__
#define __OPEN_AVD_N2M_VideoPage_H__

#include "WidgetSizer.h"
#include "api/mvideo.h"
#include "api/videocapture.h"
#include "api/avdrecord.h"
#include "api/avdlive.h"
#include "api/mlivecast.h"
//#define __BUILD_USE_FAKECAPTURER__

using namespace tee3;
using namespace avd;
class CVideoRenderWnd;
class CVideoLayoutFormatBase;
class RoomWindow;

class CVideoPage : public QWidget, public CWidgetSizer<CVideoPage>
	, public IMVideo::IListener, public ICameraDevice::IListener
	, public IAVDRecord::IListener, public IAVDLive::IListener, public IMLivecast::IListener
#ifdef  __BUILD_USE_FAKECAPTURER__
	, public FakeVideoCapturer::Listener
#endif//__BUILD_USE_FAKECAPTURER__
{
public:
	Q_OBJECT
public:
	 CVideoPage(RoomWindow* parent);
	 CVideoPage(RoomWindow* parent, int width, int height);
	~CVideoPage();
public:
	void RegistVideoHandle(IMVideo* video, IMLivecast* livecast);
	void AddVideoRender(const UserId& userId, const DeviceId& deviceId);
	void RemoveVideoRender(const User& user);
	void RemoveVideoRender(const UserId& userId, const DeviceId& deviceId);
	void UpdateAllRenderScalingType(tee3::avd::IVideoRenderView::ScalingType type);
	bool OpenVideo(const UserId& userId);
	bool OpenVideo();
	bool CloseVideo(const UserId& userId);
	bool CloseVideo();
	bool SwitchCamera();
	void UpdateCameraData(int level, const std::string& description);
	void StopLiveRecord();
	//
	void paintEvent(QPaintEvent *event);
	void resizeEvent(QResizeEvent * event);

	DeviceId getLocalVideoId() const { return _localVideoId;  }
public:
	// IMVideo::IListener imp
	virtual void onCameraStatusNotify(CameraStatus status, const DeviceId& fromId);
	virtual void onCameraDataNotify(int level, const String& description, const DeviceId& fromId);
	virtual void onPublishCameraNotify(const Camera& camera);
	virtual void onUnpublishCameraNotify(const Camera& camera);
	virtual void onSubscribeResult(Result result, const DeviceId& fromId) ;
	virtual void onUnsubscribeResult(Result result, const DeviceId& fromId) ;
	virtual void onPublishLocalResult(Result result, const DeviceId& fromId) ;
	virtual void onUnpublishLocalResult(Result result, const DeviceId& fromId) ;
	// ICameraDevice::IListener imp
	virtual void onCameraPlugin(const Camera& item);
	virtual void onCameraPlugout(const Camera& item);
	/// IAVDRecord::listener imp
	void onWorkingRooms(uint32 callId, Result result, uint32 total, uint32 begin, const RoomInfosType& items);
	void onPublishedVideos(uint32 callId, Result result, const RoomId& roomId, const VideoDevicesType& items);
	void onCreateRecord(uint32 callId, Result result, const RecordInfo& info);
	void onSelectMedia4Record(uint32 callId, Result result, const RecordId& recordId, const DeviceId& selectedVideo);
	void onCreateUserRecord(uint32 callId, Result result, const RecordInfo& info);
	void onStopRecord(uint32 callId, Result result, const RecordId& recordId);
	void onRecordInfo(uint32 callId, Result result, const RecordInfo& info);
	void onRecordInfos(uint32 callId, Result result, uint32 total, uint32 begin, const RecordInfosType& items);
	void onDeleteRecord(uint32 callId, Result result, const RecordId& recordId);
	/// IAVDLive::listener imp
	void onCreateLive(uint32 callId, Result result, const LiveInfo& info);
	void onSelectMedia4Live(uint32 callId, Result result, const LiveId& recordId, const DeviceId& selectedVideo);
	void onCreateUserLive(uint32 callId, Result result, const LiveInfo& info);
	void onStopLive(uint32 callId, Result result, const LiveId& recordId);
	void onLiveInfo(uint32 callId, Result result, const LiveInfo& info);
	void onLiveInfos(uint32 callId, Result result, uint32 total, uint32 begin, const LiveInfosType& items);
	void onDeleteLive(uint32 callId, Result result, const LiveId& recordId);
	/// IMLivecast::IListener
	void onLivecastStatus(LivecastStatus status);
protected:
	void initMember(RoomWindow* parent, int width, int height);
	CVideoRenderWnd* getViewByLiveId(const QString& id);
	CVideoRenderWnd* getViewByRecordId(const QString& id);
	CVideoRenderWnd* getViewByUserId(const UserId& id);
	bool hasRecord(const DeviceId& videoId);
	bool hasLive(const DeviceId& videoId);
	RecordVideoType getH264Stream(const QString& videoId);
protected slots:
	void videoOptionChanged();
	bool recordStart(const QString& videoId);
	bool recordStop(const QString& recordId);
	bool liveStart(const QString& videoId);
	bool liveStop(const QString& liveId);
#ifdef  __BUILD_USE_FAKECAPTURER__
public:
	virtual bool OnStart();
	virtual void OnStop();
protected:
	void timerEvent(QTimerEvent *event);
	bool isMainThread() const;
signals:
	bool onstart_s();
	void onstop_s();
#endif//__BUILD_USE_FAKECAPTURER__
private:
	typedef std::map<QString, CVideoRenderWnd*> ID2ViewType;
	IMVideo*				m_video;
	ICameraDevice*			m_videoDevice;
	IAVDRecord*				m_record;
	IAVDLive*				m_live;
	RoomWindow*				_room_wnd;
	ID2ViewType				_video_id2view;
	DeviceId				_localVideoId;
	CVideoLayoutFormatBase*	_render_layout_format;
	int						_black_boad_width;
	int						_black_boad_heigth;
	LiveInfo				_currLiveInfo;
	RecordInfo				_currRecordInfo;
	IMLivecast*				m_livecast;
#ifdef  __BUILD_USE_FAKECAPTURER__
	FakeVideoCapturer* _fakecapturer;
	int _timer_fakecapturer;
	int64 _time_stamp_ns;
	uint8* _buf_frame;
	int _buf_size;
	Qt::HANDLE _ui_main_thread;
#endif//__BUILD_USE_FAKECAPTURER__
};

#endif //__OPEN_AVD_N2M_VideoPage_H__
