#include "VideoRenderWnd.h"
#include "VideoShowWnd.h"
#include "MainToolWnd.h"
#include "AutoLabel.h"
#include <QtGui/qpainter.h>
#include <QtGui/qbitmap.h>

static bool g_isRegistRenderWndClasss = false;
const LPCTSTR VideoRenderWndClass = TEXT("VideoRenderClass");

///
CVideoRenderWnd::CVideoRenderWnd(const QString id, const QString user_name, QWidget* parent)
: QWidget(parent), _id(id), _parent(parent), _user_name(user_name), _hasBorder(true), _isCenter(false) {
	_video_show_wnd = new CVideoShowWnd(id, user_name, this);
	_video_show_wnd->SetSizer(1, 1, 1, 1, -1, -1);
	_video_show_wnd->show();

	_name_lbl = new QLabel(user_name, this);
	_name_lbl->setGeometry(2, 0, 150, 32);
	_name_lbl->adjustSize();
	_name_lbl->show();
	_name_lbl->setStyleSheet("background:transparent; color:white;");
	_record_btn = new CButton2M2(this, -1, -1, ":/nice2meet/Resources/image/record_close.png", ":/nice2meet/Resources/image/record_open.png");
	_record_btn->SetSizer(-1, 0, 2, -1, 32, 32);
	_record_btn->hide();
	_record_btn->setToolTip(QString("¼"));
	_record_btn->setStyleSheet("background:transparent;");
	_live_btn = new CButton2M2(this, -1, -1, ":/nice2meet/Resources/image/live_close.png", ":/nice2meet/Resources/image/live_open.png");
	_live_btn->SetSizer(-1, 0, 4 + 32, -1, 32, 32);
	_live_btn->hide();
	_record_btn->setToolTip(QString("ֱ"));
	_live_btn->setStyleSheet("background:transparent;");

	this->setGeometry(0, 0, 800, 600);
	this->show();
	connect(_record_btn, SIGNAL(state1Click()), this, SLOT(recordStart()));
	connect(_record_btn, SIGNAL(state2Click()), this, SLOT(recordStop()));
	connect(_live_btn, SIGNAL(state1Click()), this, SLOT(liveStart()));
	connect(_live_btn, SIGNAL(state2Click()), this, SLOT(liveStop()));
	connect(_video_show_wnd, SIGNAL(doubleClick()), this, SLOT(doubleClickSlot()));
	connect(_video_show_wnd, SIGNAL(click()), this, SLOT(clickSlot()));
}
CVideoRenderWnd::~CVideoRenderWnd() {
	delete _record_btn;
	delete _live_btn;
	delete _name_lbl;
	delete _video_show_wnd;
}
void CVideoRenderWnd::enableRecord(bool enable) {
	if (enable) {
		_record_btn->show();
	}
	else {
		_record_btn->hide();
	}
}
void CVideoRenderWnd::enableLive(bool enable) {
	if (enable) {
		_live_btn->show();
	}
	else {
		_live_btn->hide();
	}
}
void CVideoRenderWnd::paintEvent(QPaintEvent *event) {
	QPainter dc(this);
	dc.fillRect(0, 0, this->width(), this->height(), QColor(0, 0, 0));
	QFont font(dc.font());
	font.setPixelSize(15);
	font.setFamily(QString("Microsoft YaHei"));
	QPen pen(QColor(255, 0, 0));
	dc.setPen(pen);
	dc.setFont(font);
	dc.drawText(10, 0, rect().width() - 10, 30, Qt::AlignVCenter || Qt::AlignRight, _user_name);
	QRect borderRect(0, 0, rect().width() - 1, rect().height() - 1);
	if (_hasBorder) {
		dc.setPen(QColor(255, 255, 255));
		dc.drawRect(borderRect);
	}
}

void CVideoRenderWnd::setBorder(bool hasBorder) {
	_hasBorder = hasBorder;
}
void CVideoRenderWnd::setCenter(bool isCenter) {
	_isCenter = isCenter;
}

void CVideoRenderWnd::RegistVideo(IVideoRenderer* render) {
	_video_show_wnd->RegistVideo(render);
}
IVideoRender* CVideoRenderWnd::GetRender() {
	return _video_show_wnd->GetRender();
}
bool CVideoRenderWnd::recordStart() {
	if (emit recordStart(_id)) {
		setRecordState(CButton2M2::State2);
	}
	return true;
}
bool CVideoRenderWnd::recordStop() {
	if (emit recordStop(_recordid)) {
		setRecordState(CButton2M2::State1);
	}
	return true;
}
bool CVideoRenderWnd::liveStart() {
	if (emit liveStart(_id)) {
		setLiveState(CButton2M2::State2);
	}
	return true;
}
bool CVideoRenderWnd::liveStop() {
	if (emit liveStop(_liveid)) {
		setLiveState(CButton2M2::State1);
	}
	return true;
}
void CVideoRenderWnd::setRecordId(const QString& id) {
	_recordid = id;
	setRecordState(_recordid.isEmpty() ? CButton2M2::State1 : CButton2M2::State2);
}
void CVideoRenderWnd::setLiveId(const QString& id) {
	_liveid = id;
	setLiveState(_liveid.isEmpty() ? CButton2M2::State1 : CButton2M2::State2);
}
void CVideoRenderWnd::setLiveState(CButton2M2::STATE state) {
	_live_btn->SetState(state);
	QString tooltip = (CButton2M2::State1 == state) ? QSTRING("ֱ") : QSTRING("ֱ");
	_live_btn->setToolTip(tooltip);
}
void CVideoRenderWnd::setRecordState(CButton2M2::STATE state) {
	_record_btn->SetState(state);
	QString tooltip = (CButton2M2::State1 == state) ? QSTRING("¼") : QSTRING("¼");
	_record_btn->setToolTip(tooltip);
}
void CVideoRenderWnd::doubleClickSlot() {
	if (_isCenter) {
		if (isFullScreen()) {
			setParent(_parent);
			showNormal();
			OnSizer();
		}
		else {
			QDesktopWidget*  desktop = QApplication::desktop();
			int curNum = desktop->screenNumber(this);
			QWidget* dd = desktop->screen(curNum);
			QRect rc = desktop->screenGeometry(curNum);
			setParent(dd);
			setGeometry(rc);
			move(rc.x(), rc.y());
			showFullScreen();
		}
	}
	emit doubleClick();
}
void CVideoRenderWnd::resizeEvent(QResizeEvent *event) {
	_video_show_wnd->OnSizer();
	_record_btn->OnSizer();
	_live_btn->OnSizer();
}
void CVideoRenderWnd::mousePressEvent(QMouseEvent *event) {
	event->accept();
	activateWindow();
}
void CVideoRenderWnd::mouseReleaseEvent(QMouseEvent *event) {
	if (event->button() == Qt::LeftButton) {
		if (rect().contains(event->pos())) {
			emit click();
		}
	}
	event->accept();
}
void CVideoRenderWnd::clickSlot() {
	emit click();
}
void CVideoRenderWnd::changeEvent(QEvent *event) {
	if (event->type() == QEvent::WindowActivate) {
		if (isFullScreen()) {
			setParent(_parent);
			showNormal();
			OnSizer();
		}
	}
	switch (event->type()) {
	case QEvent::WindowActivate:
		break;
	case QEvent::WindowDeactivate:
		break;
	case QEvent::ApplicationActivate:
		break;
	case QEvent::ApplicationDeactivate:
		break;
	default:
		break;
	}
	QWidget::changeEvent(event);
}

bool CVideoRenderWnd::SetScalingType(IVideoRenderView::ScalingType type) {
	if (_video_show_wnd) {
		return _video_show_wnd->SetScalingType(type);
	}
	return false;
}
