#include "roomprofile.h"
#include <QtWidgets\qapplication>
#include <QtCore\qsettings>
#include <QtCore\quuid>
#include <QtCore\qfile>
#include "UIControl/UIDefine.h"
#include "api/avdengine.h"
#include "api/videorender.h"

QString accessKey = "F89EB5C71E494850A061CC0C5F42C177";
QString secretKey = "DDDF7445961C4D27A7DCE106001BBB4F";
//QString accessKey = "demo_access";
//QString secretKey = "demo_secret";
///
const QString& CRoomProfile::getAppKey() {
	return accessKey;
}
const QString& CRoomProfile::getSecretKey() {
	return secretKey;
}

CRoomProfile* g_instance = NULL;
CRoomProfile* CRoomProfile::Instance() {
	if (NULL == g_instance) {
		g_instance = new CRoomProfile();
		g_instance->loadProfileFile();
	}
	return g_instance;
}
CRoomProfile::CRoomProfile() {
	m_video_pubtype = vp_single;
	m_datatcp = false;
	m_auto_audio = false;
	m_auto_video = false;
	m_has_livecast = false;
	m_render_scaling_type = tee3::avd::IVideoRenderView::Scale_Aspect_Fit;
	m_ismainbroadcaster = false;
	m_lvvideolayout = vl_auto_overlay;
	m_usedGDIRender = false;
}
CRoomProfile::~CRoomProfile() {
}

bool CRoomProfile::loadProfileFile() {
	IAVDEngine*	avdengine = IAVDEngine::Instance();

	// log setting
	QString ini_path = QString("%1/setting.ini").arg(QCoreApplication::applicationDirPath());
	QSettings setting(ini_path, QSettings::IniFormat);
	QString loglevel = setting.value("setting/loglevel").toString().toUtf8().constData();
	QString logfile  = setting.value("setting/logfile").toString().toUtf8().constData();
	if (logfile.isEmpty()) {
		QString currentDir = QApplication::applicationDirPath();
		int pos = currentDir.lastIndexOf(QString("\\"));
		if (pos != currentDir.length() - 1) {
			currentDir.append(QString("\\"));
		}
		currentDir.append(QSTRING("Nice2Meet_Log_") + QString::number(QApplication::applicationPid(), 10));
		logfile = currentDir.toUtf8().data();
	}
	avdengine->setLogParams(loglevel.toUtf8().data(), logfile.toUtf8().data());
	
	// room setting
	m_siteurl	= setting.value("setting/init_site").toString();
	m_rooturl   = setting.value("setting/rooturl").toString();
	m_inviteurl = setting.value("setting/url").toString();
	m_dtlsflag	= setting.value("setting/mediausedtls").toInt();
	m_roommode	= setting.value("setting/default_mode").toString();
	m_roomid	= setting.value("setting/roomid").toString();
	m_userid	= setting.value("setting/userid").toString();
	if (m_userid.isEmpty()) {
		m_userid = QUuid::createUuid().toString();
		m_userid.remove('{');
		m_userid.remove('}');
	}

	// room video setting
	CameraCapability cap;
	cap.width	= setting.value("setting/videoWidth").toInt();
	cap.height	= setting.value("setting/videoHeight").toInt();
	cap.maxFPS	= setting.value("setting/videoFrameRate").toInt();
	if (cap.isValid()) {
		avdengine->setOption(eo_camera_capability_default, &cap);
	}
	avdengine->getOption(eo_camera_capability_default, &m_maxcap);

	VideoQuality quality;
	quality = (VideoQuality)setting.value("setting/subscribeQuality").toInt();
	if (quality > 0) {
		avdengine->setOption(eo_video_quality_subscribe_default, &quality);
	}

	quality = (VideoQuality)setting.value("setting/publishedQuality").toInt();
	if (quality > 0) {
		avdengine->setOption(eo_video_quality_publish_default, &quality);
	}

	m_video_ratio = (VideoRatio)setting.value("setting/videoRation").toInt();
	avdengine->setOption(eo_video_ration_default, &m_video_ratio);
	m_video_codec = (VideoCodec)setting.value("setting/videoCodec").toInt();
	avdengine->setOption(eo_video_codec_priority, &m_video_codec);
	m_audio_codec = String(setting.value("setting/audioCodec").toString().toUtf8());

	m_video_pubtype = (VideoPubType)setting.value("setting/publishOption").toInt();
	if (vp_3_svc == m_video_pubtype) {
		bool svc = true;
		avdengine->setOption(eo_video_publish_svc_default, &svc);
	}

	m_render_scaling_type = (tee3::avd::IVideoRenderView::ScalingType)setting.value("setting/renderScalingType").toInt();

	m_rtmpurl = setting.value("setting/rtmp_site").toString();

	QString vrender = setting.value("setting/videorender").toString();
	if (vrender.toLower() == QString("gdi")) {
		m_usedGDIRender = true;
	}
	return true;
}

const QString& CRoomProfile::GetInviteUrl() {
	if (m_inviteurl.isEmpty()) {
		m_inviteurl = m_rooturl + QSTRING("jroom");
	}
	return m_inviteurl;
}
void CRoomProfile::writeSetting(const QString &key, const QVariant &value) {
	QString ini_path = QString("%1/setting.ini").arg(QCoreApplication::applicationDirPath());
	QSettings setting(ini_path, QSettings::IniFormat);
	setting.setValue(key, value);
}

void CRoomProfile::SetRoomID(const QString& value) {
	if (value == m_roomid) {
		return;
	}
	m_roomid = value;
}
void CRoomProfile::SetUserID(const QString& value) {
	if (value == m_userid) {
		return;
	}
	m_userid = value;
}

bool CRoomProfile::SetVideoRatio(VideoRatio value) {
	if (value == m_video_ratio) {
		return false;
	}
	m_video_ratio = value;
	IAVDEngine::Instance()->setOption(eo_video_ration_default, &m_video_ratio);
	writeSetting("setting/videoRation", m_video_ratio);
	return true;
}
bool CRoomProfile::SetVideoCodec(VideoCodec value) {
	if (value == m_video_codec) {
		return false;
	}
	m_video_codec = value;
	IAVDEngine::Instance()->setOption(eo_video_codec_priority, &m_video_codec);
	writeSetting("setting/videoCodec", m_video_codec);
	return true;
}
bool CRoomProfile::SetVideoPubType(VideoPubType value) {
	if (value == m_video_pubtype) {
		return false;
	}
	m_video_pubtype = value;
	bool issvc = (vp_3_svc == m_video_pubtype);
	IAVDEngine::Instance()->setOption(eo_video_publish_svc_default, &issvc);
	writeSetting("setting/publishOption", m_video_pubtype);
	return true;
}
bool CRoomProfile::SetDtls(bool value) {
	if (value == m_dtlsflag) {
		return false;
	}
	m_dtlsflag = value;
	writeSetting("setting/mediausedtls", (m_dtlsflag ? 1 : 0));
	return true;
}
bool CRoomProfile::SetDataTcp(bool value) {
	if (value == m_datatcp) {
		return false;
	}
	m_datatcp = value;
	IAVDEngine::Instance()->setOption(eo_data_channel_tcp_priority, &m_datatcp);
	return true;
}
bool CRoomProfile::SetAutoVideo(bool value) {
	if (value == m_auto_video) {
		return false;
	}
	m_auto_video = value;
	return true;
}
bool CRoomProfile::SetAutoAudio(bool value) {
	if (value == m_auto_audio) {
		return false;
	}
	m_auto_audio = value;
	return true;
}
bool CRoomProfile::SetLivecast(bool value) {
	if (value == m_has_livecast) {
		return false;
	}
	m_has_livecast = value;
	return true;
}

void CRoomProfile::SetMainBroadcaster(bool value) {
	m_ismainbroadcaster = value;
}
void CRoomProfile::SetLivecastVideoLayout(MixerVideoLayoutType value) {
	m_lvvideolayout = value;
}

bool CRoomProfile::SetRenderType(tee3::avd::IVideoRenderView::ScalingType renderType) {
	if (renderType == m_render_scaling_type) {
		return false;
	}
	m_render_scaling_type = renderType;
	writeSetting("setting/renderScalingType", m_video_codec);
	return true;
}
///
QString CRoomProfile::GetLogo() const {
	QString logoImage("./logo.png");
	QFile mFile(logoImage);
	if (!mFile.exists()) {
		logoImage = QString(":/nice2meet/Resources/image/logo.png");
	}
	return logoImage;
}
QImage CRoomProfile::GetTitleBarLogo() const {
	QImage logoImage("./logoinside.png");
	if (logoImage.isNull()) {
		logoImage = QImage(":/nice2meet/Resources/image/logoinside.png");
	}
	return logoImage;
}
QIcon CRoomProfile::GetAppIcon() const {
	QFile mFile("./app_icon.ico");
	if (mFile.exists()) {
		return QIcon("./app_icon.ico");
	}
	else {
		return QIcon(":/nice2meet/Resources/image/app_icon.ico");
	}
}

